/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/fscapi.h"
#include "ClientConfig.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/DeviceManagementNode.h"
#include "MacDefaultConfigFactory.h"
#include "spds/DataTransformerFactory.h"
#include "spds/DESDecoder.h"
#include "spds/DESEncoder.h"
#include "base/adapter/PlatformAdapter.h"
#include "spdm/constants.h"

#include "customization.h"
#include "base/util/StringBuffer.h"

#include "spdm/DMTreeFactory.h"
#include "spdm/DMTree.h"
#include "spdm/ManagementNode.h"
#include "utils.h"
#include "SyncQueueManager.h"

#include "UpgradeActions.h"
#include "CustomUpgradeActions.h"
#include "updater/UpdaterConfig.h"
#include "http/URL.h"

USE_NAMESPACE

ClientConfig *ClientConfig::instance = NULL;

/**
 * Default constructor
 */
ClientConfig::ClientConfig():DMTClientConfig(){
    isInitialized = false;
}

/**
 * Singleton implementation: get the unique instance of the config.
 *
 * @return  the singleton object
 */
ClientConfig *ClientConfig::getInstance() {
    if(!instance) {
        StringBuffer appcontext(APPNAME);
        appcontext += APPCONTEXT;
        PlatformAdapter::init(appcontext);        
        instance = new ClientConfig();
    }
    return instance;
}

/**
 * Singleton implementation: release the unique instance of the config.
 */
void ClientConfig::dispose() {
    delete instance;
    instance = NULL;
}

StringBuffer ClientConfig::getNewReleaseStringFormat(PropertyFile& pf, bool isFunambol) {
    
    StringBuffer releaseMajor;
    StringBuffer releaseMinor;  
    StringBuffer releaseBuildNumber; 
    
    if (isFunambol) {
        releaseMajor        = pf.readPropertyValue("funambolsw.major");
        releaseMinor        = pf.readPropertyValue("funambolsw.minor");
        releaseBuildNumber  = pf.readPropertyValue("funambolsw.buildnumber");
    } else {
        releaseMajor        = pf.readPropertyValue("release.major");
        releaseMinor        = pf.readPropertyValue("release.minor");
        releaseBuildNumber  = pf.readPropertyValue("build.number");
    }    
    
    StringBuffer res;
    res.append(releaseMajor);
    res.append(".");
    res.append(releaseMinor);
    res.append(".");
    res.append(releaseBuildNumber);
    
    return res;
    
}

unsigned long ClientConfig::getNewReleaseNumber(PropertyFile& pf, bool isFunambol) {
    /*
    StringBuffer releaseMajor;
    StringBuffer releaseMinor;  
    StringBuffer releaseBuildNumber; 
    
    if (isFunambol) {
        releaseMajor        = pf.readPropertyValue("funambolsw.major");
        releaseMinor        = pf.readPropertyValue("funambolsw.minor");
        releaseBuildNumber  = pf.readPropertyValue("funambolsw.buildnumber");
    } else {
        releaseMajor        = pf.readPropertyValue("release.major");
        releaseMinor        = pf.readPropertyValue("release.minor");
        releaseBuildNumber  = pf.readPropertyValue("build.number");
    }    
    
    int maj = 0, min = 0, build = 0;
    sscanf(releaseMajor.c_str(), "%d", &maj);
    sscanf(releaseMinor.c_str(), "%d", &min);
    sscanf(releaseBuildNumber.c_str(), "%d", &build);
    
    unsigned long releaseNumber = (maj*10000 + min*100 + build);
    return releaseNumber;
    */
    StringBuffer res = getNewReleaseStringFormat(pf, isFunambol);
    return getCurrentReleaseNumber(res);
    
}


unsigned long ClientConfig::getCurrentReleaseNumber(const char* value) {
    
    if (value == NULL) {
        return 0;
    }
    int maj = 0, min = 0, build = 0;
    sscanf(value, "%d.%d.%d", &maj, &min, &build);
    unsigned long releaseNumber = (maj*10000 + min*100 + build);
    return releaseNumber;
}

void ClientConfig::init() 
{    
    isInitialized = true;
    const char* DIST_PROPERTIES = "/distribution.properties";
    // read the distribution property file
    StringBuffer pfPath(CLIENT_SYSTEM_APPLICATION_FOLDER);
    pfPath.append(DIST_PROPERTIES);
 
    PropertyFile *pf = NULL;
    if (fileExists(pfPath.c_str())) {
        pf = new PropertyFile(pfPath);
    }
    
    bool toSave = false;
    long funambolNewRelease = 0, 
         customerNewRelease = 0, 
         funambolCurrentRelease = 0,
         customerCurrentRelease = 0;    
    
    // Read the configuration
    if (!read() || !getDeviceConfig().getSwv() 
                || getDeviceConfig().getSwv() == "0"
                || getDeviceConfig().getSwv() == "") {
        // Config not found, generate a default one and save it
        createConfig();
        toSave = true;
    } else {
        funambolCurrentRelease = getCurrentReleaseNumber(getFunambolSwv());
        customerCurrentRelease = getCurrentReleaseNumber(getDeviceConfig().getSwv());    
        
        if (pf) {
        
            funambolNewRelease = getNewReleaseNumber(*pf, true);
            customerNewRelease = getNewReleaseNumber(*pf, false);
            
            //
            // do actions due to the update of the funambol product
            //
            if (funambolCurrentRelease < funambolNewRelease) {
                // do something
                
                UpgradeActions::upgradeConfig(getInstance(), funambolCurrentRelease);
                //
                // reset the udpater if needed
                // 
                StringBuffer appuri(PlatformAdapter::getAppContext());
                UpdaterConfig* upconfig = new UpdaterConfig(appuri.c_str());
                upconfig->createDefaultConfig();
                upconfig->save();
                
            }
            
            //
            // do specific actions due to the customer changes
            //
            if (customerCurrentRelease < customerNewRelease) {
                // do something
                
                CustomUpgradeActions::upgradeConfig(getInstance(), customerCurrentRelease);
            }
                        
            toSave = true;
        }
    }
    
    if (toSave) {
        if (pf) {
            // set the software version parameters
            this->getDeviceConfig().setSwv(getNewReleaseStringFormat(*pf, false));
            this->setFunambolSwv(getNewReleaseStringFormat(*pf, true));
        } else {
            if (funambolCurrentRelease == 0) {
                this->getDeviceConfig().setSwv("0.0.0");
                this->setFunambolSwv("0.0.0");
            } else {
                // othewise we leave as is
            }
        }

        // compone the user agent
        StringBuffer ua(MAC_USER_AGENT);
        ua.append(" ");
        ua.append(getDeviceConfig().getSwv());
        getAccessConfig().setUserAgent(ua.c_str());
        save();
    } 
    
    if (pf) {
        delete pf;
    }

    setupLog();
    
    // Handle backward compatibility: if the stored version is
    // different from the current one, take the proper action
    // if (strcmp(this->getDeviceConfig().getSwv(), VERSION) != 0) {
    // Handle differences in config. For instance, if a new parameter has
    // been added, initialize it so that it will be stored at the end.
    // TODO: just an example now, remember this at next version.
}

bool ClientConfig::read() {
    
    if (!DMTClientConfig::read()) {
        return false; // error in the common config read.
    }
    
    if (!open()) {
        return false;
    }
 
    if (CUSTOM_ENCODE_PASSWORD) {
        char* password = strdup(getAccessConfig().getPassword());
        DataTransformer* dec = DataTransformerFactory::getDecoder("b64");
    
        TransformationInfo info;
        info.size = (long)strlen(password)*sizeof(char);
    
        dec->transform(password, info);
        password[info.size] = 0;
    
        getAccessConfig().setPassword(password);
        delete password;
    }
    
    //
    // read also extra stuff:
    // - scheduling interval
    // - funambol software version
    //
    for(int i = 0; i < sourceConfigsCount; i++) {
        SyncSourceConfig* ssc = getSyncSourceConfig(i);
        const char* val = readSyncSourceConfigProperty(ssc->getName(), "schedulingInterval");
        scheduledIntervalValues.put(ssc->getName(), val);
        delete [] val;
    }
    
    const char* val = readClientConfigProperty("funambol_swv");
    setFunambolSwv(val);
    delete [] val; val = NULL;
    
    val = readClientConfigProperty("briefCaseFolder");
    setBriefCaseFolder(val);
    delete [] val; val = NULL;
    
    return true;
}

bool ClientConfig::safeSave()
{    
    if (SyncQueueManager::isSyncRunning()) {
        return false;
    }
    
    savePreferences();
    return true;
}

void ClientConfig::savePreferences(){
    //we save only the value modified inside the preferences
    for(int i = 0; i < sourceConfigsCount; i++) {
        SyncSourceConfig* ssc = getSyncSourceConfig(i);
        setSyncSourceConfigProperty(ssc->getName(), PROPERTY_SOURCE_SYNC, ssc->getSync());
        StringBuffer schedVal = scheduledIntervalValues.get(ssc->getName());
        if (!(schedVal.empty())){
            setSyncSourceConfigProperty(ssc->getName(), "schedulingInterval" , schedVal);
        }
    }
    
    char* password = strdup(getAccessConfig().getPassword());
    if (CUSTOM_ENCODE_PASSWORD){
        DataTransformer* enc = DataTransformerFactory::getEncoder("b64");
        TransformationInfo info;
        info.size = (long)strlen(password)*sizeof(char);
        StringBuffer encodedPassword = enc->transform(password, info);
        getAccessConfig().setPassword(encodedPassword.c_str());
    }
    saveAccountSettings();
    saveLogSettings();
    if(CUSTOM_ENCODE_PASSWORD){
        getAccessConfig().setPassword(password);
    }
    delete password;
}

/**
 * Overload the save() method, adding the code to store client-specific values.
 *
 * @return  true for success
 */
bool ClientConfig::save() 
{
    char* password = strdup(getAccessConfig().getPassword());
    
	if (CUSTOM_ENCODE_PASSWORD) {
        DataTransformer* enc = DataTransformerFactory::getEncoder("b64");
        TransformationInfo info;
        info.size = (long)strlen(password)*sizeof(char);
        StringBuffer encodedPassword = enc->transform(password, info);
        getAccessConfig().setPassword(encodedPassword.c_str());
    }
    
    if ( !DMTClientConfig::save() ) {
		delete password;
		return false; // error in the common config save.
    }

    if (CUSTOM_ENCODE_PASSWORD) {
        getAccessConfig().setPassword(password);
    }
	
    delete password;
    
    //
    // save also extra stuff:
    // - scheduling interval
    // - funambol software version
    //
    for (int i = 0; i < sourceConfigsCount; i++) {
        SyncSourceConfig* ssc = getSyncSourceConfig(i);
        StringBuffer schedVal = scheduledIntervalValues.get(ssc->getName());
        if (!(schedVal.empty())){
            setSyncSourceConfigProperty(ssc->getName(), "schedulingInterval" , schedVal);
        }
    }
    
    setClientConfigProperty("funambol_swv", funambolSwv.c_str());
    setClientConfigProperty("briefCaseFolder", briefCaseFolder.c_str());

    return true;
}

SyncSourceConfig* ClientConfig::getSyncSourceConfig(const char* name){
    
    SyncSourceConfig* ssc = DMTClientConfig::getSyncSourceConfig(name);
    if(strcmp(name, "addressbook") == 0){
        ArrayList* p = MacDefaultConfigFactory::getVCardProperties();
        ssc->addCtCap(p, "text/x-vcard", "2.1");
        delete p;
    }
    return ssc;
}

SyncSourceConfig* ClientConfig::getSyncSourceConfig(int index)
{    
    return DMTClientConfig::getSyncSourceConfig(index);
}

bool ClientConfig::areCredentialSaved(){
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s%s%s", PlatformAdapter::getAppContext().c_str(),
                     CONTEXT_SPDS_SYNCML,
                     CONTEXT_AUTH);
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    const char* username = node->readPropertyValue(PROPERTY_USERNAME);
    const char* password = node->readPropertyValue(PROPERTY_PASSWORD);
    delete node;
    delete dmt;   
    
    bool ret = true;
    
    if (username == NULL                ||
        password == NULL                ||
        strcmp(username, "") == 0       ||
        strcmp(password, "") == 0        ) {
        ret = false;
    }
    delete [] username;
    delete [] password;
    return ret;
}

bool ClientConfig::areCredentialValid() {
    
    bool ret = true;
    
    if (/*readLastSyncStatusCode() == 401 ||*/
        !(areCredentialSaved())      ) {
        
        ret = false;
    }
    
    return ret;
}
        
/**
 * Method to create a default config.
 */
void ClientConfig::createConfig() {
	AccessConfig*	  ac = NULL;
	DeviceConfig*	  dc = NULL;
    SyncSourceConfig* sc = NULL;

	ac = MacDefaultConfigFactory::getAccessConfig();
    this->setAccessConfig(*ac);
	delete ac;
	
	dc = MacDefaultConfigFactory::getDeviceConfig();
    this->setDeviceConfig(*dc);
	delete dc;
	
    sc = MacDefaultConfigFactory::getSyncSourceConfig("addressbook");
    this->setSyncSourceConfig(*sc); 
    this->setSchedulingInterval("addressbook", "0");
    delete sc;

    sc = MacDefaultConfigFactory::getSyncSourceConfig("calendar");
    this->setSyncSourceConfig(*sc);
    delete sc;
 
    sc = MacDefaultConfigFactory::getSyncSourceConfig("task");
    this->setSyncSourceConfig(*sc);
    delete sc;

    sc = MacDefaultConfigFactory::getSyncSourceConfig("note");  
    this->setSyncSourceConfig(*sc);
    delete sc;

    sc =  MacDefaultConfigFactory::getSyncSourceConfig("config");
    this->setSyncSourceConfig(*sc);
    delete sc; 

    sc = MacDefaultConfigFactory::getSyncSourceConfig("briefcase");
    this->setSyncSourceConfig(*sc);
    delete sc;
    
	// by default enable menubar UI
    this->saveMenubarUIEnabled(true);
	// set default folder for briefcase
	this->setBriefCaseFolder(BRIEFCASE_DEFAULT_PATH);
}

const char* ClientConfig::readSyncSourceConfigProperty(const char* syncSourceName, const char* propertyName){
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s/%s/%s", PlatformAdapter::getAppContext().c_str(),"spds/sources", syncSourceName );
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    const char* ret = node->readPropertyValue(propertyName);
    delete node;
    delete dmt;
    return ret;
}


void ClientConfig::setSyncSourceConfigProperty(const char* syncSourceName, 
                                               const char* propertyName, 
                                               const char* propertyValue) {
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s/%s/%s", PlatformAdapter::getAppContext().c_str(),"spds/sources", syncSourceName );
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    node->setPropertyValue(propertyName, propertyValue);
    delete node;
    delete dmt;
}

const char* ClientConfig::readClientConfigProperty(const char* propertyName){
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s", PlatformAdapter::getAppContext().c_str());
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    const char* ret = node->readPropertyValue(propertyName);
    delete node;
    delete dmt;
    return ret;
}

void ClientConfig::setClientConfigProperty(const char* propertyName, const char* propertyValue){
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s", PlatformAdapter::getAppContext().c_str());
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    node->setPropertyValue(propertyName, propertyValue);
    delete node;
    delete dmt;
}

void ClientConfig::saveAccountSettings() {
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s%s%s", PlatformAdapter::getAppContext().c_str(),
                                 CONTEXT_SPDS_SYNCML,
                                 CONTEXT_AUTH);
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    node->setPropertyValue(PROPERTY_USERNAME, this->getAccessConfig().getUsername());
    node->setPropertyValue(PROPERTY_PASSWORD, this->getAccessConfig().getPassword());
    delete node;
    
    nodeName = "";
    nodeName.sprintf("%s%s%s", PlatformAdapter::getAppContext().c_str(),
                     CONTEXT_SPDS_SYNCML,
                     CONTEXT_CONN);
    node = dmt->readManagementNode(nodeName);
    StringBuffer url = node->readPropertyValue(PROPERTY_SYNC_URL);
    if (strcmp(url, this->getAccessConfig().getSyncURL()) != 0) { 
        node->setPropertyValue(PROPERTY_SYNC_URL, this->getAccessConfig().getSyncURL());
        
        UpdaterConfig* upconfig = new UpdaterConfig(PlatformAdapter::getAppContext());
        upconfig->createDefaultConfig();
        URL url;
        url.setURL(ClientConfig::getInstance()->getSyncURL());
        StringBuffer port(":80");
        if (url.port != 0) { 
            port = ":"; 
            port += itow(url.port); 
        }
        
        // Compose the url
        StringBuffer updateUrl(url.protocol);
        updateUrl += "://"; 
        updateUrl += url.host;
        updateUrl += port;
        updateUrl += UPDATE_SOFTWARE_URL_RESOURCE;
        
        upconfig->setUrlCheck(updateUrl);
        upconfig->setRecommended("1");
        upconfig->save();
    }
   
    delete node;
    delete dmt;   
}

void ClientConfig::saveLogSettings() {
    StringBuffer nodeName;
    DMTree* dmt = DMTreeFactory::getDMTree(rootContext);
    nodeName.sprintf("%s%s%s", PlatformAdapter::getAppContext().c_str(),
                     CONTEXT_SPDS_SYNCML,
                     CONTEXT_EXT);
    ManagementNode* node;
    node = dmt->readManagementNode(nodeName);
    StringBuffer s(""); s.append(this->getDeviceConfig().getLogLevel());
    node->setPropertyValue(PROPERTY_LOG_LEVEL, s.c_str());
    delete node;
    delete dmt;   
}

int ClientConfig::setupLog()
{
    struct stat st;
    int ret = 0;
    // initialize the PlatformAdapter and create a default config if needed
    StringBuffer logsFolder = PlatformAdapter::getConfigFolder();
    logsFolder += "/logs/";
    
    memset(&st, 0, sizeof(struct stat));
    
    if (stat(logsFolder.c_str(), &st)) {
        if (errno == ENOENT) {
            if (mkdir(logsFolder, S_IRWXU)) {
                fprintf(stderr, "error creating sync client logs folder \"%s\": %s\n",
                    logsFolder.c_str(), strerror(errno));
                
                ret = errno;
            } 
        } else {
            fprintf(stderr, "error accessing sync client logs folder \"%s\": %s\n",
                logsFolder.c_str(), strerror(errno));
            
            ret = errno;
        }
    }

    // set up log and print banner
    if (ret == 0) {
        LOG.setLogPath(logsFolder);
        LOG.setLogName(LOG_FILE_NAME);
        LOG.setLevel(getDeviceConfig().getLogLevel());
    }
    
    return ret;
}

void ClientConfig::setBriefCaseFolder(const char *newPath) 
{
    if (briefCaseFolder != newPath) {
        briefCaseFolder = newPath;
        //Get the SyncSource config
        SyncSourceConfig* sc = this->getSyncSourceConfig("briefcase");
        
        if (sc != NULL) {
            //Update the last sync anchor
            sc->setLast(0);
        }
    }
}
