/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import "LaunchCtlConfigManager.h"
#import "LaunchCtlManager.h"

USE_NAMESPACE;

static const NSString* menuBarPlistFmt = 
@"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
"<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n"
"<plist version=\"1.0\">\n"
"<dict>\n"
"<key>Label</key>\n"
"<string>%s</string>\n"
"<key>ProgramArguments</key>\n"
"<array>\n"
"<string>%s</string>\n"
"</array>\n"
"<key>RunAtLoad</key>\n"
"<true/>\n"
"<key>KeepAlive</key>\n"
"<true/>\n"
"</dict>\n"
"</plist>\n";


static const NSString* scheduledSyncPlistFmt =
    @"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n \
    <!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n \
    <plist version=\"1.0\">\n \
    <dict>\n                  \
    <key>Label</key>\n        \
        <string>%s</string>\n \
    <key>ProgramArguments</key>\n         \
        <array>\n                         \
            <string>%s</string>\n         \
            <string>%s</string>\n         \
            <string>%s</string>\n         \
        </array>\n                        \
    <key>StartInterval</key>\n            \
        <integer>%d</integer>\n           \
    </dict>\n                             \
    </plist>\n";


static const NSString* updaterPlist =
    @"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n \
    <!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n \
    <plist version=\"1.0\">\n \
    <dict>\n                  \
    <key>Label</key>\n        \
        <string>%s</string>\n \
    <key>ProgramArguments</key>\n         \
        <array>\n                         \
            <string>%s</string>\n         \
        </array>\n                        \
    <key>StartInterval</key>\n            \
        <integer>%d</integer>\n           \
    </dict>\n                             \
    </plist>\n";

@implementation LaunchCtlConfigManager

/*
 * FIXME - a log policy should be determined for pref panel...
 */

- (id) init
{
    if (self = [super init]) {
       launchCtlManager = [[LaunchCtlManager alloc] init]; 
       fileMan = [[NSFileManager alloc] init];
       [self setup];
    }
    
    return (self);
}

- (void) dealloc
{
    [launchCtlManager dealloc];
    [fileMan dealloc];
    
    [menuBarPlistFile release];
    [scheduledSyncPlistFile release];
    
    [super dealloc];
}

-(BOOL) setNewScheduledSync: (int) scheduleInterval
{
    BOOL ret = YES;
    NSString *pListContents = nil;

    if (scheduleInterval <= 0) {
        return NO;
    }

    // unload current sync job for launchd
    if ([fileMan fileExistsAtPath: scheduledSyncPlistFile]) {
        // try unloading launchd job: file can exists but job 
        // could be unloaded so we don't check errors
        [launchCtlManager unloadLaunchAgent: scheduledSyncPlistFile];
    }

    pListContents = [NSString stringWithFormat: scheduledSyncPlistFmt, SYNC_APPLICATION_PLIST_LABEL, 
                        SYNC_APPLICATION_PATH, SYNC_CONTACTS_OPT, SYNC_WAIT_QUEUE_OPT, scheduleInterval];

    // create new plist file
    if ((ret = [self updateLaunchAgent: scheduledSyncPlistFile withContents: pListContents]) == NO) {
        //LOG.error("error creating new plist file for scheduled sync");

        return ret;
    }
   
    // load new lauchd job
    if ((ret = [launchCtlManager loadLaunchAgent: scheduledSyncPlistFile]) == NO) {
        //LOG.error("error setting new launchctl job for scheduled sync");

        return ret;
    }

    return ret;
}

-(BOOL) loadLaunchAgent: (NSString *)plistFile
{
    BOOL ret = YES;
    
    if ([plistFile length] == 0) {
        return NO;
    }
    
    // unload job for launchd
    ret = [launchCtlManager loadLaunchAgent: plistFile];
    
    return ret;
}



-(BOOL) unloadLaunchAgent: (NSString *)plistFile
{
    BOOL ret = YES;
    
    if ([plistFile length] == 0) {
        return NO;
    }
    
    // unload job for launchd
    ret = [launchCtlManager unloadLaunchAgent: plistFile];
        
    return ret;
}

-(BOOL) removeLaunchAgent:(NSString *)plistFile
{
    BOOL ret = YES;
    
    if ([plistFile length] == 0) {
        return NO;
    }
    
    // unload job for launchd
    if ((ret = [launchCtlManager unloadLaunchAgent: plistFile]) == NO) {
        return ret;
    }
    
    if ((ret = [self removeLaunchAgentPlist: plistFile]) == NO) {
        return ret;
    }
    
    return ret;
}

-(BOOL) loadScheduledSync
{
    BOOL ret = NO;
   
    if ([fileMan fileExistsAtPath: scheduledSyncPlistFile]) {
        ret = [self loadLaunchAgent: scheduledSyncPlistFile];
    }
 
    return ret;
}


-(BOOL) unloadScheduledSync
{
    BOOL ret = NO;
    
    if ([fileMan fileExistsAtPath: scheduledSyncPlistFile]) {
        ret = [self unloadLaunchAgent: scheduledSyncPlistFile];
    }
    
    return ret;
}


-(BOOL) removeScheduledSync
{
    return [self removeLaunchAgent: scheduledSyncPlistFile];
}


-(BOOL) setMenuBarAgent
{
    BOOL ret = YES;
    NSString *pListContents = nil;

    pListContents = [NSString stringWithFormat: menuBarPlistFmt, MENUBAR_APPLICATION_PLIST_LABEL, MENUBAR_APPLICATION_PATH];

    // create new plist file
    if ((ret = [self updateLaunchAgent: menuBarPlistFile withContents: pListContents]) == NO) {
        //LOG.error("error creating new plist file for menubar");

        return ret;
    }

    
    // load new job file
    if ((ret = [launchCtlManager loadLaunchAgent: menuBarPlistFile]) == NO) {
        //LOG.error("error setting new launchctl job for menubar");

        return ret;
    }

    return ret;
}

-(BOOL) setUpdaterAgent{
    BOOL ret = YES;
    NSString *pListContents = nil;
    pListContents = [NSString stringWithFormat: updaterPlist, UPDATER_APPLICATION_PLIST_LABEL, UPDATER_APPLICATION_PATH, 86400];
    
    // create new plist file
    if ((ret = [self updateLaunchAgent: updaterPlistFile withContents: pListContents]) == NO) {
        //LOG.error("error creating new plist file for menubar");
        NSLog(@"error creating plist file\n");
        return ret;
    }
    
    
    // load new job file
    if ((ret = [launchCtlManager loadLaunchAgent: updaterPlistFile]) == NO) {
        //LOG.error("error setting new launchctl job for menubar");
        
        return ret;
    }
    
    return ret;
}

- (BOOL) loadUpdater{
    BOOL ret = NO;
    
    if ([fileMan fileExistsAtPath: updaterPlistFile]) {
        ret = [self loadLaunchAgent: updaterPlistFile];
    } else {
        ret = [self setUpdaterAgent];
    }
    
    return ret;
}

- (BOOL) unloadUpdater{
    BOOL ret = NO;
    
    if ([fileMan fileExistsAtPath: updaterPlistFile]) {
        ret = [self unloadLaunchAgent: updaterPlistFile];
    }
    
    return ret;
}

- (BOOL) removeUpdater{
    return [self removeLaunchAgent: updaterPlistFile];
}

- (BOOL) loadMenubar
{
    BOOL ret = NO;
    
    if ([fileMan fileExistsAtPath: menuBarPlistFile]) {
        ret = [self loadLaunchAgent: menuBarPlistFile];
    }
    
    return ret;
}

- (BOOL) unloadMenubar
{
    BOOL ret = NO;
    
    if ([fileMan fileExistsAtPath: menuBarPlistFile]) {
        ret = [self unloadLaunchAgent: menuBarPlistFile];
    }
    
    return ret;
}

- (BOOL) removeMenubar
{
    return [self removeLaunchAgent: menuBarPlistFile];
}

-(BOOL) updateLaunchAgent: (NSString *)plistFile withContents: (NSString *)pListContents
{
    BOOL ret = YES;
    int dataLen = 0;
    NSData *fileData = nil;
    
    if ((dataLen = [pListContents length]) == 0) {
        return NO;
    }
    
    fileData = [NSData dataWithBytes:[pListContents UTF8String] length: dataLen];
    
    if ((ret = [fileMan createFileAtPath:plistFile contents:fileData attributes: nil]) == NO) {
        //LOG.error("error creating plist file");
    }   

    return ret;
}

-(BOOL) removeLaunchAgentPlist: (NSString *)pListFilePath
{
    BOOL ret = YES;
    
    if ([fileMan fileExistsAtPath: pListFilePath]) {
        ret = [fileMan removeItemAtPath: pListFilePath error: nil];
    }
    
    return ret;
}

-(BOOL) setup
{
    BOOL ret = YES;
    const char* homeDirPath = getenv("HOME");
    NSString *userLaunchAgentPath = nil;

    if (homeDirPath == NULL) {
        return NO;
    }

    userLaunchAgentPath = [NSString stringWithFormat: @"%s/%s", homeDirPath, LAUNCHD_PLIST_PATH];
    menuBarPlistFile = [NSString stringWithFormat: @"%@/%s", userLaunchAgentPath, MENUBAR_PLIST_FILE_NAME];         
    scheduledSyncPlistFile = [NSString stringWithFormat: @"%@/%s", userLaunchAgentPath, SYNC_PLIST_FILE_NAME];         
    updaterPlistFile = [NSString stringWithFormat: @"%@/%s", userLaunchAgentPath, UPDATER_PLIST_FILE_NAME];         

    [menuBarPlistFile retain];
    [scheduledSyncPlistFile retain];
    [updaterPlistFile retain];
    
    // create LaunchAgent dir in user home directory if doesn't exists yet
    if ([fileMan fileExistsAtPath: userLaunchAgentPath] == NO) {
        ret = [fileMan createDirectoryAtPath: userLaunchAgentPath withIntermediateDirectories: NO attributes: nil error: nil];
    }

    return ret;
}

@end
