/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/fscapi.h"
#include "base/debug.h"
#include "base/errors.h"
#include "base/Log.h"
#include "base/globalsdef.h"
#include "base/util/utils.h"

#include "MacDefaultConfigFactory.h"
#include <Foundation/Foundation.h>
#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>

#include "syncml/core/PropParam.h"
#include "syncml/core/Property.h"
#include "customization.h"

USE_NAMESPACE


MacDefaultConfigFactory::MacDefaultConfigFactory() {
}

MacDefaultConfigFactory::~MacDefaultConfigFactory() {
}


AccessConfig* MacDefaultConfigFactory::getAccessConfig() {
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    AccessConfig* ac = new AccessConfig();
    
    ac->setUsername             ("");
    ac->setPassword             ("");
    ac->setFirstTimeSyncMode    (SYNC_NONE);
    ac->setUseProxy             (false);
    ac->setProxyHost            ("");
    ac->setProxyPort            (8080);
    ac->setProxyUsername        ("");
    ac->setProxyPassword        ("");
    ac->setSyncURL              (DEFAULT_SYNC_URL);
    ac->setBeginSync            (0);
    ac->setEndSync              (0);
    ac->setServerAuthRequired   (false);
    ac->setClientAuthType       ("syncml:auth-basic");
    ac->setServerAuthType       ("syncml:auth-basic");
    ac->setServerPWD            ("funambol");
    ac->setServerID             ("funambol");
    ac->setServerNonce          ("");
    ac->setClientNonce          ("");
    ac->setMaxMsgSize           (256000);
    ac->setReadBufferSize       (0);
    ac->setUserAgent            ("");
    ac->setCheckConn            (true);
    ac->setResponseTimeout      (0);
    //ac->setEncryption           (false);
    [pool release];
    
    return ac;
}



DeviceConfig* MacDefaultConfigFactory::getDeviceConfig() {
    
    DeviceConfig* dc = new DeviceConfig();
    
    StringBuffer deviceId = "mox-";
    char base64      [64];
    char digest      [16];
    memset(digest,      0, 16);
    memset(base64,      0, 64);

    
    CFStringRef serialNumber = NULL;
    io_service_t    platformExpert = IOServiceGetMatchingService(kIOMasterPortDefault,
                                                                 IOServiceMatching("IOPlatformExpertDevice"));
    if (platformExpert) {
        CFTypeRef serialNumberAsCFString =
        IORegistryEntryCreateCFProperty(platformExpert,
                                        CFSTR(kIOPlatformSerialNumberKey),
                                        kCFAllocatorDefault, 0);
        if (serialNumberAsCFString) {
            serialNumber = (CFStringRef)serialNumberAsCFString;
        }
        
        IOObjectRelease(platformExpert);
    }
    
    StringBuffer sn (CFStringGetCStringPtr(serialNumber, kCFStringEncodingUTF8));
    CFRelease(serialNumber);
    
    CFStringRef usernamecfstring = NULL;
    //if ( CSCopyUserName != (void *) kUnresolvedCFragSymbolAddress) {
        
    if ( CSCopyUserName != NULL) {
        usernamecfstring = CSCopyUserName(false);
    }
    
    StringBuffer username(CFStringGetCStringPtr(usernamecfstring, kCFStringEncodingUTF8));
    CFRelease(usernamecfstring);
    
    
    StringBuffer val(sn);
    val += ":";
    val += username;
    
    calculateMD5(val.c_str(), val.length(), digest);
    b64_encode((char*)base64, digest, 16);
    
    deviceId.append(base64);
    
    dc->setMan                  ("Funambol Inc.");
    dc->setMod                  ("Mac");
    dc->setOem                  ("Open Source");
    dc->setFwv                  ("");
    dc->setSwv                  ("");
    dc->setHwv                  ("");
    dc->setDevID                (deviceId.c_str());
    dc->setDevType              ("Mac");
    dc->setDsV                  ("");
    dc->setUtc                  (true);
    dc->setLoSupport            (false);
    dc->setNocSupport           (true);
    dc->setLogLevel             (DEFAULT_LOG_LEVEL);
    dc->setMaxObjSize           (5000000);
    dc->setDevInfHash           ("");
    
    return dc;
}



SyncSourceConfig* MacDefaultConfigFactory::getSyncSourceConfig(const char* name) {
    
    SyncSourceConfig* sc = new SyncSourceConfig();
    
    sc->setName                 (name);
    sc->setSync                 ("two-way");
    sc->setSyncModes            ("slow,two-way,one-way-from-server,one-way-from-client");
    sc->setLast                 (0);
    sc->setEncryption           ("");
    
    if (!strcmp(name, "addressbook")){
        sc->setEncoding         ("bin");
        sc->setURI              ("card");
        sc->setSupportedTypes   ("text/x-vcard");
        sc->setType             ("text/x-vcard");
        sc->setVersion          ("2.1");
        sc->setIsEnabled        ("1");
    }
    else if (!strcmp(name, "calendar")){
        sc->setEncoding         ("bin");
        sc->setURI              ("event");
        sc->setSupportedTypes   ("text/x-vcalendar");
        sc->setType             ("text/x-vcalendar");
        sc->setVersion          ("1.0");
        sc->setIsEnabled        ("0");
    }
    else if (!strcmp(name, "task")){
        sc->setEncoding         ("bin");
        sc->setURI              ("task");
        sc->setSupportedTypes   ("text/x-vcalendar");
        sc->setType             ("text/x-vcalendar");
        sc->setVersion          ("1.0");
        sc->setIsEnabled        ("0");
    }
    else if (!strcmp(name, "note")){
        sc->setEncoding         ("b64");
        sc->setURI              ("note");
        sc->setSupportedTypes   ("text/x-vnote");
        sc->setType             ("text/x-vnote");
        sc->setVersion          ("1.1");
        sc->setIsEnabled        ("0");
    }
    else if (!strcmp(name, "config")){
        sc->setEncoding         ("b64");
        sc->setURI              ("configuration");
        sc->setSupportedTypes   ("text/plain");
        sc->setType             ("text/plain");
        sc->setIsEnabled        ("1");
    } 
    else if (!strcmp(name, "briefcase")) {
        sc->setEncoding			("b64");
        sc->setType				("application/*");
        sc->setURI				("briefcase");
		sc->setSupportedTypes	("application/*");   
        sc->setIsEnabled		("1");
    }

	// *** TBD ***
    //sc->setCtCap
    
    return sc;
}

int getBuildNumberFromVersion(const char* swv) {
    
    int major=0, minor=0, build=0;
    if (!swv) {
        return 0;
    }
    
    sscanf(swv, "%d.%d.%d", &major, &minor, &build);
    return (major*10000 + minor*100 + build);
}

void MacDefaultConfigFactory::upgrade(DMTClientConfig* clientconfig){
    const char* swversion= clientconfig->getClientConfig().getSwv();
    if( strcmp(swversion, "") == 0 ){

    }
} 

ArrayList* MacDefaultConfigFactory::getVCardProperties(){
    
    ArrayList* props = new ArrayList();
    Property* p = new Property();
    PropParam* pp = new PropParam();
    ArrayList pparams;
    ArrayList valenums;
    StringBuffer val;
    
    
    p->setPropName("BEGIN");
    val = "VCARD";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);
    
    valenums.clear();
    
    p->setPropName("END");
    val = "VCARD";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);
    
    valenums.clear();
    
    p->setPropName("VERSION");
    val = "2.1";
    valenums.add(val);
    p->setValEnums(&valenums);
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-ANNIVERSARY");
    props->add(*p);
    
    p->setPropName("BDAY");
    props->add(*p);
    
    p->setPropName("NOTE");
    props->add(*p);
    
    p->setPropName("TEL");
    pp->setParamName("VOICE");
    pparams.add(*pp);
    pp->setParamName("WORK");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("ADR");
    pp->setParamName("WORK");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("TEL");
    pp->setParamName("WORK");
    pparams.add(*pp);
    pp->setParamName("FAX");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("TEL");
    pp->setParamName("CAR");
    pparams.add(*pp);
    pp->setParamName("VOICE");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("CATEGORIES");
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-CHILDREN");
    props->add(*p);
    
    p->setPropName("TEL");
    pp->setParamName("WORK");
    pparams.add(*pp);
    pp->setParamName("PREF");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("ORG");
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-CUSTOMERID");
    props->add(*p);
    
    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    pp->setParamName("HOME");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    pp->setParamName("WORK");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("ADR");
    pp->setParamName("HOME");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("TEL");
    pp->setParamName("HOME");
    pparams.add(*pp);
    pp->setParamName("FAX");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("TEL");
    pp->setParamName("VOICE");
    pparams.add(*pp);
    pp->setParamName("HOME");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("EMAIL");
    pp->setParamName("INTERNET");
    pparams.add(*pp);
    pp->setParamName("HOME");
    pparams.add(*pp);
    pp->setParamName("X-FUNAMBOL-INSTANTMESSENGER");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("TITLE");
    props->add(*p);
    
    p->setPropName("N");
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-MANAGER");
    props->add(*p);
    
    p->setPropName("TEL");
    pp->setParamName("CELL");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("NICKNAME");
    props->add(*p);
    
    p->setPropName("ADR");
    props->add(*p);
    
    p->setPropName("TEL");
    pp->setParamName("PAGER");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("PHOTO");
    props->add(*p);
    
    p->setPropName("TEL");
    pp->setParamName("X-FUNAMBOL-RADIO");
    pparams.add(*pp);
    p->setPropParams(&pparams);
    props->add(*p);
    
    pparams.clear();
    delete p; p = NULL;
    delete pp; pp = NULL;
    p = new Property();
    pp = new PropParam();
    
    p->setPropName("X-FUNAMBOL-SPOUSE");
    props->add(*p);
    
    p->setPropName("URL");
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-YOMICOMPANYNAME");
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-YOMIFIRSTNAME");
    props->add(*p);
    
    p->setPropName("X-FUNAMBOL-YOMILASTNAME");
    props->add(*p);
    
    delete p; p = NULL;
    delete pp; pp = NULL;
    
    return props;
}
