/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef _SYNC_QUEUE_MANAGER_H_
#define _SYNC_QUEUE_MANAGER_H_

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <err.h>
#include <unistd.h>
#include <signal.h>

#include "base/Log.h"
#include "base/adapter/PlatformAdapter.h"
#include "customization.h"
#include "constants.h"
#include "utils.h"

class SyncQueueManager
{
    public:
        SyncQueueManager();
        SyncQueueManager(unsigned int info);
        ~SyncQueueManager();

        int lockSyncQueue();
        int unlockSyncQueue();

        int checkSyncQueue();
        int setCurrentSyncType();

        int resetSyncQueue();
        int cleanUpSyncQueue();
    
        static int getCurrentSyncType();
        static bool isSyncRunning();
        static bool isContactSyncRunning();
        static bool isContactResetRunning();
    
    private:
        unsigned int sync_info;
        const char* key_path;  // path for ftok() 
        key_t sem_key;         // sem key 
        key_t queued_shm_key;  // shared memory key for queue info
        key_t current_shm_key; // shared memory key for current sync
        struct sembuf sb;
        int sem_id;            // sync process semaphore
        int queued_shm_id;     // info shared memory id for sync queue
        int current_shm_id;    // info shared memory id for current sync

        int *queued_shm_val;   // value of queued sync type stored in shared memory
        int *current_shm_val;  // value of current sync type stored in shared memory

        static const int sem_key_id;
        static const int queued_shm_key_id;
        static const int current_shm_key_id;
        static const int sem_perms;
        static const int shm_perms;
        static const int shm_size;
    
        bool currentSyncInfoSet;
        bool queuedSyncInfoSet;
        bool syncLockSet;
    
        bool syncLockRelinquished;
    
        int checkLockStatus();
        int cleanUpSyncInfo();
};

#endif // _SYNC_QUEUE_MANAGER_H_
