/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef _MAC_CONSTANTS_H_
#define _MAC_CONSTANTS_H_

#include "customization.h"            // Funambol customizations header

// standard system applications paths
#define SYSTEM_APPLICATION_PATH "/Applications/"
#define SYSTEM_LIBRARY_PATH     "/Library/"
#define SYSTEM_PREFPANE_PATH    "/Library/PreferencePanes/"

#define SQLITE_DB_NAME       "/signatures.sql"

#define MAX_SYNC_SOURCES_NUM 2
#define CONFIG_SOURCE_NAME   "config"
#define CONTACTS_SOURCE_NAME "addressbook"
#define BRIEFCASE_SOURCE_NAME "briefcase"
/*
 * Notification event names
 */
#define SYNC_STATUS_NOTIFICATION           "FunambolSync_StatusNotification"
#define SYNC_ITEM_NOTIFICATION             "FunambolSync_ItemNotification"
#define SYNC_SOURCE_NOTIFICATION           "FunambolSync_SourceNotification"
#define SYNC_TRANSPORT_NOTIFICATION        "FunambolSync_TransportNotification"

#define CLIENT_SYNC_START_NOTIFICATION     "FunambolSync_SyncStartNotification"
#define CLIENT_SYNC_ENDED_NOTIFICATION     "FunambolSync_SyncEndedNotification"
#define SYNC_CANCELLED_NOTIFICATION        "FunambolSync_CancelledSyncNotification"
#define MENUBAR_CANCEL_SYNC_NOTIFICATION   "FunambolMenuBar_CancelSyncNotification"
#define MENUBAR_SET_UI_STATUS_NOTIFICATION "FunambolMenuBar_SetUiStatusNotification"

#define UPDATE_SOFTWARE_NOTIFICATION       "FunambolUpdateSoftware_UpdateAvailableNotification"

#define SYNC_SOURCE_NAME_NOTIFICATION       "FunambolSync_SourceName"

/*
 * Notification event value keys
 */
#define MENUBAR_STATUS_VALUE               "menuBarStatusValue"

/*
 * sync status codes 
 */
enum {
    kSyncSuccessful				= 0,
    kCodeServerBadRequest		= 400,
    kCodeWrongCredential		= 401,
    kCodeAuthExpired			= 402,
    kCodeForbidden				= 403,
    kCodeNotFound				= 2060, // ERR_HTTP_NOT_FOUND written in MacTransportAgent.h
    kCodeInvalidCredential		= 407,   // it is handled as the 401
    kCodeServerRequestTimeout	= 2061, // ERR_HTTP_REQUEST_TIMEOUT,    
    kCodeServerBusy				= 503,
    kCodeServerError			= 2052, // ERR_SERVER_ERROR (from libfunambol common/http/errors.h)
    kCodeServerErrorConnect		= 2001, // ERR_CONNECT		(from libfunambol common/http/errors.h)
    kSyncCancelledByUser		= 701,
	kSourceDefinitionNotFound	= 10001 // ERR_SOURCE_DEFINITION_NOT_FOUND (from libfunambol common/spdm/errors.h)
};

/*
 * menubar UI status
 */
enum {
    kMenuBarUiGotoHiddenStatus = 0,
    kMenubarUiGotoActiveStatus
};

/*
 * applications names and paths
 */

#define OPEN_COMMAND_PATH				   "/usr/bin/open"
#define SYSTEM_PREFERENCES_APP			   "/Applications/System Preferences.app/Contents/MacOS/System Preferences"
#define LOG_VIEWER_APPLICATION_PATH        "/Applications/TextEdit.app/Contents/MacOS/TextEdit"

// number of "spinning icons in menubar application
#define MENUBAR_SYNC_PROGRESS_ICONS_NUM     8

// shared memory constants
#define SHM_KEY_ID                  1
#define SHM_PERMS                   0600
#define SHM_SIZE                    sizeof(int)

#define SYNC_CONTACTS_OPT           "-a"
#define SYNC_WAIT_QUEUE_OPT         "-W"

#define LAUNCHCTL_PATH              "/bin/launchctl" 
#define LAUNCHCTL_LOAD_OPT          "load"
#define LAUNCHCTL_UNLOAD_OPT        "unload"

#define  SYNC_SOURCE_NONE      0
#define  SYNC_SOURCE_RESET     1
#define  SYNC_SOURCE_CONFIG    2
#define  SYNC_SOURCE_CONTACTS  4
#define  SYNC_SOURCE_BRIEFCASE 8

#endif
