/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef __CUSTOMIZATION_H__
#define __CUSTOMIZATION_H__

// customer prefix for all applications
#define APP_PREFIX_NAME  "funambol"
// customer application name
#define APPNAME          "Funambol"

#define APPCONTEXT       "/macosxclient"

#define WEBSITE          "www.funambol.com"
#define CORPORATION      "Funambol, Inc."
#define MAC_USER_AGENT   "Funambol Mac Sync OS Client" // to add the version 

//scheduling values
#define SCHEDULED_MINUTES_VALUES            "15,30"
#define SCHEDULED_HOURS_VALUES              "1,2,4,6,8,12,24"

// applications names
#define SYNC_APPLICATION_NAME                   APP_PREFIX_NAME"-sync"
#define MENUBAR_APPLICATION_NAME                APP_PREFIX_NAME"-menubar"
#define UPDATER_APPLICATION_NAME                APP_PREFIX_NAME"-updater"

#define LAUNCHER_APPLICATION_NAME               APPNAME
#define UNINSTALLER_APPLICATION_NAME            "Uninstaller"
#define PREFERENCE_APPLICATION_NAME             APP_PREFIX_NAME".prefPane"

// preference panel UI visible name
#define PREFERENCE_VISIBLE_NAME                 APPNAME" Sync"

// preference panel ID (used in applescript launcher script)
#define PREFERENCES_PANEL_ID     "com."APP_PREFIX_NAME".prefpanel"

// applications folders
#define CLIENT_SYSTEM_APPLICATION_FOLDER   "/Library/"APPNAME
#define CLIENT_USER_APPLICATION_FOLDER     "/Applications/"APPNAME
#define CLIENT_PREFERENCES_FOLDER          "/Library/PreferencePanes/"PREFERENCE_APPLICATION_NAME

// applications full paths
#define SYNC_APPLICATION_PATH           CLIENT_SYSTEM_APPLICATION_FOLDER"/"SYNC_APPLICATION_NAME
#define MENUBAR_APPLICATION_PATH        CLIENT_SYSTEM_APPLICATION_FOLDER"/"MENUBAR_APPLICATION_NAME".app/Contents/MacOS/"MENUBAR_APPLICATION_NAME
#define UPDATER_APPLICATION_PATH        CLIENT_SYSTEM_APPLICATION_FOLDER"/"UPDATER_APPLICATION_NAME".app/Contents/MacOS/"UPDATER_APPLICATION_NAME
#define LAUNCHER_APPLICATION_PATH       CLIENT_USER_APPLICATION_FOLDER"/"APPNAME".app/Contents/MacOS/"LAUNCHER_APPLICATION_NAME

// plist identifier for sync app (job label identifier)
#define SYNC_APPLICATION_PLIST_LABEL    "com."APP_PREFIX_NAME".sync"
#define MENUBAR_APPLICATION_PLIST_LABEL "com."APP_PREFIX_NAME".menubar"
#define UPDATER_APPLICATION_PLIST_LABEL "com."APP_PREFIX_NAME".updater"

// plist file name
#define SYNC_PLIST_FILE_NAME            "com."APP_PREFIX_NAME".sync.plist"
#define UPDATER_PLIST_FILE_NAME         "com."APP_PREFIX_NAME".updater.plist"
#define MENUBAR_PLIST_FILE_NAME         "com."APP_PREFIX_NAME".menubar.plist"

// path to plist file for launchd jobs
#define LAUNCHD_PLIST_PATH              "Library/LaunchAgents"

// application log file name and path
/* this is the path relative to PlatformAdapter::getConfigFolder() (built runtime by application) */
#define LOG_FILE_PATH               "logs/synclog.txt"
#define LOG_FILE_NAME               "synclog.txt"
#define DEFAULT_LOG_LEVEL           LOG_LEVEL_INFO
#define DEFAULT_SYNC_URL            "http://my.funambol.com/sync"
#define LICENSE_FILE                "LICENSE.rtf"
#define DISTRIBUITION_PROP_FILE     "distribution.properties"

//customize poweredby
#define POWEREDBY           0

// Easily turn on and off all custom changes
#define CUSTOM_ENABLE

#ifdef CUSTOM_ENABLE
// Change these to suit your needs
//enum { CUSTOM_HIDE_DISABLED_CATEGORIES         = 1 };
enum { CUSTOM_ENCODE_PASSWORD                    = 0 };
enum { CUSTOM_CHECK_CREDENTIALS_ON_EXIT          = 0 };
#else
// Do not change these
//enum { CUSTOM_HIDE_DISABLED_CATEGORIES         = 0 };
enum { CUSTOM_ENCODE_PASSWORD                    = 1 };
enum { CUSTOM_CHECK_CREDENTIALS_ON_EXIT          = 0 };
#endif

// funambol menubar icons
#define MENUBAR_MAIN_LOGO           "main-logo-18x18"
#define MENUBAR_ERROR_ICON          "icon_menubar_alert_18x18"
#define MENUBAR_PROGRESS_ICON_1     "icon_menubar_progress1_18x18"
#define MENUBAR_PROGRESS_ICON_2     "icon_menubar_progress2_18x18"
#define MENUBAR_PROGRESS_ICON_3     "icon_menubar_progress3_18x18"
#define MENUBAR_PROGRESS_ICON_4     "icon_menubar_progress4_18x18"
#define MENUBAR_PROGRESS_ICON_5     "icon_menubar_progress5_18x18"
#define MENUBAR_PROGRESS_ICON_6     "icon_menubar_progress6_18x18"
#define MENUBAR_PROGRESS_ICON_7     "icon_menubar_progress7_18x18"
#define MENUBAR_PROGRESS_ICON_8     "icon_menubar_progress8_18x18"
#define MENUBAR_LOGO_MENU_TYPE      "png"

//updater
#define UPDATE_SOFTWARE_URL_RESOURCE      "/sapi/profile/client?action=get-update-info"
#define UPDATER_COMPONENT                 "macos"
#define UPDATER_PACKAGE_DESTINATION       "/tmp/funsyncapp.zip"
#define UPDATER_UNZIP_PATH                UPDATER_FOLDER_DESTINATION"funsyncapp/"
#define UPDATER_FOLDER_DESTINATION        "/tmp/"

#define BRIEFCASE_DEFAULT_PATH            "briefcase"

#endif //__CUSTOMIZATION_H__
