/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef UTILS_H_DEF
#define UTILS_H_DEF


#include <sys/sysctl.h>

/**
 * Check if at least a process named 'processName' is running on
 * the system. If exclude_pid parameter is not 0, only processes
 * with pid different from the callers one will be considered: 
 * this is useful if, for example, the sync program wants to check 
 * if another sync is already running.
 */
int checkProcessRunning(const char* processName, int excludeSelf);

/**
 * Check if a process with pid target_pid is running
 * returns 0 if not found or if there were errors.
 */
int checkProcessRunningByPid(pid_t target_pid);

/**
 * check if there are processes with name 'processName' running
 * on the system and wait until they have exited. 
 */
int waitForRunningProcesses(const char* processName);

/**
 * wait for running pid until it's exited. 
 * returns < 0 if there are errors in kqueue
 * creation or if pid is not running
 */
int waitForProcessByPid(pid_t target_pid);

/* Returns a list of all BSD processes on the system.  This routine
 * allocates the list and puts it in *procList and a count of the
 * number of entries in *procCount. 
 * On success, the function returns 0.
 * On error, the function returns the errno value.
 * see: Technical Q&A QA1123: "Getting List of All Processes on Mac OS X"
 */
int getProcessList(struct kinfo_proc **procList, size_t *procCount);

/*
 * terminate a process sending a SIGTERM signal to target_pid
 * returns 0 on success and < 0 if error or pid is invalid (i.e. <= 0)
 */
int killProcess(pid_t target_pid);

/* start a process */
int execProcess(char * const args[], bool waitForChild);

/* lauches System Preferences opening sync client view */
int startSyncClientPrefPane();

#endif