/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import <Cocoa/Cocoa.h>

@class MenuBarUiUpdater;
@class SyncProcessWatchDog;

@interface MenuBarController : NSObject 
{
    // menu item in system menubar
    IBOutlet NSMenu *statusMenu;
    
    IBOutlet NSWindow *alertPopup; // the popup to be shown
    IBOutlet NSImageView *logo;    // the logo near the message
    IBOutlet NSTextField *textMessage;    // the message where message is shown
    IBOutlet NSButton *okButton;    // the message where message is shown
    
    IBOutlet NSWindow *firstSyncWindow;
    IBOutlet NSTextField* firstSyncTitle;
    IBOutlet NSTextField* firstSyncMessage1;
    IBOutlet NSTextField* firstSyncMessage2;
    //IBOutlet NSPopUpButton* firstSyncComboBox;
    IBOutlet NSButton* firstSyncButtonCancel;
    IBOutlet NSButton* firstSyncButtonOk;
    IBOutlet NSImageView* firstSyncImage;
    
    NSStatusItem *statusItem;
    
    // menubar images
    NSImage *statusImage;
    NSImage *statusHighlightImage;
    
    // images displayed during sync
    NSImage *syncProgressIcon1, *syncProgressIcon2,
            *syncProgressIcon3, *syncProgressIcon4,
            *syncProgressIcon5, *syncProgressIcon6,
            *syncProgressIcon7, *syncProgressIcon8;
    
    // sync error state icon
    NSImage *errorIcon;
    
    NSMutableArray* progressIcons;
    
    // notification center to get UI 
    // updates from sync process
    CFNotificationCenterRef nCenter;
    
    // sync process handle 
    NSTask *syncTask;
    
    MenuBarUiUpdater* syncProgressUiUpdater; 
    // menubar UI updater thread
    NSThread *syncProgressUiUpdaterThread;
    
    SyncProcessWatchDog* syncProcessWatchDog;
    NSThread *syncProcessWatchDogThread;
    
    NSString *currentSourceName;
    unsigned int totalClientItemsCount;
    unsigned int totalServerItemsCount;
    unsigned int currentProgressIconIndex;
    
    BOOL  uiEnabled;
    BOOL  syncRunning;
}

@property (retain) NSString *currentSourceName;
@property (readwrite,assign) unsigned int totalClientItemsCount; 
@property (readwrite,assign) unsigned int totalServerItemsCount; 
@property (readwrite, assign) BOOL syncRunning;

- (id)init;

- (IBAction) startSync : (id) sender;
- (IBAction) stopSync : (id) sender;
- (IBAction) showLastSyncDetails: (id) sender;
- (IBAction) closeAlertPopup: (id) sender;
- (IBAction) firstSyncWarningCancelAction : (id) sender;
- (IBAction) firstSyncWarningOkAction : (id) sender;
- (IBAction) startSoftwareUpdate : (id) sender;


/* 
 * creation and release methods
 */
- (void) setupMenu;
- (void) shutdownMenu;
- (void) setupNotificationObservers;
- (void) shutdownNotificationObservers;

- (BOOL) startSyncTask;
- (void) sendCancelMessageToSyncProcess;

- (void) checkSyncProcessStatus:(NSNotification *) aNotification;
- (void) setupDropDownMenu;

/*
 * control menubar visibility
 */

- (void) setHidden;
- (void) setVisibile;

/* 
 * methods to update the menubar UI status
 */

- (void) setMenuStartSyncCommand;
- (void) setMenuStopSyncCommand;
- (void) updateSyncStatusMenu: (NSString *)itemTitle;
- (void) setupSyncStatusMenu;
- (NSString *) getSourceTimeStamp: (NSString *)sourceName; // get last sync timestamp
- (void) hideSyncDetailsMenu: (BOOL) show;
- (void) hideSoftwareUpdateMenu: (BOOL) show;
- (BOOL) isAnUpdateAvailable;
/*
 * sync process watchdog thread
 */
- (void) startSyncProcessWatchDog: (int) syncProcessPid;
- (void) stopSyncProcessWatchDog;

/*
 * update menubar icon while syncing
 */
- (void) startSyncProgressIconsUpdate;
- (void) stopSyncProgressIconsUpdate;
- (void) displaySyncProgressIcon;

/*
 * set menubar icon
 */
- (void) setDefaultIcon;
- (void) setErrorIcon;

/*
 * utility method to convert config source 
 * names to corresponding display label
 */
- (NSString*) getSourceDisplayName: (CFStringRef)sourceName;

/*
 * get the name of the last source synced
 */
- (NSString*) getLastSourceSynced;

/*
 * menu items indexes
 */
enum {
    SYNC_STATUS_MENU_INDEX  = 0,
    SYNC_DETAILS_MENU_INDEX = 1,
    SEPARATOR_ITEM_1_MENU_INDEX = 2,
    SYNC_COMMAND_MENU_INDEX = 3,
    SEPARATOR_ITEM_2_MENU_INDEX = 4,
    SYNC_PREFERENCES_MENU_INDEX  = 5, 
    SEPARATOR_ITEM_3_MENU_INDEX  = 6,
    SOFTWARE_UPDATE_MENU_INDEX   = 7
};

@end

@interface MenuBarUiUpdater : NSObject
{
    CFRunLoopTimerRef syncProgressTimer;
    CFRunLoopRef threadRunLoop;
}

// ui updater thread main function
- (void) start: (MenuBarController *)menuBarUi;
- (void) stop;

@end


@interface SyncProcessWatchDog : NSObject
{
    NSDistributedNotificationCenter *nCenter;
    BOOL notifyEndSync;
}

- (id) init;
- (void) start: (NSNumber *)syncProcessPid;
- (void) stop;

@end

/**
 * NotificationCenter event handlers
 */
void syncItemEventCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);
void syncSourceEventCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);
void syncStatusEventCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);
void syncStartedCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);
void syncCancelledCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);
void syncEndedCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo); 
void syncCurrentSourceNameCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);
void setVisibleMenuBarCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo); 
void displaySyncProgressIconCallBack(CFRunLoopTimerRef timer, void *info);
void setUpdateNotificationCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo);

