/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import <PreferencePanes/NSPreferencePane.h>
#import "MenuBarController.h"
#import "Foundation/Foundation.h"
#import "Foundation/NSTask.h"
#import "stringUtils.h"
#import "ClientConfig.h"
#import "constants.h"
#import "event/constants.h"  // sync events defines from Funambol API
#import "messageUtils.h"
#import "utils.h"
#include "base/adapter/PlatformAdapter.h"
#include "updater/UpdaterConfig.h"
/*
 * This is the name of the last sync source just synced. It cames from the last message sent by the 
 * sync process and it is stored in a global variable since it is out of the class.
 * When there will be multiple source, this can be obsolete, since to show the status of the sources
 * we should go in the Sync Status panel and retrieve the status of every sources
 */
StringBuffer lastSyncSourceName;

USE_NAMESPACE;

int kSyncProgressIconsNum = MENUBAR_SYNC_PROGRESS_ICONS_NUM;

@implementation MenuBarUiUpdater

-(void) start: (MenuBarController *)menuBarUi
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init]; 
    
    // start the timer to update menubar icon
    CFAllocatorRef allocator = kCFAllocatorDefault;
    CFAbsoluteTime fireDate = CFAbsoluteTimeGetCurrent();
    CFRunLoopTimerContext context = { 0, menuBarUi, NULL, NULL, NULL };
    CFTimeInterval interval = 0.2;
    CFOptionFlags flags = 0;
    CFIndex order = 0;
    CFRunLoopTimerCallBack callback = (CFRunLoopTimerCallBack)displaySyncProgressIconCallBack;
    
    // create timer
    syncProgressTimer = CFRunLoopTimerCreate(allocator, fireDate, interval, flags, order, callback, &context);
    // attach to thread runloop
    threadRunLoop = CFRunLoopGetCurrent();
    CFRunLoopAddTimer(threadRunLoop, syncProgressTimer,  kCFRunLoopCommonModes);    
    // start thread runloop
    CFRunLoopRun();
    
    [pool release];
}

-(void) stop
{   
    // stop timer
    CFRunLoopTimerInvalidate(syncProgressTimer);
    CFRelease(syncProgressTimer);
    syncProgressTimer = nil;
    CFRunLoopStop(threadRunLoop);
}

@end

@implementation SyncProcessWatchDog

- (id) init
{
    if (self = [super init]) {
        nCenter = [NSDistributedNotificationCenter defaultCenter];
    }
    
    return self;
}

- (void) start: (NSNumber *)syncProcessPid
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init]; 
    int target_pid = 0;
    
    target_pid = [syncProcessPid integerValue];
    
    notifyEndSync = YES;
    
    while (1) {
        if (waitForProcessByPid(target_pid) < 0) {
            return;
        }

        if (notifyEndSync) {
            // sync process has exited unexpectedly: synthetize a sync end notification
            NSMutableDictionary* infoDict = [NSMutableDictionary dictionaryWithCapacity: 1];
            NSNumber* syncStatusCode = [NSNumber numberWithInt: kSyncCancelledByUser]; 
            
            [infoDict setObject:syncStatusCode forKey:(id)CFSTR("SyncStatusCode")];
            
            [nCenter postNotificationName:@CLIENT_SYNC_ENDED_NOTIFICATION object:nil userInfo: infoDict deliverImmediately: YES];
        }
        
        break;
    }
    
    [pool release];
}

- (void) stop
{
    notifyEndSync = NO;
}
@end


@implementation MenuBarController

- (id)init {
    self = [super init];
    return self;
}

- (void) awakeFromNib
{
    pid_t sync_process_pid = 0;
    
    if(!(ClientConfig::getInstance()->getIsInitialized())){
        ClientConfig::getInstance()->init();
        ClientConfig::getInstance()->read();
    }    
    
    if (ClientConfig::getInstance()->readMenubarUIEnabled()) {
        uiEnabled = YES;
    } else {
        uiEnabled = NO;
    }
    
    nCenter = CFNotificationCenterGetDistributedCenter();
    
    syncProgressUiUpdaterThread = nil;
    syncProcessWatchDogThread = nil;
    
    // register observer for sync notifications
    [self setupNotificationObservers];
    
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    NSString *path = [b pathForImageResource:@"logoImage.tiff"];
    NSImage* image = [[[NSImage alloc] initWithContentsOfFile:path] autorelease];
    [logo setImage:image];
    
    // create and initialize sync watchdog thread handler
    syncProcessWatchDog = [[SyncProcessWatchDog alloc] init];
    
    // create and initialize menu updater (handles sync progress spinning icon)
    syncProgressUiUpdater = [[MenuBarUiUpdater alloc] init];
    
    if ((sync_process_pid = checkProcessRunning(SYNC_APPLICATION_NAME, 0)) > 0) {
        syncRunning = YES;
        [self startSyncProcessWatchDog: sync_process_pid];
        [self startSyncProgressIconsUpdate];
    } else {
        syncRunning = NO;
    }
    
    if (uiEnabled) {
        // create menubar entry
        [self setupMenu];
        // add menu entries
        [self setupDropDownMenu];
    }
}

- (void) setupMenu
{
    // Create the NSStatusBar and set its length
    statusItem = [[[NSStatusBar systemStatusBar] statusItemWithLength:NSSquareStatusItemLength] retain];
    
    //Used to detect where our files are
    NSBundle *bundle = [NSBundle mainBundle];
    
    //Allocates and loads the images into the application which will be used for our NSStatusItem
    statusImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_MAIN_LOGO ofType:@MENUBAR_LOGO_MENU_TYPE]];
    statusHighlightImage = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_MAIN_LOGO ofType:@MENUBAR_LOGO_MENU_TYPE]];
    errorIcon = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_ERROR_ICON ofType:@MENUBAR_LOGO_MENU_TYPE]];
    
    progressIcons = [[NSMutableArray alloc] initWithCapacity: kSyncProgressIconsNum];
    
    syncProgressIcon1 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_1 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon1];
    syncProgressIcon2 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_2 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon2];
    syncProgressIcon3 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_3 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon3];
    syncProgressIcon4 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_4 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon4];
    syncProgressIcon5 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_5 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon5];
    syncProgressIcon6 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_6 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon6];
    syncProgressIcon7 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_7 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon7];
    syncProgressIcon8 = [[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@MENUBAR_PROGRESS_ICON_8 ofType:@MENUBAR_LOGO_MENU_TYPE]];
    [progressIcons addObject: (id)syncProgressIcon8];

    if (syncRunning == NO) {
        //Sets the images in our NSStatusItem
        [statusItem setImage:statusImage];
        [statusItem setAlternateImage:statusHighlightImage];
    }
    
    //Tells the NSStatusItem what menu to load
    [statusItem setMenu:statusMenu];
    
    //Sets the tooptip for our item
    NSString* ttip = [[[NSString alloc] initWithFormat:NSLocalizedString(@"toolTipMenu",@"-"), APPNAME] autorelease];
    [statusItem setToolTip:ttip];
       
    //Enables highlighting
    [statusItem setHighlightMode:YES];
}

- (void) setupNotificationObservers
{
    // set up observers for sync client notifications
    
    CFNotificationCenterAddObserver(nCenter, self, syncItemEventCallBack, 
                                    CFSTR(SYNC_ITEM_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    CFNotificationCenterAddObserver(nCenter, self, syncStatusEventCallBack, 
                                    CFSTR(SYNC_STATUS_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    CFNotificationCenterAddObserver(nCenter, self, syncSourceEventCallBack, 
                                    CFSTR(SYNC_SOURCE_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    CFNotificationCenterAddObserver(nCenter, self, syncStartedCallBack, 
                                    CFSTR(CLIENT_SYNC_START_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    CFNotificationCenterAddObserver(nCenter, self, syncCancelledCallBack, 
                                    CFSTR(SYNC_CANCELLED_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    
    CFNotificationCenterAddObserver(nCenter, self, syncEndedCallBack, 
                                    CFSTR(CLIENT_SYNC_ENDED_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    CFNotificationCenterAddObserver(nCenter, self, syncCurrentSourceNameCallBack, 
                                    CFSTR(SYNC_SOURCE_NAME_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);

    CFNotificationCenterAddObserver(nCenter, self, setVisibleMenuBarCallBack, 
                                    CFSTR(MENUBAR_SET_UI_STATUS_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
    
    CFNotificationCenterAddObserver(nCenter, self, setUpdateNotificationCallBack, 
                                    CFSTR(UPDATE_SOFTWARE_NOTIFICATION),  
                                    NULL, 
                                    CFNotificationSuspensionBehaviorDeliverImmediately);
}

- (void) shutdownNotificationObservers
{
    // deregister observers for sync events
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(SYNC_ITEM_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(SYNC_STATUS_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(SYNC_SOURCE_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(CLIENT_SYNC_START_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(SYNC_CANCELLED_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(CLIENT_SYNC_ENDED_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(MENUBAR_SET_UI_STATUS_NOTIFICATION), NULL);
    CFNotificationCenterRemoveObserver(nCenter, (id)self, CFSTR(UPDATE_SOFTWARE_NOTIFICATION), NULL);
}

- (void) shutdownMenu
{    
    //Releases all the images we loaded into memory
    [statusImage release];
    [statusHighlightImage release];

    // release alert icon
    [errorIcon release];
    // release "spinning" icons
    [syncProgressIcon1 release];
    [syncProgressIcon2 release];
    [syncProgressIcon3 release];
    [syncProgressIcon4 release];
    [syncProgressIcon5 release];
    [syncProgressIcon6 release];
    [syncProgressIcon7 release];
    [syncProgressIcon8 release];

    [progressIcons removeAllObjects];
    [progressIcons release];
    
    NSInteger n = [statusMenu numberOfItems];
    for (int i = 0; i < n; i++) {
        [statusMenu removeItemAtIndex:0];
        
    }
    
    [[NSStatusBar systemStatusBar] removeStatusItem:statusItem];
}

- (void) dealloc 
{
    [self shutdownNotificationObservers];
    
    [syncProcessWatchDog dealloc];
    [syncProgressUiUpdater dealloc];

    [self shutdownMenu];
    [super dealloc];
}

- (void) setupDropDownMenu
{
    // set up sync status menu
    NSMenuItem* menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] 
                            initWithTitle:NSLocalizedString(@"neverSyncedMenuItem",@"-") action:NULL keyEquivalent:@""];
    [menuItem setEnabled:NO];
	[statusMenu insertItem: menuItem atIndex: SYNC_STATUS_MENU_INDEX];
    [menuItem release];
    
    // set up sync details menu (enabled only if a sync error occurred)
    menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] 
                initWithTitle:NSLocalizedString(@"showDetailsMenuItem",@"-") action:NULL keyEquivalent:@""];
    [menuItem setEnabled:NO];
    [menuItem setHidden: YES];
	[statusMenu insertItem:menuItem atIndex: SYNC_DETAILS_MENU_INDEX];
    [menuItem release];
    
    menuItem = [NSMenuItem separatorItem];
    [statusMenu insertItem:menuItem atIndex: SEPARATOR_ITEM_1_MENU_INDEX];
    
    if (syncRunning) {
        menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] 
                    initWithTitle:NSLocalizedString(@"cancelSyncMenuItem",@"-") action: @selector(stopSync:) keyEquivalent:@""];
     } else {
        // set up start/stop menu
        menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] 
                    initWithTitle:NSLocalizedString(@"syncNowMenuItem",@"-") action: @selector(startSync:) keyEquivalent:@""];
    }
    
    [menuItem setEnabled:YES];
    [menuItem setTarget:self];
    [statusMenu insertItem:menuItem atIndex: SYNC_COMMAND_MENU_INDEX];
    [menuItem release];
    
    menuItem = [NSMenuItem separatorItem];
    [statusMenu insertItem:menuItem atIndex: SEPARATOR_ITEM_2_MENU_INDEX];
    
    // set up preferences hook menu
    menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] 
                initWithTitle:NSLocalizedString(@"preferencesMenuItem",@"-") action: @selector(showPreferences:) keyEquivalent: @""];
    [menuItem setEnabled:YES];
    [menuItem setTarget: self];
	//[menuItem setAction: @selector(showPreferences:)];
	[statusMenu insertItem:menuItem atIndex: SYNC_PREFERENCES_MENU_INDEX];
    [menuItem release];

    if (syncRunning) {
        [self updateSyncStatusMenu: NSLocalizedString(@"syncRunningMenuItem", @"-")];
    } else {
        // setup menu item with last sync infos
        [self setupSyncStatusMenu];
    } 
    
    // create menu for autoupdate    
    menuItem = [NSMenuItem separatorItem];
    [statusMenu insertItem:menuItem atIndex: SEPARATOR_ITEM_3_MENU_INDEX];
    [menuItem setEnabled:NO];
    [menuItem setHidden:YES];
    
    menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] 
                initWithTitle:NSLocalizedString(@"softwareUpdate",@"-") action: @selector(startSoftwareUpdate:) keyEquivalent: @""];
    
    [menuItem setEnabled:YES];
    [menuItem setHidden:YES];
    [menuItem setTarget: self];
    [statusMenu insertItem:menuItem atIndex: SOFTWARE_UPDATE_MENU_INDEX];
    [menuItem release];
    
    if ([self isAnUpdateAvailable]) {
        [self hideSoftwareUpdateMenu:FALSE];
    }
    
}

- (void) setMenuStartSyncCommand
{
    if (uiEnabled) {
        NSMenuItem* menuItem = [statusMenu itemAtIndex: SYNC_COMMAND_MENU_INDEX];
        [menuItem setEnabled:YES];
        [menuItem setTarget:self];
        [menuItem setTitle: NSLocalizedString(@"syncNowMenuItem",@"-")];
        [menuItem setAction: @selector(startSync:)];
    }
}

- (void) setMenuCancellingSyncCommand
{
    if (uiEnabled) {
        NSMenuItem* menuItem = [statusMenu itemAtIndex: SYNC_COMMAND_MENU_INDEX];
        [menuItem setEnabled:NO];
        [menuItem setTarget: nil];
        [menuItem setTitle: NSLocalizedString(@"cancellingSyncMenuItem",@"-")];
    }
}

- (void) setMenuStopSyncCommand
{
    if (uiEnabled) {
        NSMenuItem* menuItem = [statusMenu itemAtIndex: SYNC_COMMAND_MENU_INDEX];
        [menuItem setEnabled:YES];
        [menuItem setTarget:self];
        [menuItem setTitle: NSLocalizedString(@"cancelSyncMenuItem",@"-")];
        [menuItem setAction: @selector(stopSync:)];
    }
}

- (void) updateSyncStatusMenu: (NSString *)itemTitle
{
    if (uiEnabled) {
        NSMenuItem* menuItem = [statusMenu itemAtIndex: SYNC_STATUS_MENU_INDEX];
        [menuItem setEnabled:NO];
        [menuItem setTitle: itemTitle];
    }
}

- (void) hideSyncDetailsMenu: (BOOL) show;
{
    if (uiEnabled) {
        NSMenuItem* menuItem = [statusMenu itemAtIndex: SYNC_DETAILS_MENU_INDEX];
        [menuItem setEnabled:YES];
        [menuItem setHidden: show];
        [menuItem setTarget:self];
        [menuItem setTitle: NSLocalizedString(@"showDetailsMenuItem",@"-")];
        [menuItem setAction: @selector(showLastSyncDetails:)];
    }
}

- (void) hideSoftwareUpdateMenu: (BOOL) show;
{
    if (uiEnabled) {
        NSMenuItem* menuItemSep = [statusMenu itemAtIndex: SEPARATOR_ITEM_3_MENU_INDEX];
        [menuItemSep setEnabled:YES];
        [menuItemSep setHidden: show];
        
        NSMenuItem* menuItem = [statusMenu itemAtIndex: SOFTWARE_UPDATE_MENU_INDEX];        
        [menuItem setTarget:self];
        [menuItem setEnabled:YES];
        [menuItem setHidden: show];
        [menuItem setTitle: NSLocalizedString(@"softwareUpdate",@"-")];
        [menuItem setAction: @selector(startSoftwareUpdate:)];
    }
}

- (IBAction) showLastSyncDetails: (id) sender;
{
    ClientConfig* config = ClientConfig::getInstance();
    
    // 
    // get the last status code of the sync and show it
    //
    int ret = config->readLastSyncStatusCode();
    NSString* message = getMessageFromCode(ret);
    NSString* title = [[[NSString alloc] initWithFormat:NSLocalizedString(@"genericPopupMenuTitle",@"-"), APPNAME] autorelease];
    [alertPopup setTitle:title];
    [okButton setStringValue:NSLocalizedString(@"ok",@"-")];
    [textMessage setStringValue:message];
    
    [NSApp activateIgnoringOtherApps:YES];
    [alertPopup makeKeyAndOrderFront:self];    
}

- (IBAction) closeAlertPopup: (id) sender;
{
    [alertPopup close];
    if(!(ClientConfig::getInstance()->areCredentialValid())){
        NSLog(@"showing preferences...");
        
        startSyncClientPrefPane();
    }
    
    return;
}

- (NSString*) getLastSourceSynced
{
    ClientConfig* config = ClientConfig::getInstance();
    int sourcesNum = 0, i = 0;
    NSString* lastSourceSynced = nil;
    unsigned long lastTimeStamp = 0L;
    
    if (config) {
        config->read();
        sourcesNum = config->getNumSources();
        
        for (i = 0; i < sourcesNum; i++) {
            SyncSourceConfig* srcConfig = config->getSyncSourceConfig(i);
            
            if (srcConfig && strcmp(srcConfig->getName(), CONFIG_SOURCE_NAME) != 0 ) {
                unsigned long sourceTimeStamp = srcConfig->getLast();
               
                if (sourceTimeStamp > lastTimeStamp) {
                    const char* sourceName = srcConfig->getName();
                    if (sourceName) {
                        lastSourceSynced = [NSString stringWithUTF8String: sourceName];
                    }
                }
            }
        }
        
        [lastSourceSynced retain];
    } 
    
    return lastSourceSynced;
}


- (void) setupSyncStatusMenu
{
    ClientConfig* config = ClientConfig::getInstance();
    int sourcesNum = 0, i = 0;
    NSString* lastSourceSynced = nil;
    unsigned long lastTimeStamp = 0L;
    BOOL allSourcesNotSynced = YES;
    
    if (config) {
        config->read();
        
        //
        // Check if there are some errors from the last sync
        // @TODO: for moultiple sources, check also if there are other errors
        // at source level
        //
        int code = config->readLastSyncStatusCode();
        if ( code != 0) {
            LOG.debug("There are error from the previous sync...");
            NSString* toShow = NSLocalizedString(@"lastSyncInfoMessage",@"-");
            if (code == kSyncCancelledByUser) {
                toShow = [toShow stringByAppendingString: NSLocalizedString(@"cancelledStatusMenuItem",@"-")];
                [self hideSyncDetailsMenu: YES];
            } else {
                [self setErrorIcon];
                [self hideSyncDetailsMenu: NO];
                toShow = [toShow stringByAppendingString: NSLocalizedString(@"lastSyncInfoFailed",@"-")];
            }          
            
            [self updateSyncStatusMenu: toShow];
            
            return;
        }
        
        sourcesNum = config->getNumSources();
        
        for (i = 0; i < sourcesNum; i++) {
            SyncSourceConfig* srcConfig = config->getSyncSourceConfig(i);

            if (srcConfig && strcmp(srcConfig->getName(), CONFIG_SOURCE_NAME) != 0 ) {
                unsigned long sourceTimeStamp = srcConfig->getLast();
                if (sourceTimeStamp > 100 && allSourcesNotSynced){
                    allSourcesNotSynced = NO;
                }
                if (sourceTimeStamp > lastTimeStamp) {
                    const char* sourceName = srcConfig->getName();
                    if (sourceName) {
                        lastSourceSynced = [NSString stringWithUTF8String: sourceName];
                    }
                }
            }
        }
    } 
    
    if (lastSourceSynced && !allSourcesNotSynced) {
        NSLog(@"last source synced: %@", lastSourceSynced);
        NSString* toShow = NSLocalizedString(@"lastSyncInfoMessage",@"-");
        NSString* timeStamp = [self getSourceTimeStamp: lastSourceSynced]; 
        toShow = [toShow stringByAppendingString: timeStamp];
        [self updateSyncStatusMenu: toShow];
    } else {
        [self updateSyncStatusMenu: NSLocalizedString(@"neverSyncedMenuItem", @"-")];
    }
}


-(void) startSyncProgressIconsUpdate
{
    if (syncProgressUiUpdater) {
        if (syncProgressUiUpdaterThread == nil) {
            // reset spinning icon index
            currentProgressIconIndex = 0;
            syncProgressUiUpdaterThread = [[NSThread alloc] initWithTarget:syncProgressUiUpdater 
                                                                  selector:@selector(start:) 
                                                                    object:(id)self];
        
            [syncProgressUiUpdaterThread start];
        }
    }
 }

-(void) stopSyncProgressIconsUpdate
{
    if (syncProgressUiUpdaterThread) {
        if ([syncProgressUiUpdaterThread isExecuting]) {
            [syncProgressUiUpdater stop];
        }
        
        [syncProgressUiUpdaterThread release];
        syncProgressUiUpdaterThread = nil;
    }
}


-(void) startSyncProcessWatchDog: (int)syncProcessPid
{
    if (syncProcessWatchDog) {
        if (syncProcessWatchDogThread == nil) {
            NSNumber *syncPid = [NSNumber numberWithInt: syncProcessPid];
        
            [syncPid retain];
        
            syncProcessWatchDogThread = [[NSThread alloc] initWithTarget:syncProcessWatchDog 
                                                                selector:@selector(start:) 
                                                                  object:(id)syncPid];
        
            [syncProcessWatchDogThread start];
        }
    }
}

-(void) stopSyncProcessWatchDog
{
    if (syncProcessWatchDogThread) {
        if ([syncProcessWatchDogThread isExecuting]) {
            [syncProcessWatchDog stop];
        }
        
        [syncProcessWatchDogThread release];
        syncProcessWatchDogThread = nil;
    }
}

-(void) displaySyncProgressIcon
{
    if (uiEnabled) {
        NSImage* progressIcon = (NSImage*)[progressIcons objectAtIndex: currentProgressIconIndex];
        
        if (progressIcon) {
            [statusItem setImage:progressIcon];
            [statusItem setAlternateImage:progressIcon];
        }
        
        if (currentProgressIconIndex < (kSyncProgressIconsNum - 1)) {
            currentProgressIconIndex++;
        } else {
            currentProgressIconIndex = 0;
        }
    }
}

- (void) setDefaultIcon
{
    if (uiEnabled) {
        [statusItem setImage:statusImage];
        [statusItem setAlternateImage:statusImage];
    }
}

- (void) setErrorIcon
{
    if (uiEnabled) {
        [statusItem setImage:errorIcon];
        [statusItem setAlternateImage:errorIcon];
    }
}

-(void) setHidden
{
    if (uiEnabled) {
        uiEnabled = NO; // first of all update the UI status
        
        [self shutdownMenu];
    }
}

-(void) setVisibile
{
    if (!uiEnabled) {
        uiEnabled = YES; // first of all update the UI status
        
        [self setupMenu];
        // add menu entries
        [self setupDropDownMenu];
    }
}


-(BOOL) startSyncTask 
{
    NSString *sync_contacts_opt = @"-a";
    syncTask = [[NSTask alloc] init];
    
    // register app as an observer for sync client
    [[NSNotificationCenter defaultCenter] addObserver:self  
                                              selector:@selector(checkSyncProcessStatus:)
                                              name:NSTaskDidTerminateNotification
                                              object: syncTask];
    
    NSMutableArray *args = [NSMutableArray array];
    [args addObject: sync_contacts_opt];
    [syncTask setLaunchPath: @SYNC_APPLICATION_PATH];
    [syncTask setArguments:args];
    // execute program 
    [syncTask launch];
        
    if ([syncTask isRunning]) {
        [self setMenuStopSyncCommand];
        return TRUE;
    } else {
        return FALSE;
    }
}


- (IBAction) startSync: (id) sender 
{
    BOOL ret = FALSE;
    
    if(ClientConfig::getInstance()->areCredentialSaved()){
        if(ClientConfig::getInstance()->getSyncSourceConfig("addressbook")->getLast() == 0){
        
            NSString* menuTitle = [[NSString  alloc] initWithFormat:NSLocalizedString(@"genericPopupMenuTitle", @"-"),APPNAME];
            [firstSyncWindow setTitle:menuTitle];
        
            [firstSyncTitle setStringValue:NSLocalizedString(@"firstSyncTitle", @"-")];
            [firstSyncMessage1 setStringValue:NSLocalizedString(@"firstSyncMessage1", @"-")] ;
            [firstSyncMessage2 setStringValue:NSLocalizedString(@"firstSyncMessage2", @"-")];

            [firstSyncImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
        
            [NSApp activateIgnoringOtherApps:YES];
            [firstSyncWindow makeKeyAndOrderFront:self];
            [menuTitle release];
        }else{

            NSLog(@"starting sync process...");
            [self hideSyncDetailsMenu:YES];
            ret = [self startSyncTask];
    
            if (ret == TRUE) {
                NSLog(@"sync task started");
            } else {
                NSLog(@"error starting sync task!");
            }
        }
    }else{
        NSString* mtitle = [[NSString alloc] initWithFormat:NSLocalizedString(@"genericPopupMenuTitle",@"-"),APPNAME];
        [alertPopup setTitle:mtitle];
        [okButton setStringValue:NSLocalizedString(@"ok",@"-")];
        [textMessage setStringValue:NSLocalizedString(@"EmptyMessage",@"-")];
        
        [NSApp activateIgnoringOtherApps:YES];
        [alertPopup makeKeyAndOrderFront:self];  
        [mtitle release];
        }
}


- (void) sendCancelMessageToSyncProcess
{
    CFMutableDictionaryRef infoDict = CFDictionaryCreateMutable(NULL, 1, &kCFCopyStringDictionaryKeyCallBacks, 
                                                                       &kCFTypeDictionaryValueCallBacks);
    NSLog(@"sending cancel sync notification...");
    CFNotificationCenterPostNotification(nCenter, CFSTR(SYNC_CANCELLED_NOTIFICATION), NULL, infoDict, TRUE);
    CFRelease(infoDict);
}

-(IBAction) stopSync : (id) sender 
{
    NSLog(@"stopping sync!");
    [[NSNotificationCenter defaultCenter]  removeObserver: self];
    [self setMenuCancellingSyncCommand];
    [self sendCancelMessageToSyncProcess];
}

-(IBAction) showPreferences : (id) sender 
{
    NSLog(@"showing preferences...");
    
	startSyncClientPrefPane();
}

-(IBAction) startSoftwareUpdate : (id) sender 
{
    NSLog(@"starting software udpate...");
    
	NSString *sync_contacts_opt = @"force_update";
    syncTask = [[NSTask alloc] init];
       
    NSMutableArray *args = [NSMutableArray array];
    [args addObject: sync_contacts_opt];
    [syncTask setLaunchPath: @UPDATER_APPLICATION_PATH];
    [syncTask setArguments:args];
    // execute program 
    [syncTask launch];
    
    if ([syncTask isRunning]) {
        NSLog(@"starting software udpate: syncTask running");
        //return TRUE;
    } else {
        NSLog(@"starting software udpate: syncTask not running...");
        //return FALSE;
    }
    //UpdateManager
    
}


-(void)checkSyncProcessStatus:(NSNotification *) aNotification 
{    
    int status = [[aNotification object] terminationStatus];
    
    if (status == 0) {
        NSLog(@"sync ended succesfully.");
    } else {
        NSLog(@"sync failed.");
    }        

    [syncTask release];
    
    [[NSNotificationCenter defaultCenter]  removeObserver: self];
  
    [self setMenuStartSyncCommand];
}

- (NSString*) getSourceDisplayName: (CFStringRef)sourceName
{
    NSString* sourceLabel = nil;
    
    if (CFStringCompare(sourceName, CFSTR(CONTACTS_SOURCE_NAME), 0) == kCFCompareEqualTo) {
        sourceLabel = NSLocalizedString(@"contactSourceDisplayName", @"-");
    } else if (CFStringCompare(sourceName, CFSTR(BRIEFCASE_SOURCE_NAME), 0) == kCFCompareEqualTo) {
        sourceLabel = NSLocalizedString(@"briefcaseSourceDisplayName", @"-");
    } 
    
    return sourceLabel;
}

- (NSString *) getSourceTimeStamp: (NSString*) sourceName 
{
    NSString* timeStamp = nil;
    NSString* format = @"yyyy-MM-dd";
    
    
    NSDate* today = [NSDate date];
    NSDate* yesterday = [NSDate dateWithTimeIntervalSinceNow:-86400];
    
    NSDateFormatter *outputFormatter = [[[NSDateFormatter alloc] init] autorelease];
    [outputFormatter setDateFormat:format];
    
    NSString *todayDateString = [outputFormatter stringFromDate:today];
    NSString *yesterdayDateString = [outputFormatter stringFromDate:yesterday];
    
       
    if (sourceName) {
        ClientConfig* config = ClientConfig::getInstance();
        const char* sourcename = [sourceName UTF8String];
        SyncSourceConfig *srcconfig = NULL;
        
        ClientConfig::getInstance()->read();
        srcconfig =  config->getSyncSourceConfig(sourcename);
         
        if (srcconfig) {
            int sourcesNum = config->getNumSources();
            unsigned long lastTimeStamp = 0;
            for (int i = 0; i < sourcesNum; i++) {
                SyncSourceConfig* srcConfig = config->getSyncSourceConfig(i);
                
                if (srcConfig && strcmp(srcConfig->getName(), CONFIG_SOURCE_NAME) != 0 ) {
                    unsigned long sourceTimeStamp = srcConfig->getLast();
                    
                    if (sourceTimeStamp > lastTimeStamp) {
                        lastTimeStamp = sourceTimeStamp;
                    }
                }
            }
            
            
            unsigned long sourceTimeStamp = lastTimeStamp;
            NSDate* date = [NSDate dateWithTimeIntervalSince1970: sourceTimeStamp];
            NSString* syncDateString = [outputFormatter stringFromDate:date];
            
            if ([syncDateString isEqualToString:todayDateString]) {
                format = NSLocalizedString(@"todayFormat",@"-");
            } else if ([syncDateString isEqualToString:yesterdayDateString]) {
                format = NSLocalizedString(@"yesterdayFormat",@"-");
            } else {
                format = NSLocalizedString(@"otherFormat",@"-");
            }
            
            [outputFormatter setDateFormat:format];
            
            NSArray* arr = [NSLocale availableLocaleIdentifiers];
            if ([arr containsObject:NSLocalizedString(@"localeFormat",@"-")]) {
                [outputFormatter setLocale:[[[NSLocale alloc] 
                                            initWithLocaleIdentifier:NSLocalizedString(@"localeFormat",@"-")]
                                            autorelease]];
            } else {
                [outputFormatter setLocale:[NSLocale currentLocale]];
            }
            
            timeStamp = [outputFormatter stringFromDate:date];
            
        }
    }
     
    return timeStamp;
}

- (IBAction) firstSyncWarningOkAction : (id) sender{
    
    BOOL ret = FALSE;

    NSLog(@"starting sync process...");
    [self hideSyncDetailsMenu:YES];
    ret = [self startSyncTask];
    
    if (ret == TRUE) {
        NSLog(@"sync task started");
    } else {
        NSLog(@"error starting sync task!");
    }
    [firstSyncWindow close];
}
- (IBAction) firstSyncWarningCancelAction : (id) sender{
    [firstSyncWindow close];
}


- (BOOL) isAnUpdateAvailable
{
    NSLog(@"isAnUpdateAvailable");
    BOOL ret = FALSE;
    StringBuffer appuri(PlatformAdapter::getAppContext());
    UpdaterConfig* upconfig = new UpdaterConfig(appuri.c_str());
    upconfig->read();
    
    NSLog(@"isAnUpdateAvailable LATER %s", upconfig->getLater().c_str());
    NSLog(@"isAnUpdateAvailable SKIPPED %s", upconfig->getSkipped().c_str());
    NSLog(@"isAnUpdateAvailable NOW %s", upconfig->getNow().c_str());
    if (upconfig->getLater().empty() == false && upconfig->getLater() != "0") {
        ret = TRUE;
    } else if (upconfig->getSkipped().empty() == false && upconfig->getSkipped() != "0") {
        ret = TRUE;
    } else if (upconfig->getNow().empty() == false && upconfig->getNow() != "0") {
        ret = TRUE;
    } 
    delete upconfig;
    return ret;    
    
}

// synthesize setters/getters
@synthesize currentSourceName;
@synthesize totalClientItemsCount;
@synthesize totalServerItemsCount;
@synthesize syncRunning;

@end


void syncItemEventCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo) 
{    
    if (userInfo) {
        CFStringRef sourceName = (CFStringRef)CFDictionaryGetValue(userInfo, CFSTR("SourceName"));
        CFNumberRef notificationType = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("NotificationType"));
        CFNumberRef itemCount = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("ItemCount"));
        
        if (itemCount) {
            int item_count = 0;
            int notification_type = 0;
            
            if (CFNumberGetValue(itemCount, kCFNumberIntType, &item_count) &&
                    CFNumberGetValue(notificationType, kCFNumberIntType, &notification_type)) {
                CFStringRef statusString = NULL; 
                unsigned int totalItems = 0;
            
                switch (notification_type) {
                    case ITEM_ADDED_BY_SERVER:
                    case ITEM_DELETED_BY_SERVER:
                    case ITEM_UPDATED_BY_SERVER:
                        totalItems = [(id)observer totalServerItemsCount];
                        statusString = CFStringCreateWithFormat(kCFAllocatorDefault, nil, 
                                                                (CFStringRef)NSLocalizedString(@"eventReceivingMenuItem",@"-"),
                                                                item_count, totalItems, [(id)observer getSourceDisplayName: sourceName]);
                        break;
                        
                    case ITEM_ADDED_BY_CLIENT:
                    case ITEM_DELETED_BY_CLIENT:
                    case ITEM_UPDATED_BY_CLIENT:
                        totalItems = [(id)observer totalClientItemsCount];
                        statusString = CFStringCreateWithFormat(kCFAllocatorDefault, nil, 
                                                                (CFStringRef)NSLocalizedString(@"eventSendingMenuItem",@"-"),                    
                                                                item_count, totalItems, [(id)observer getSourceDisplayName: sourceName]);
                        break;
                        
                    default:
                        NSLog(@"invalid event type");
                        break;
                }
                
                if (statusString) {
                    [(id)observer updateSyncStatusMenu:(NSString *)statusString];
                    
                    CFRelease(statusString); 
                }
            }
        }
    }
}

void syncSourceEventCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo) 
{    
    if (userInfo) {
        CFStringRef sourceName = (CFStringRef)CFDictionaryGetValue(userInfo, CFSTR("SourceName"));
        CFNumberRef notificationType = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("NotificationType"));
        int notification_type = 0;
        
        if (CFNumberGetValue(notificationType, kCFNumberIntType, &notification_type)) {
            CFNumberRef itemCount = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("ItemCount"));
            int count = 0;
            
            CFNumberGetValue(itemCount, kCFNumberIntType, &count);
        
            switch (notification_type) {
                case SYNC_SOURCE_TOTAL_CLIENT_ITEMS:
                    [(id)observer setTotalClientItemsCount: count];
                    [(id)observer setCurrentSourceName:(NSString*)sourceName];
                    break;
                    
                case SYNC_SOURCE_TOTAL_SERVER_ITEMS:
                    [(id)observer setCurrentSourceName:(NSString*)sourceName];
                    [(id)observer setTotalServerItemsCount: count];
                    break;

				case SYNC_SOURCE_END: {
					NSString* statusString = NSLocalizedString(@"eventWaitingServerData",@"-");
					[(id)observer updateSyncStatusMenu: statusString];
				
					break;
				}
					
                default:
                    break;
            }
        }
    }
}


void syncStatusEventCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo) 
{    
    CFNumberRef notificationType = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("NotificationType"));
    int notification_type = 0;
    
    if (CFNumberGetValue(notificationType, kCFNumberIntType, &notification_type)) {
        NSString* statusString = nil;
        
        switch (notification_type) {
            case SYNC_BEGIN: {
                statusString =  NSLocalizedString(@"eventConnectingMenuItem",@"-");
                break;
            }
            case SEND_INITIALIZATION:
                statusString = NSLocalizedString(@"eventLoggingOnMenuItem",@"-");
                break;
                
            case SEND_FINALIZATION:
                statusString = NSLocalizedString(@"eventDisconnectingMenuItem",@"-");
                break;
            
            case SYNC_END:
                statusString = NSLocalizedString(@"eventSyncEndedMenuItem",@"-"); 
                break;
            
            case SYNC_ERROR:
                statusString = NSLocalizedString(@"eventErrorMenuItem",@"-");
                break;
        }

        if (statusString) {
            [(id)observer updateSyncStatusMenu: statusString];
        }
    }
}

void syncStartedCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo) 
{    
    NSLog(@"%@ notification received", name);
    
    if ([(id)observer syncRunning] == NO) {
        [(id)observer setSyncRunning: YES];
        [(id)observer startSyncProgressIconsUpdate];
        [(id)observer setMenuStopSyncCommand];
        [(id)observer hideSyncDetailsMenu:YES];
        
        if (userInfo) {
            pid_t sync_process_pid = 0;
            CFNumberRef syncProcessPid = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("SyncProcessPid"));
        
            if (CFNumberGetValue(syncProcessPid, kCFNumberIntType, &sync_process_pid)) {
                [(id)observer startSyncProcessWatchDog: sync_process_pid];
            }
        }
    }
}


void syncCancelledCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo)
{
    [(id)observer setMenuCancellingSyncCommand];
}

void syncEndedCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo) 
{    
    NSLog(@"%@ notification received", name);
   
    if ([(id)observer syncRunning]) {
        [(id)observer setSyncRunning: NO];
        
        // stop sync process watchdog 
        [(id)observer stopSyncProcessWatchDog];

        // stop icon spinning on menubar ui
        [(id)observer stopSyncProgressIconsUpdate];

        if (userInfo) {
            int sync_status_code = 0;
            CFNumberRef syncStatusCode = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR("SyncStatusCode"));
            NSString* currentSource = [(id)observer currentSourceName];
            
            
            if (CFNumberGetValue(syncStatusCode, kCFNumberIntType, &sync_status_code)) {
                NSString* toShow = NSLocalizedString(@"lastSyncInfoMessage",@"-");
                switch (sync_status_code) {
                    case kSyncSuccessful:  {
                        [(id)observer setDefaultIcon];
                        // menubar app can be started while a sync is running: in this case 
                        // we could have missed some of the sync client notifications and 
                        // not set the source name
                        if ([currentSource length] == 0) {
                            currentSource = [(id)observer getLastSourceSynced];
                            
                        }
                        // sync successful: update menu item with current source last time stamp
                        NSString* timeStamp = [(id)observer getSourceTimeStamp: currentSource];
                        toShow = [toShow stringByAppendingString: timeStamp];
                        [(id)observer hideSyncDetailsMenu: YES];
                        break;
                    }

                    case kSyncCancelledByUser:
                        toShow = [toShow stringByAppendingString: NSLocalizedString(@"cancelledStatusMenuItem",@"-")];
                        [(id)observer hideSyncDetailsMenu: YES];
                        [(id)observer setDefaultIcon];
                        break;
                        
                    default:
                        // sync error
                        toShow = [toShow stringByAppendingString: NSLocalizedString(@"lastSyncInfoFailed",@"-")];
                        [(id)observer hideSyncDetailsMenu: NO];
                        [(id)observer setErrorIcon];
                        break;
                   
                }

                [(id)observer updateSyncStatusMenu: toShow];
                
            }
        }

        // we update anyway the menubar item with sync start command 
        [(id)observer setMenuStartSyncCommand];
    }
}
void syncCurrentSourceNameCallBack(CFNotificationCenterRef center, void *observer,
                                   CFStringRef name, const void *object, CFDictionaryRef userInfo)
{
    if (userInfo) {
        CFStringRef sourceName = (CFStringRef)CFDictionaryGetValue(userInfo, CFSTR("SourceName"));
        [(id)observer setCurrentSourceName:(NSString*)sourceName];
        lastSyncSourceName = CFString2Std(sourceName);
    }
}

void setVisibleMenuBarCallBack(CFNotificationCenterRef center, void *observer, CFStringRef name, const void *object, CFDictionaryRef userInfo)
{
    NSLog(@"%@ notification received", name);
 
    if (userInfo) {
        CFNumberRef statusValue = (CFNumberRef)CFDictionaryGetValue(userInfo, CFSTR(MENUBAR_STATUS_VALUE));
        int status_value = 0;
        
        if (CFNumberGetValue(statusValue, kCFNumberIntType, &status_value)) {
            switch (status_value) {
                case kMenuBarUiGotoHiddenStatus:
                    [(id)observer setHidden];
                    break;
                    
                case kMenubarUiGotoActiveStatus:
                    [(id)observer setVisibile];
                    break;
                    
                default:
                    break;
            }
        }
    }
}

void displaySyncProgressIconCallBack(CFRunLoopTimerRef timer, void *info)
{
    [(id)info displaySyncProgressIcon];
}

void setUpdateNotificationCallBack(CFNotificationCenterRef center, void *observer,
                                   CFStringRef name, const void *object, CFDictionaryRef userInfo)
{
    // if the notification comes, it is surely true because it is sent by the updater module
    // only when the update is available
    [(id)observer hideSoftwareUpdateMenu:FALSE];
    
}


 
