#!/bin/sh

#
# build the installer programs
#
set -e

CREATEPACKAGE=./createPackage.sh
CUSTOMIZATION_DEFS=customization.inc

$CREATEPACKAGE

source $CUSTOMIZATION_DEFS

# define variables
APP_NAME=$APPNAME
CLIENT_NAME="Mac OS Sync Client"
INSTALLER_SUFFIX="Installer"
PACKAGE_NAME="$APP_NAME $CLIENT_NAME $INSTALLER_SUFFIX.mpkg"
DMG_NAME="$APP_NAME $CLIENT_NAME.dmg"
PACKAGE_MAKER_DIR="package-maker"
POSTINSTALL_SCRIPT="$PACKAGE_MAKER_DIR/presetup/postsetupScript.sh"
RELEASE_PROPERTIES_FILE=release.properties
RELEASE_DMG_SUFFIX="$APP_PREFIX_NAME-macos-sync-client"

# it is the dir where to put the file to go in the image. When mount the dmg, the shown name is $TMP_FILE_NAME
TMP_FILE_NAME="$APPNAME $CLIENT_NAME"

if [ -e "$DMG_NAME" ]; then 
    rm "./$DMG_NAME"
fi

cp  ./distribution.properties     "../$PACKAGE_MAKER_DIR/$CLIENT_NAME/distribution.properties"

/Developer/Applications/Utilities/PackageMaker.app/Contents/MacOS/PackageMaker --id "com.$APPNAME" --title "$APP_NAME $CLIENT_NAME" --no-relocate --doc "../$PACKAGE_MAKER_DIR/Installer.pmdoc" --out "../$PACKAGE_MAKER_DIR/$PACKAGE_NAME"

# copy the installer and uninstaller packages into the dir to be packaged
cp -R "../$PACKAGE_MAKER_DIR/$PACKAGE_NAME"          "../$TMP_FILE_NAME/$PACKAGE_NAME"
if [ $POWEREDBY == 0 ]; then
    cp -R "../$PACKAGE_MAKER_DIR/presetup/LICENSE.rtf"   "../$TMP_FILE_NAME/LICENSE.rtf"
fi

hdiutil create -megabytes 5 -verbose -srcdir "../$TMP_FILE_NAME"  -format UDRO  "$DMG_NAME"

# get release numbers from ant properties file
eval release=$(awk 'BEGIN { FS="=" } { if ($1 ~ /release.major/) { major=$2; } else if ($1 ~ /release.minor/) { minor=$2 } else if ($1 ~ /build.number/) { build=$2 } }; END { printf("(%d %d %d)", major, minor, build); } ' $RELEASE_PROPERTIES_FILE)

mv "$DMG_NAME" "../output/$RELEASE_DMG_SUFFIX-${release[0]}.${release[1]}.${release[2]}.dmg"

exit 0
