    #!/bin/sh

# generate an include file where are all the 
# sync client applications names are defined
# from customization.h

set -e

CUSTOMIZATION_DEFS=customization.inc
CUSTOMIZATION_FILE=../../common/include/customization.h
CUSTOMIZATION_TOOL=./customization.awk

if [ -f $CUSTOMIZATION_DEFS ]; then
    rm -f $CUSTOMIZATION_DEFS

    if [ $? -gt 0 ]; then
        echo >&2 "error removing customization settings file"
        exit 1
    fi
fi

# exec script to mangle customization.h
$CUSTOMIZATION_TOOL $CUSTOMIZATION_FILE > $CUSTOMIZATION_DEFS

# some sanity checks...
if [ $? -gt 0 ]; then
    echo >&2 "error precessing customization file: include file preprocessing script failed"
    
    exit 1
fi

if [ ! -s $CUSTOMIZATION_DEFS ]; then
    echo >&2 "error precessing customization file: customization file hasn't been generated"

    exit 1
fi

# source in file
source $CUSTOMIZATION_DEFS 

# create package info file for build
(
cat << EOF 
package.client_name=$APP_PREFIX_NAME-macos-sync-client
package.dmg_name=$APPNAME\ Mac\ OS\ Sync\ Client
package.zip_name=\${package.client_name}
EOF
) > package.properties 

# check if all the needed variables are actually defined
if [ -z "$APPNAME" -o -z "$SYNC_APPLICATION_NAME" -o -z "$MENUBAR_APPLICATION_NAME" \
     -o -z "$LAUNCHER_APPLICATION_NAME" -o -z "$PREFERENCE_APPLICATION_NAME"  ]; then
    echo >&2 "error precessing customization file"
    exit 1
fi

cd ..

# define variables
APP_NAME=$APPNAME
CLIENT_NAME="Mac OS Sync Client"
INSTALLER_SUFFIX="Installer"
PACKAGE_NAME="$APP_NAME $CLIENT_NAME $INSTALLER_SUFFIX.mpkg"
DMG_NAME="$APP_NAME $CLIENT_NAME.dmg"
PACKAGE_MAKER_DIR="package-maker"
POSTINSTALL_SCRIPT="$PACKAGE_MAKER_DIR/presetup/postsetupScript.sh"

# it is the dir where to put the file to go in the image. When mount the dmg, the shown name is $TMP_FILE_NAME
TMP_FILE_NAME="$APP_NAME $CLIENT_NAME"

# remove previous dir and package if exists
rm -rf "$TMP_FILE_NAME"
#rm "$DMG_NAME"
rm -rf "$PACKAGE_MAKER_DIR/$CLIENT_NAME"
#rm -rf output
rm -rf "$PACKAGE_MAKER_DIR/$PACKAGE_NAME"

# Create the directory where all the files will be prepared
mkdir "$TMP_FILE_NAME"
# Create dir where to put the output file
#mkdir output

#
# Copy all file needed in a dir. Other never changed files are in the presetup dir
#
mkdir "$PACKAGE_MAKER_DIR/$CLIENT_NAME"
cp $PACKAGE_MAKER_DIR/presetup/presetupScript.sh                                  "$PACKAGE_MAKER_DIR/$CLIENT_NAME/"
cp $PACKAGE_MAKER_DIR/presetup/postsetupScript.sh                                 "$PACKAGE_MAKER_DIR/$CLIENT_NAME/"
cp $PACKAGE_MAKER_DIR/presetup/LICENSE.rtf                                        "$PACKAGE_MAKER_DIR/$CLIENT_NAME/"
cp installer/build/Release/installer                                              "$PACKAGE_MAKER_DIR/$CLIENT_NAME/"
cp installer/build/Release/kickstarter                                            "$PACKAGE_MAKER_DIR/$CLIENT_NAME/"
cp installer/build/Release/launchdJobsController                                  "$PACKAGE_MAKER_DIR/$CLIENT_NAME/"

# copy sync client engine   
cp -R ../sync-engine/build/Release/sync-engine                                      "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$SYNC_APPLICATION_NAME"

# copy menubar application folder and rename its components according to custom names
cp -R ../menubar/build/Release/menubar.app                                          "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$MENUBAR_APPLICATION_NAME.app"
mv "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$MENUBAR_APPLICATION_NAME.app/Contents/MacOS/menubar"    "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$MENUBAR_APPLICATION_NAME.app/Contents/MacOS/$MENUBAR_APPLICATION_NAME"

# copy preferences panel folder and rename its components according to custom names
cp -R ../preferences/build/Release/prefpanel.prefPane                      "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME"
mv "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME/Contents/MacOS/prefpanel"   "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME/Contents/MacOS/$PREFERENCE_APPLICATION_NAME"

# copy preferences launcher folder and rename its components according to custom names
cp -R ../launcher/build/Release/Launcher.app                                 "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$LAUNCHER_APPLICATION_NAME.app"
mv "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$LAUNCHER_APPLICATION_NAME.app/Contents/MacOS/Launcher"  "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$LAUNCHER_APPLICATION_NAME.app/Contents/MacOS/$LAUNCHER_APPLICATION_NAME"

# copy updater application folder and rename its components according to custom names
cp -R ../updater/build/Release/updater.app                                   "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$UPDATER_APPLICATION_NAME.app"
mv "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$UPDATER_APPLICATION_NAME.app/Contents/MacOS/updater"    "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$UPDATER_APPLICATION_NAME.app/Contents/MacOS/$UPDATER_APPLICATION_NAME"

# uninstaller name will be changed runtime by the postInstall script int "Uninstall $APPNAME"
cp -R ./uninstaller/build/Release/Uninstaller.app                            "$PACKAGE_MAKER_DIR/$CLIENT_NAME/Uninstaller.app"

#
# customize info plist
#
# setup preferences panel info file
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME/Contents/Info.plist" -c "Set CFBundleIdentifier $PREFERENCES_PANEL_ID"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME/Contents/Info.plist" -c "Set CFBundleName $PREFERENCE_VISIBLE_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME/Contents/Info.plist" -c "Set NSPrefPaneIconLabel \"$PREFERENCE_VISIBLE_NAME \""
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$PREFERENCE_APPLICATION_NAME/Contents/Info.plist" -c "Set CFBundleExecutable $PREFERENCE_APPLICATION_NAME"

# setup menubar info file
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$MENUBAR_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleName $MENUBAR_APPLICATION_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$MENUBAR_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleExecutable $MENUBAR_APPLICATION_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$MENUBAR_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleIdentifier com.$APPNAME.$MENUBAR_APPLICATION_NAME"

# setup prefpanel launcher info file
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$LAUNCHER_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleName $LAUNCHER_APPLICATION_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$LAUNCHER_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleExecutable $LAUNCHER_APPLICATION_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$LAUNCHER_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleIdentifier com.$APPNAME.app"

# setup updater info file
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$UPDATER_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleName $UPDATER_APPLICATION_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$UPDATER_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleExecutable $UPDATER_APPLICATION_NAME"
/usr/libexec/PlistBuddy "$PACKAGE_MAKER_DIR/$CLIENT_NAME/$UPDATER_APPLICATION_NAME.app/Contents/Info.plist" -c "Set CFBundleIdentifier com.$APPNAME.$UPDATER_APPLICATION_NAME"

exit 0
