#!/bin/sh

#
# create zip archive for updater component
#

set -e

CREATEPACKAGE=./createPackage.sh
CUSTOMIZATION_DEFS=customization.inc

$CREATEPACKAGE

# source in customized build names
source $CUSTOMIZATION_DEFS

PACKAGE_MAKER_DIR="package-maker"
APP_NAME=$APPNAME
CLIENT_NAME="Mac OS Sync Client"
INSTALLER_SUFFIX="Installer"
PACKAGE_MAKER_DIR="package-maker"
POSTINSTALL_SCRIPT="$PACKAGE_MAKER_DIR/presetup/postsetupScript.sh"
UPDATE_ZIP_FILE="$APP_PREFIX_NAME-macos-sync-client.zip"
TMP_FILE_NAME="$CLIENT_NAME"
ARCHIVE_DIR="./funsyncapp"
UPDATE_INFO_FILE=macos-upd-info
RELEASE_PROPERTIES_FILE=release.properties

cp  ./distribution.properties  "../$PACKAGE_MAKER_DIR/$CLIENT_NAME/distribution.properties"

# create a zip archive with client components for updater package
if [ -e $ARCHIVE_DIR ]; then
    rm -rf $ARCHIVE_DIR
fi 

if [ -e $UPDATE_INFO_FILE ]; then
    rm -rf $UPDATE_INFO_FILE
fi 

mv "../$PACKAGE_MAKER_DIR/$CLIENT_NAME" $ARCHIVE_DIR
zip -r "./$UPDATE_ZIP_FILE" $ARCHIVE_DIR
# clean up directory 
rm -rf $ARCHIVE_DIR

# get release numbers from ant properties file
eval release=$(awk 'BEGIN { FS="=" } { if ($1 ~ /release.major/) { major=$2; } else if ($1 ~ /release.minor/) { minor=$2 } else if ($1 ~ /build.number/) { build=$2 } }; END { printf("(%d %d %d)", major, minor, build); } ' $RELEASE_PROPERTIES_FILE)

# create update info file for package
(
cat << EOF 
#File Descriptor
#$(date)

# The latest version available for update
version=${release[0]}.${release[1]}.${release[2]}

# The activation date expressed in the format YYYYMMDD
activation-date=$(date +%Y%m%d)

# The file size expressed in bytes
size=$(stat -f %z $UPDATE_ZIP_FILE)

# The update type: optional, recommended, mandatory
type=optional
EOF
) > $UPDATE_INFO_FILE

# move updater zip archive to output directory
mv "$UPDATE_ZIP_FILE" "../output/$UPDATE_ZIP_FILE"
mv "$UPDATE_INFO_FILE" "../output/$UPDATE_INFO_FILE"

exit 0
