/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <AppKit/AppKit.h>
#include <getopt.h>

#include "utils.h"
#include "constants.h"
#include "customization.h"

#define MACOS_SYSTEM_INSTALLER      "Installer"

int main (int argc, char * const argv[]) 
{
    NSAutoreleasePool* pool = nil;
    NSTask* funambolLauncherApp = nil;
    bool waitForInstaller = false;
    int c = 0, option_index = 0;
    
    struct option long_options[] = {
        {"wait-installer", no_argument, 0, 'w' },
        {0, 0, 0, 0}
    };
    
    while ((c = getopt_long(argc, argv, "w", long_options, &option_index)) != EOF)
    {
        switch (c) {
            case 'w':
                waitForInstaller = true;
                break;
               
            case '?':
            default:
                exit(EXIT_FAILURE);
        }
    }

    pool = [[NSAutoreleasePool alloc] init];

    if (waitForInstaller) {
        if (waitForRunningProcesses(MACOS_SYSTEM_INSTALLER) < 0) {
            NSLog(@"error waiting for Mac OS system installer process\n");
        }
    }

    funambolLauncherApp = [[NSTask alloc] init];
    
    [funambolLauncherApp setLaunchPath: @LAUNCHER_APPLICATION_PATH];
    
    // execute program 
    [funambolLauncherApp launch];
    [funambolLauncherApp dealloc];

    [pool release];

    exit(EXIT_SUCCESS);
}
