/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <AppKit/AppKit.h>

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <pwd.h>

#import "LaunchCtlConfigManager.h"

                    
int main (int argc, char * const argv[]) 
{
    NSAutoreleasePool* pool  = nil;
    int c                    = 0;
    int option_index         = 0;
    const char* userName     = NULL;
    struct passwd *pwd       = NULL;
    bool   loadJobs          = false;
    bool   unloadJobs        = false;

    LaunchCtlConfigManager* launchCtlManager   = nil;
    
    struct option long_options[] = {
        {"user",   required_argument, 0, 'U' },
        {"load",   no_argument,       0, 'l' },
        {"unload", no_argument,       0, 'u' },
        {"reload", no_argument,       0, 'r' },
        {0, 0, 0, 0}
    };
    
    while ((c = getopt_long(argc, argv, "U:ulr", long_options, &option_index)) != EOF) {
        switch (c) {
            case 'U':
                userName = optarg;
                break;

            case 'l':
                loadJobs = true;
                break;

            case 'u':
                unloadJobs = true;
                break;

            case 'r':
                loadJobs = true;
                unloadJobs = true;
                break;
                
            case '?':
            default:
                exit(EXIT_FAILURE);
        }
    }
    
    if ((userName == NULL) || (strlen(userName) == 0)) {
        NSLog(@"invalid username: can't unload launchd jobs\n");
        exit(EXIT_FAILURE);
    }
    
    if ((loadJobs == false) && (unloadJobs == false)) {
        NSLog(@"no actions specified: exiting\n");
        
        exit(EXIT_FAILURE);
    }
    
    if ((pwd = getpwnam(userName)) == NULL) {
        NSLog(@"can't get account info for user \"%s\": %s\n", userName, strerror(errno));
        exit(EXIT_FAILURE);
    }
    
    if (setuid(pwd->pw_uid) < 0) {
        NSLog(@"can't set uid to user \"%s\": %s\n", userName, strerror(errno));
        exit(EXIT_FAILURE);
    }

    pool = [[NSAutoreleasePool alloc] init];
    
    launchCtlManager = [[LaunchCtlConfigManager alloc] init]; 
    if (unloadJobs) {
        [launchCtlManager unloadMenubar];
        [launchCtlManager unloadScheduledSync];
        //[launchCtlManager unloadUpdater];
    }

    if (loadJobs) {
        [launchCtlManager loadMenubar];
        [launchCtlManager loadScheduledSync];
        //[launchCtlManager loadUpdater];
    }

    [launchCtlManager dealloc];
    
    [pool release];

    exit(EXIT_SUCCESS);
}
