/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import "MessageController.h"
#import "customization.h"
#import "constants.h"
#import "LaunchCtlConfigManager.h"

#include "base/adapter/PlatformAdapter.h"

USE_NAMESPACE

@implementation MessageController

- (id)init 
{
    self = [super init];
    
    return self;
}

- (void) awakeFromNib
{    
    NSString * messageString = [[NSString alloc] initWithFormat:NSLocalizedString(@"unistallerMessageText",@"-"), APPNAME];
    
    [messageText setStringValue:messageString];
    [yesButton setTitle:NSLocalizedString(@"yes", @"-")];
    [noButton setTitle:NSLocalizedString(@"no",@"-")];
    NSString * popupTitle = [[NSString alloc] initWithFormat:NSLocalizedString(@"unistallPopup",@"-"), APPNAME];

    [popup setTitle:popupTitle];

    [logoImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
    [NSApp activateIgnoringOtherApps:YES];
    [messageString release];
    [popupTitle release];
}

void removeDir(StringBuffer dir) 
{
    ArrayList files = readFilesInDirRecursive(dir.c_str(), true);
    ArrayList dirs = readDirsInDirRecursive(dir.c_str(), true);
 
    for (int i = 0; i < files.size(); i++) {
        StringBuffer* b = (StringBuffer*)files.get(i);
        remove(b->c_str());
    }
    int maxlen = 0;
    for (int i = 0; i < dirs.size(); i++) {
        StringBuffer* b = (StringBuffer*)dirs.get(i);
        if (maxlen < b->length()) {
            maxlen = b->length();
        }
    }
    do {
        for (int i = 0; i < dirs.size(); i++) {
            StringBuffer* b = (StringBuffer*)dirs.get(i);
            if (maxlen == b->length()) {
                rmdir(b->c_str());
            }
        }
        maxlen--;
    } while (maxlen > 0);
    int ret = rmdir(dir.c_str());
    
    if (ret == -1){
        NSLog(@"error :%s\n\n", strerror(errno));
    }
}


- (int) removeInstalledElement 
{    
    StringBuffer configFolder (PlatformAdapter::getHomeFolder());
    configFolder += "/.config/";
    configFolder += APPNAME;
    removeDir(configFolder);
 
    [self removeLauchdJobs];
    
    return 0;
}

- (void) removeLauchdJobs
{
    // configuration manager for lauchctl
    LaunchCtlConfigManager *lauchCtlJobsManager = [[LaunchCtlConfigManager alloc] init];
    pid_t menubarAppPid = 0;
    
    [lauchCtlJobsManager removeScheduledSync];
    [lauchCtlJobsManager removeMenubar];
    [lauchCtlJobsManager removeUpdater];
    
    // terminate menubar application
    if ((menubarAppPid = checkProcessRunning(MENUBAR_APPLICATION_NAME, 0)) > 0) {
        killProcess(menubarAppPid);
    }
 
    [lauchCtlJobsManager dealloc];
}

- (NSString*) getApplicationName 
{
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}

// This method is called when the user picks a different object to 
// receive messages using the PopUp menu
- (IBAction)firstButton:(NSButton*)sender
{
    OSStatus status;
    AuthorizationFlags authFlags = kAuthorizationFlagDefaults;              
    AuthorizationRef authorizationRef;                                  
    
    status = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, authFlags, &authorizationRef);
    
    if (status != errAuthorizationSuccess) {
        AuthorizationFree(authorizationRef, kAuthorizationFlagDefaults);
        exit(status);
    } else {
        AuthorizationItem authItems = {kAuthorizationRightExecute, 0, NULL, 0};
        AuthorizationRights authRights = {1, &authItems};        
        
        authFlags = kAuthorizationFlagDefaults | kAuthorizationFlagInteractionAllowed |
                    kAuthorizationFlagPreAuthorize | kAuthorizationFlagExtendRights;
       
        status = AuthorizationCopyRights(authorizationRef, &authRights, NULL, authFlags, NULL );
        
        if (status == errAuthorizationSuccess) {
            const char progPath[] = "/bin/rm";
            const char *progArguments[5];
            
            progArguments[0] = "-rf";
            progArguments[1] = CLIENT_SYSTEM_APPLICATION_FOLDER; 
            progArguments[2] = CLIENT_PREFERENCES_FOLDER;
            progArguments[3] = CLIENT_USER_APPLICATION_FOLDER;
            progArguments[4] = NULL;
            
            // remove system directories with appropriate privileges
            status = AuthorizationExecuteWithPrivileges(authorizationRef, progPath, kAuthorizationFlagDefaults, (char **)progArguments, NULL);
            
            if (status == errAuthorizationSuccess) {
                int ret = [self removeInstalledElement];
                
                NSString * title   = [[NSString alloc] initWithFormat:NSLocalizedString(@"unistallPopup",@"-"), APPNAME];
                NSString * message = [[NSString alloc] initWithFormat:NSLocalizedString(@"unistallOK",@"-"),    APPNAME];
                
                if (ret == 0) {
                    NSRunInformationalAlertPanel(title, 
                                                 message,
                                                 NSLocalizedString(@"ok",@"-"), 
                                                 nil,
                                                 nil);
                } else {
                    NSRunInformationalAlertPanel(title, 
                                                 message,
                                                 NSLocalizedString(@"ok",@"-"), 
                                                 nil,
                                                 nil);
                }
                
                [title release];
                [message release];
            } else {
                AuthorizationFree(authorizationRef, kAuthorizationFlagDefaults);
                exit(status);
            }
        }
    }
   
    AuthorizationFree(authorizationRef, kAuthorizationFlagDefaults);
    
    exit(EXIT_SUCCESS);
}

- (IBAction)secondButton:(NSButton*)sender
{
    exit(EXIT_SUCCESS);    
}
@end
