/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import "funambol_preferencesPref.h"

#include "base/adapter/PlatformAdapter.h"
#include "customization.h"
#include "base/util/StringBuffer.h"
#include "ClientConfig.h"
#include "constants.h" // costants and defines for Funambol sync clien and UI components
#include "messageUtils.h"


USE_NAMESPACE


@implementation funambol_preferencesPref

- (void) mainViewDidLoad
{
        syncConfigInProcess = NO;
}

- (NSPreferencePaneUnselectReply) shouldUnselect{
    
    if(CUSTOM_CHECK_CREDENTIALS_ON_EXIT){
        NSBundle* b = [NSBundle bundleForClass:[self class]];
        if ([[usernameField stringValue] isEqual:@""]  || [[passwordField stringValue] isEqual:@""] || [[serverField stringValue] isEqual:@""]){
            [emptyCredButton setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
            [emptyCredMessageText setStringValue:NSLocalizedStringFromTableInBundle(@"EmptyMessage",nil,b, @"-")];
            [NSApp beginSheet:emptyCredWindow modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
            ClientConfig::getInstance()->getAccessConfig().setUsername([[usernameField stringValue] UTF8String]);
            ClientConfig::getInstance()->getAccessConfig().setPassword([[passwordField stringValue] UTF8String]);
            ClientConfig::getInstance()->getAccessConfig().setSyncURL([[serverField stringValue] UTF8String]);
            ClientConfig::getInstance()->safeSave();
            return NSUnselectLater;
        }
    
        if( strcmp(ClientConfig::getInstance()->getAccessConfig().getUsername(), [[usernameField stringValue] UTF8String]) != 0 ||
           strcmp(ClientConfig::getInstance()->getAccessConfig().getPassword(),[[passwordField stringValue] UTF8String] ) != 0 ||
           strcmp(ClientConfig::getInstance()->getAccessConfig().getSyncURL(),[[serverField stringValue] UTF8String] ) != 0 ){
        
            //actionsheet
            [okButton setTitle:NSLocalizedStringFromTableInBundle(@"save",nil,b, @"-")];
            [cancelButton setTitle:NSLocalizedStringFromTableInBundle(@"Cancel",nil,b, @"-")];
            [messageText setStringValue:NSLocalizedStringFromTableInBundle(@"CredentialMessage",nil,b, @"-")];
        
            [NSApp beginSheet:modalWindow modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
            return NSUnselectLater;
        }
    }else{
        //ClientConfig::getInstance()->getAccessConfig().setUsername([[usernameField stringValue] UTF8String]);
        //ClientConfig::getInstance()->getAccessConfig().setPassword([[passwordField stringValue] UTF8String]);
        //ClientConfig::getInstance()->getAccessConfig().setSyncURL([[serverField stringValue] UTF8String]);
        ClientConfig::getInstance()->safeSave();
    }

    return NSUnselectNow;
}

- (void)modalSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo{
    [sheet orderOut:self];
}

- (IBAction)cancelButtonAction:(id)sender{
    const char* usernameValue = ClientConfig::getInstance()->getAccessConfig().getUsername();
    [usernameField setStringValue:[NSString stringWithCString:usernameValue encoding:NSUTF8StringEncoding]];
    
    const char* passwordValue = ClientConfig::getInstance()->getAccessConfig().getPassword();
    [passwordField setStringValue:[NSString stringWithCString:passwordValue encoding:NSUTF8StringEncoding]];
    
    const char* serverValue = ClientConfig::getInstance()->getAccessConfig().getSyncURL();
    [serverField setStringValue:[NSString stringWithCString:serverValue encoding:NSUTF8StringEncoding]];
    
    [NSApp endSheet:modalWindow ];
    [self replyToShouldUnselect:YES];
}


- (IBAction)okButtonAction:(id)sender{
    ClientConfig::getInstance()->getAccessConfig().setUsername([[usernameField stringValue] UTF8String]);
    ClientConfig::getInstance()->getAccessConfig().setPassword([[passwordField stringValue] UTF8String]);
    ClientConfig::getInstance()->getAccessConfig().setSyncURL([[serverField stringValue] UTF8String]);
    ClientConfig::getInstance()->safeSave();
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    [NSApp endSheet:modalWindow];    
    [syncingCredentialsText setStringValue:NSLocalizedStringFromTableInBundle(@"SyncingCredentials",nil,b, @"-")];
    [NSApp beginSheet:syncCredential modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
    task = [[NSTask alloc]init];
    
    // register app as an observer for sync client
    [[NSNotificationCenter defaultCenter] addObserver:self  
                                             selector:@selector(checkSyncProcessStatus:)
                                                 name:NSTaskDidTerminateNotification
                                               object: task];
    
    
    NSString* arg = @"-c";
    NSMutableArray* args = [NSMutableArray array];
    [args addObject:arg];
    NSString* path = [[NSString alloc] initWithFormat:@"%s", SYNC_APPLICATION_PATH];
    [task setLaunchPath:path];
    [task setArguments:args];
    [task launch];
    [indicator startAnimation:self];
    syncConfigInProcess = YES;
    NSString* notName = [[NSString alloc] initWithCString:CLIENT_SYNC_ENDED_NOTIFICATION];
    [[NSDistributedNotificationCenter defaultCenter]  addObserver:self  
                                                         selector:@selector(endedSyncFromMenubarCallBack:)
                                                             name:notName
                                                           object: nil];

    
}
-(void)checkSyncProcessStatus:(NSNotification *) aNotification {
    //int status = [[aNotification object] terminationStatus];
    [task release];
    [[NSNotificationCenter defaultCenter]  removeObserver: self];
}

- (IBAction)emptyCredAction:(id)sender{
    [self replyToShouldUnselect:NO];

    [NSApp endSheet:emptyCredWindow ];
    
}

-(void)endedSyncFromMenubarCallBack:(NSNotification *) aNotification 
{
    [NSApp endSheet:syncCredential];
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    
    
    NSNumber* statuscode = [[aNotification userInfo] objectForKey:@"SyncStatusCode"];
    
    switch ([statuscode intValue]) {
        case 0:
            [self replyToShouldUnselect:YES];
            if ( syncConfigInProcess ){
                syncConfigInProcess = NO;
            }
            break;

        default://generic Error!
            if( syncConfigInProcess ){
                [emptyCredButton setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
                [emptyCredMessageText setStringValue:getMessageFromCode([statuscode intValue], b)];
                [NSApp beginSheet:emptyCredWindow modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
                [passwordField setStringValue:@""];
                ClientConfig::getInstance()->getAccessConfig().setPassword("");
                ClientConfig::getInstance()->safeSave();
                [self replyToShouldUnselect:NO];
                syncConfigInProcess = NO;
            }
            break;
    }
}
@end
