/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// we use #import to make sure the header is only read in once

#import "AboutController.h"
#include "base/adapter/PlatformAdapter.h"
#include "customization.h"
#include "base/util/StringBuffer.h"
#include "ClientConfig.h"
#include "updater/UpdaterConfig.h"
#import "constants.h"

USE_NAMESPACE

@implementation AboutController

- (id)init {
    self = [super init];    
    return self;
}

- (void) awakeFromNib{
    
    
    if(!(ClientConfig::getInstance()->getIsInitialized())){
        ClientConfig::getInstance()->init();
        ClientConfig::getInstance()->read();
    }
    
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    NSString *path = [b pathForImageResource:@"logoImage.tiff"];
    NSImage* logo = [[[NSImage alloc] initWithContentsOfFile:path] autorelease];
    [funambolLogo setImage:logo];
    
    NSString* title = [[NSString alloc] initWithFormat:NSLocalizedStringFromTableInBundle(@"aboutTitle",nil,b, @"-"), APPNAME];
    [funambolTitle setStringValue:title];
    [funambolTitle setFont:[NSFont boldSystemFontOfSize:14]];
    [title release];
    
    
    StringBuffer ver = ClientConfig::getInstance()->getDeviceConfig().getSwv();
    NSString* version = [[NSString alloc] initWithFormat:NSLocalizedStringFromTableInBundle(@"aboutVersion",nil,b, @"-"), ver.c_str()];
    [funambolVersion setStringValue:version];
    [funambolVersionToUpgrade setStringValue:version];
    [version release];
    
    NSString* website = [[NSString alloc] initWithCString:WEBSITE];
    [funambolWebSite setStringValue:website];
    [website release];
    
    NSString* copyright = [[NSString alloc] initWithFormat:NSLocalizedStringFromTableInBundle(@"aboutCopyright",nil,b, @"-"), CORPORATION];
    [funambolCopyright setStringValue:copyright];
    [copyright release];
    
   // NSTextView* text = [[NSTextView alloc] init];
    //[text setString: NSLocalizedStringFromTableInBundle(@"aboutLicense",nil,b, @"-")];
    
    //[funambolMessage setString:NSLocalizedStringFromTableInBundle(@"aboutLicense",nil,b, @"-")];
    
    NSString* license = [[NSString alloc] initWithString:NSLocalizedStringFromTableInBundle(@"aboutLicense",nil,b, @"-")];
    [funambolLicense setStringValue:license];
    [funambolLicense setFont: [NSFont systemFontOfSize: 10]];

    [license release];
   
    
    if (POWEREDBY) {
        [funambolLicense setHidden:YES];
        [poweredbyImage setHidden:NO];
    }else {
        [poweredbyImage setHidden:YES];
        [funambolLicense setHidden:NO];
    }

    if ([self isAnUpdateAvailable]) {
        [softwareUpdate setTitle:NSLocalizedStringFromTableInBundle(@"softwareUpdate",nil,b, @"-")];
        [softwareUpdate setHidden:FALSE];
        [softwareUpdate setEnabled:TRUE];
        [softwareUpdate setFont:[NSFont systemFontOfSize: 10]];
        [funambolVersionToUpgrade setHidden:FALSE];
        [funambolVersion setHidden:TRUE];
    } else {
        [softwareUpdate setHidden:TRUE];
        [softwareUpdate setEnabled:FALSE];
        [funambolVersionToUpgrade setHidden:TRUE];
        [funambolVersion setHidden:FALSE];
    }
    
    // get distribuited NSCenter singleton
    NSDistributedNotificationCenter* nCenter = [NSDistributedNotificationCenter defaultCenter];
    
    [nCenter addObserver:self  
                selector:@selector(setSoftwareUpdateCallBack:)
                    name:@UPDATE_SOFTWARE_NOTIFICATION
                  object:nil];
    
    /*NSString* messageText = [[NSString alloc] initWithFormat:NSLocalizedStringFromTableInBundle(@"prefMessage",nil,b, @"-"), APPNAME];
    [funambolMessage setStringValue:messageText];
    
    [messageText release];   
    [syncTab setLabel:NSLocalizedStringFromTableInBundle(@"syncTabText",nil,b, @"-")];
    [accountTab setLabel:NSLocalizedStringFromTableInBundle(@"accountTabText",nil,b, @"-")];
    [advancedTab setLabel:NSLocalizedStringFromTableInBundle(@"advancedTabText",nil,b, @"-")];
    [resetTab setLabel:NSLocalizedStringFromTableInBundle(@"resetTabText", nil,b, @"-")];
    
    NSString *path = [b pathForImageResource:@"logoImage.tiff"];
    
    NSImage* logo = [[[NSImage alloc] initWithContentsOfFile:path] autorelease];
    [funambolLogo setImage:logo];
    //[logo release];
    */
}

- (NSString*) getApplicationName {
    
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}

-(IBAction) startSoftwareUpdate : (id) sender 
{
    NSLog(@"starting software udpate from About screen...");
	NSString *sync_contacts_opt = @"force_update";
    NSTask* syncTask = [[NSTask alloc] init];
    
    NSMutableArray *args = [NSMutableArray array];
    [args addObject: sync_contacts_opt];
    [syncTask setLaunchPath: @UPDATER_APPLICATION_PATH];
    [syncTask setArguments:args];
    // execute program 
    [syncTask launch];
    
    if ([syncTask isRunning]) {
        NSLog(@"starting software udpate: syncTask running");
        //return TRUE;
    } else {
        NSLog(@"starting software udpate: syncTask not running...");
        //return FALSE;
    }
    
}

- (BOOL) isAnUpdateAvailable
{
    NSLog(@"isAnUpdateAvailable");
    BOOL ret = FALSE;
    StringBuffer appuri(PlatformAdapter::getAppContext());
    UpdaterConfig* upconfig = new UpdaterConfig(appuri.c_str());
    upconfig->read();
    
    NSLog(@"isAnUpdateAvailable LATER %s", upconfig->getLater().c_str());
    NSLog(@"isAnUpdateAvailable SKIPPED %s", upconfig->getSkipped().c_str());
    NSLog(@"isAnUpdateAvailable NOW %s", upconfig->getNow().c_str());
    if (upconfig->getLater().empty() == false && upconfig->getLater() != "0") {
        ret = TRUE;
    } else if (upconfig->getSkipped().empty() == false && upconfig->getSkipped() != "0") {
        ret = TRUE;
    } else if (upconfig->getNow().empty() == false && upconfig->getNow() != "0") {
        ret = TRUE;
    } 
    delete upconfig;
    return ret;    
    
}


-(void)setSoftwareUpdateCallBack:(NSNotification *) aNotification 
{
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    [softwareUpdate setTitle:NSLocalizedStringFromTableInBundle(@"softwareUpdate",nil,b, @"-")];
    [softwareUpdate setHidden:FALSE];
    [softwareUpdate setEnabled:TRUE];
    [softwareUpdate setFont:[NSFont systemFontOfSize: 10]];
    [funambolVersionToUpgrade setHidden:FALSE];
    [funambolVersion setHidden:TRUE];

}

@end