/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// we use #import to make sure the header is only read in once

#import "AdvancedController.h"
#import "customization.h"
#import "constants.h"
#import "ClientConfig.h"
#include "base/adapter/PlatformAdapter.h"
#import "base/Log.h"
#include "utils.h"
#include "SyncQueueManager.h"

@implementation AdvancedController

- (id)init {
    self = [super init];    
    return self;
}

- (void) awakeFromNib
{
    if(!(ClientConfig::getInstance()->getIsInitialized())){

        ClientConfig::getInstance()->init();
        ClientConfig::getInstance()->read();
    }

    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    [troubleshooting setStringValue:NSLocalizedStringFromTableInBundle(@"troubleshootingText",nil,b, @"-")];
    [troubleshooting setFont:[NSFont boldSystemFontOfSize:14]];
    [troubleshootingDescription setStringValue:NSLocalizedStringFromTableInBundle(@"troubleshootingDescription",nil,b, @"-")];
    [loggingLevel setStringValue:NSLocalizedStringFromTableInBundle(@"loggingLevel",nil,b, @"-")];
    [viewLogButton setTitle:NSLocalizedStringFromTableInBundle(@"viewLogText",nil,b, @"-")];
    [contactSyncConfiguration setStringValue:NSLocalizedStringFromTableInBundle(@"contactSyncConfigurationText",nil,b, @"-")];
    [remoteNameLabel setStringValue:NSLocalizedStringFromTableInBundle(@"remoteNameText",nil,b, @"-")];
    
    NSString* logError = NSLocalizedStringFromTableInBundle(@"logError",nil,b, @"-");
    NSString* logInfo = NSLocalizedStringFromTableInBundle(@"logInfo",nil,b, @"-");
    NSString* logDebug = NSLocalizedStringFromTableInBundle(@"logDebug",nil,b, @"-");
    
    [loggingLevelBox addItemWithTitle:logError];
    [loggingLevelBox addItemWithTitle:logInfo];
    [loggingLevelBox addItemWithTitle:logDebug];
    
    [logError release];
    [logInfo release];
    [logDebug release];
    
    LogLevel level = ClientConfig::getInstance()->getDeviceConfig().getLogLevel();
    if (level == LOG_LEVEL_NONE){
        [loggingLevelBox selectItemAtIndex:0];
    }else if (level == LOG_LEVEL_INFO){
        [loggingLevelBox selectItemAtIndex:1];
    }else{ //LOG_LEVEL_DEBUG
        [loggingLevelBox selectItemAtIndex:2];
    }
    
    if (SyncQueueManager::isSyncRunning()) {
        [loggingLevelBox setEnabled: NO];
    }
    
    // get distribuited NSCenter singleton
    nCenter = [NSDistributedNotificationCenter defaultCenter];
    
    [nCenter addObserver:self  
                selector:@selector(syncStartedCallBack:)
                    name:@CLIENT_SYNC_START_NOTIFICATION
                  object:nil];
    
    [nCenter addObserver:self  
                selector:@selector(syncCancelledCallBack:)
                    name:@SYNC_CANCELLED_NOTIFICATION
                  object:nil];    
    
    [nCenter addObserver:self  
                selector:@selector(syncEndedCallBack:)
                    name:@CLIENT_SYNC_ENDED_NOTIFICATION
                  object:nil];

}

-(void)syncStartedCallBack:(NSNotification *) aNotification 
{
    [loggingLevelBox setEnabled: NO];
}

-(void) syncCancelledCallBack:(NSNotification *) aNotification
{
    [loggingLevelBox setEnabled: NO];
}

-(void)syncEndedCallBack:(NSNotification *) aNotification 
{
    [loggingLevelBox setEnabled: YES];
}

- (NSString*) getApplicationName {
    
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}

- (IBAction)setValueComboBox:(id)fId{
    int i = [fId indexOfSelectedItem];
    switch (i) {
        case 0:
            ClientConfig::getInstance()->getDeviceConfig().setLogLevel(LOG_LEVEL_NONE);
            LOG.setLevel(LOG_LEVEL_NONE);
            break;
        case 1:
            ClientConfig::getInstance()->getDeviceConfig().setLogLevel(LOG_LEVEL_INFO);
            LOG.setLevel(LOG_LEVEL_INFO);
            break;
        case 2:
            ClientConfig::getInstance()->getDeviceConfig().setLogLevel(LOG_LEVEL_DEBUG);
            LOG.setLevel(LOG_LEVEL_DEBUG);
            break;
        default:
            break;
    }
    
    ClientConfig::getInstance()->safeSave();
}

- (IBAction)viewLogButtonAction:(id) pId
{
    NSBundle* bundle = [NSBundle bundleForClass:[self class]];
    
    if ([logViewTask isRunning]) {
        [[NSNotificationCenter defaultCenter]  removeObserver: self];
        [logViewTask terminate];
        [logViewTask release];
        logViewTask = nil;
        [viewLogButton setTitle:NSLocalizedStringFromTableInBundle(@"viewLogText",nil, bundle, @"-")];
    } else {
        NSString* arg = [[NSString alloc] initWithFormat:@"%s/%s", PlatformAdapter::getConfigFolder().c_str(), LOG_FILE_PATH];
        NSMutableArray* args = [NSMutableArray array];
        [args addObject:arg];

        logViewTask = [[NSTask alloc]init];
        [logViewTask setLaunchPath: @LOG_VIEWER_APPLICATION_PATH];
        [logViewTask setArguments:args];
        [logViewTask launch];

        [viewLogButton setTitle:NSLocalizedStringFromTableInBundle(@"closeLogText",nil, bundle, @"-")];
        [[NSNotificationCenter defaultCenter] addObserver:self  
                                              selector:@selector(logViewerApplicationExited:)
                                              name:NSTaskDidTerminateNotification
                                              object: logViewTask];

        [arg release];
    }
 }

-(void) logViewerApplicationExited:(NSNotification *) aNotification 
{   
    NSBundle* bundle = [NSBundle bundleForClass:[self class]];
    
    [logViewTask release];
    logViewTask = nil;
    [[NSNotificationCenter defaultCenter]  removeObserver: self];
    [viewLogButton setTitle:NSLocalizedStringFromTableInBundle(@"viewLogText",nil, bundle, @"-")];
}

@end
