/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// we use #import to make sure the header is only read in once

#import "PreferencesController.h"
#include "base/adapter/PlatformAdapter.h"
#include "customization.h"
#include "base/util/StringBuffer.h"
#include "ClientConfig.h"
#include "utils.h"
#include "SyncQueueManager.h"
#import  "LaunchCtlConfigManager.h"

USE_NAMESPACE

@implementation PreferencesController

- (id)init {
    self = [super init];    
    return self;
}

- (void) awakeFromNib
{
    if (!(ClientConfig::getInstance()->getIsInitialized())) {
        ClientConfig::getInstance()->init();
        ClientConfig::getInstance()->read();
    }
    
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    NSString* messageText = [[NSString alloc] initWithFormat:NSLocalizedStringFromTableInBundle(@"prefMessage",nil,b, @"-"), APPNAME];
    [funambolMessage setStringValue:messageText];
    
    [messageText release];   
    [syncTab setLabel:NSLocalizedStringFromTableInBundle(@"syncTabText",nil,b, @"-")];
    [accountTab setLabel:NSLocalizedStringFromTableInBundle(@"accountTabText",nil,b, @"-")];
    [advancedTab setLabel:NSLocalizedStringFromTableInBundle(@"advancedTabText",nil,b, @"-")];
    [resetTab setLabel:NSLocalizedStringFromTableInBundle(@"resetTabText", nil,b, @"-")];
    [aboutTab setLabel:NSLocalizedStringFromTableInBundle(@"aboutTabText",nil,b, @"-")];
    
    NSString *path = [b pathForImageResource:@"logoImage.tiff"];
    NSImage* logo = [[[NSImage alloc] initWithContentsOfFile:path] autorelease];
    [funambolLogo setImage:logo];
    
    bool credValid = ClientConfig::getInstance()->areCredentialValid();
    
    if (SyncQueueManager::isSyncRunning() == false) {
        if (credValid) {
            [tabView selectTabViewItem:syncTab];
        } else {
            [tabView selectTabViewItem:accountTab];
        }
    } else if (SyncQueueManager::isContactSyncRunning()) {
        [tabView selectTabViewItem:syncTab];
    } else if (SyncQueueManager::isContactResetRunning()) {
        [tabView selectTabViewItem:advancedTab];
    } 
    
    // if an instance of menubar application is already 
    // running we assume that the menubar launchd job 
    // was successfully setup previously 
    if (checkProcessRunning(MENUBAR_APPLICATION_NAME, 0) == 0) {
        // set up launchctl configuration manager for menubar application
        LaunchCtlConfigManager* menubarAppManager = [[LaunchCtlConfigManager alloc] init];

        [menubarAppManager setMenuBarAgent];
        [menubarAppManager setUpdaterAgent];
        [menubarAppManager dealloc];
    }
}

-(void)startSyncCallBack:(NSNotification *) aNotification {
    [tabView selectTabViewItem:syncTab];
}

- (NSString*) getApplicationName {
    
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}
@end