/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// we use #import to make sure the header is only read in once

#import "ResetController.h"
#import "customization.h"
#include "constants.h" // costants and defines for Funambol sync clien and UI components
#include "ClientConfig.h"
#include "utils.h"
#include "SyncQueueManager.h"

@implementation ResetController

- (id)init {
    self = [super init];    
    emptyReset = NO;
    resetRunning = NO;
    return self;
}

- (void) awakeFromNib{
    if(!(ClientConfig::getInstance()->getIsInitialized())){

        ClientConfig::getInstance()->init();
        ClientConfig::getInstance()->read();
    }
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    [reset setStringValue:NSLocalizedStringFromTableInBundle(@"resetTabText",nil,b, @"-")];
    [reset setFont:[NSFont boldSystemFontOfSize:14]];
    [descriptionBig setStringValue:NSLocalizedStringFromTableInBundle(@"resetDescription1",nil,b, @"-")];
    [descriptionSmall setStringValue:NSLocalizedStringFromTableInBundle(@"resetDescription2",nil,b, @"-")];
    [descriptionSmall setFont:[NSFont systemFontOfSize:11]];
    
    if (SyncQueueManager::isContactResetRunning()) {
        [resetButton setTitle:NSLocalizedStringFromTableInBundle(@"cancelReset",nil,b, @"-")];
        resetRunning = YES;
    } else {
        [resetButton setTitle:NSLocalizedStringFromTableInBundle(@"resetButtonText",nil,b, @"-")];
        resetRunning = NO;
    }
    
    nCenter = [NSDistributedNotificationCenter defaultCenter];
    
    [nCenter addObserver:self  
                selector:@selector(syncStartedCallBack:)
                    name:@CLIENT_SYNC_START_NOTIFICATION
                  object:nil];

    [nCenter addObserver:self  
                selector:@selector(syncCancelledCallBack:)
                    name:@SYNC_CANCELLED_NOTIFICATION
                  object:nil];    
    
    [nCenter addObserver:self  
                selector:@selector(syncEndedCallBack:)
                    name:@CLIENT_SYNC_ENDED_NOTIFICATION
                  object:nil];
}

- (NSString*) getApplicationName {
    
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}

-(void)checkSyncProcessStatus:(NSNotification *) aNotification {
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    [resettask release];
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    [resetButton setTitle:NSLocalizedStringFromTableInBundle(@"resetButtonText",nil,b, @"-")];
}

- (IBAction)resetAddressBookAction:(id)pId{
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    if(ClientConfig::getInstance()->areCredentialSaved()){
        //
        // Warning to be cleared. it should be possible to read the whole config to get the values
        // 
        StringBuffer prop = ClientConfig::getInstance()->readSyncSourceLastTimestamp(CONTACTS_SOURCE_NAME);
    
        if ([[resetButton title]isEqual:NSLocalizedStringFromTableInBundle(@"resetButtonText",nil,b, @"-")]){
            if (prop.empty() || prop == "0"){
                [okWindow setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
                [cancelWindow setTitle:NSLocalizedStringFromTableInBundle(@"Cancel",nil,b, @"-")];
                [messageText setStringValue:NSLocalizedStringFromTableInBundle(@"emptyReset",nil,b, @"-")];

                [NSApp beginSheet:resetWindow modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
            } else {
                [okWindow setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
                [cancelWindow setTitle:NSLocalizedStringFromTableInBundle(@"Cancel",nil,b, @"-")];
                [messageText setStringValue:NSLocalizedStringFromTableInBundle(@"performReset",nil,b, @"-")];
            
                [NSApp beginSheet:resetWindow modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
                //    [self startReset];
            }
        
        } else if ([[resetButton title]isEqual:NSLocalizedStringFromTableInBundle(@"cancellingAddressbook",nil,b, @"-")]){
        } else {
            //cancel sync
            [nCenter postNotificationName: (NSString*)CFSTR(SYNC_CANCELLED_NOTIFICATION) object:nil userInfo: nil deliverImmediately: YES];
            [resetButton setEnabled: NO];
        }
    }else{
        [emptyText setStringValue:NSLocalizedStringFromTableInBundle(@"EmptyMessage",nil,b, @"-")];
        [emptyOk setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
        [NSApp beginSheet:emptyWarning modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
    }
}

- (IBAction)emptyOkAction:(id)pId{
    [NSApp endSheet:emptyWarning ];
    [tabViewTemp selectTabViewItem:accountTab];
}

- (void)modalSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo{
    [sheet orderOut:self];
}

- (void) startReset
{    
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    [resetButton setTitle:NSLocalizedStringFromTableInBundle(@"cancellingAddressbook",nil,b, @"-")];
    // disable button action while resetting addressbook
    [resetButton setEnabled: NO];
    
    resettask = [[NSTask alloc]init];
   
    // register app as an observer for sync client
    [[NSNotificationCenter defaultCenter] addObserver:self  
                                             selector:@selector(checkSyncProcessStatus:)
                                                 name:NSTaskDidTerminateNotification
                                               object:resettask];

    NSString* arg = @"-A";
    NSMutableArray* args = [NSMutableArray array];
    [args addObject:arg];
    NSString* path = [[NSString alloc] initWithFormat:@"%s", SYNC_APPLICATION_PATH];
    [resettask setLaunchPath:path];
    [resettask setArguments:args];
    [resettask launch];
    resetRunning = YES;
    [NSApp endSheet:resetWindow ];
}

- (IBAction)resetOk:(id)pId{
    NSBundle* b = [NSBundle bundleForClass:[self class]];
    
    //
    // Warning to be cleared. it should be possible to read the whole config to get the values
    // 
    StringBuffer prop = ClientConfig::getInstance()->readSyncSourceLastTimestamp(CONTACTS_SOURCE_NAME);
    
    if((prop.empty() || prop == "0") && !emptyReset){
        [NSApp endSheet:resetWindow ];
        [okWindow setTitle:NSLocalizedStringFromTableInBundle(@"ok",nil,b, @"-")];
        [cancelWindow setTitle:NSLocalizedStringFromTableInBundle(@"Cancel",nil,b, @"-")];
        [messageText setStringValue:NSLocalizedStringFromTableInBundle(@"performReset",nil,b, @"-")];
        emptyReset = YES;
        
        [NSApp beginSheet:resetWindow modalForWindow:[tabViewTemp window] modalDelegate:self didEndSelector:@selector(modalSheetDidEnd:returnCode:contextInfo:) contextInfo:nil];
    }else{
        
        [self startReset];
    }
}

- (IBAction)resetCancel:(id)pId{
    [NSApp endSheet:resetWindow ];
}

-(void)syncStartedCallBack:(NSNotification *) aNotification 
{
    if (resetRunning) {
        NSBundle* b = [NSBundle bundleForClass:[self class]];
        [resetButton setTitle:NSLocalizedStringFromTableInBundle(@"cancelReset",nil,b, @"-")];
        // re-enable button action for cancel sync
        [resetButton setEnabled: YES];
    } else { 
        [resetButton setEnabled: NO];
    }
}

-(void) syncCancelledCallBack:(NSNotification *) aNotification
{
    [resetButton setEnabled: NO];
}

-(void)syncEndedCallBack:(NSNotification *) aNotification 
{
    if (resetRunning) {
        NSBundle* b = [NSBundle bundleForClass:[self class]];
        resetRunning = NO;
        [resetButton setTitle:NSLocalizedStringFromTableInBundle(@"resetButtonText",nil,b, @"-")];
    } 
        
    [resetButton setEnabled: YES];
}

@end