/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import <Foundation/Foundation.h>
#import <CoreFoundation/CoreFoundation.h>
#import <CoreFoundation/CFString.h>
#import <AddressBook/AddressBook.h>

#import "PersonDataConverter.h"

#include "stringUtils.h"
#include "spds/spdsutils.h"
#include "base/base64.h"
#include "customization.h"
#include "base/Log.h"


using namespace Funambol;

const CFStringRef kABPhoneFAXLabel     = CFSTR("fax");
const CFStringRef kABPhoneCompanyLabel = CFSTR("company");
const CFStringRef kABPhoneCarLabel     = CFSTR("car");


@implementation PersonDataConverter

- (void)  convertToSyncItem:

                (CFTypeRef)     person
   syncItem:    (SyncItem*)     item
{
    if (version3)
    {
        item->setDataType(TEXT("text/x-vcard"));
    }
    else
    {
        item->setDataType(TEXT("text/vcard"));
    }
     
    StringBuffer vcard;
    [self personToVcard:(ABPersonRef)person vcard:vcard];
    item->setData((void*)vcard.c_str(), vcard.length());
}



- (void) convertFromSyncItem:
                (SyncItem*) item
    native:     (ABPersonRef) person
{     
    StringBuffer temp((char*)item->getData());
    [self vCardToPerson:temp person:person];
}


/**
 * Person Helper Functions
**/

- (void) setPersonProperty:
                (ABPersonRef)   person
    property:   (NSString *)  property
    value:      (NSString *)   value
{
    [self setPersonProperty:person
        property:property
        value:value
        part:nil
        ];
}
    
- (void) setPersonProperty:
                (ABPersonRef)   person
    property:   (NSString *)  property
    value:      (NSString *)   value
    part:       (NSString *)   part // Only used for Multi Prop Values
{
    [self setPersonProperty:person
        property:property
        value:value
        part:part
        label:nil
        indexToModify:0
        ];
}
    
- (void) setPersonProperty:
                (ABPersonRef)   person
    property:   (NSString *)  property
    value:      (NSString *)   value
    label:      (NSString *)   label // Only used for Multi Prop Values
    indexToModify:  (int)     pos
{
    [self setPersonProperty:person
        property:property
        value:value
        part:nil
        label:label
        indexToModify:pos
        ];
}    

- (void) setPersonProperty: (ABPersonRef)   person
         property:   (NSString *)  property
         value:      (NSString *)  value
         part:       (NSString *)  part  // Only used for Multi Prop Values - @"" means ignore
         label:      (NSString *)  label // Only used for Multi Prop Values
         indexToModify:  (int)     pos
{
    /* Assume string type!
    kABPersonFirstNameProperty:          // First name - kABStringPropertyType
    kABPersonLastNameProperty:           // Last name - kABStringPropertyType
    kABPersonMiddleNameProperty:         // Middle name - kABStringPropertyType
    kABPersonPrefixProperty:             // Prefix ("Sir" "Duke" "General") - kABStringPropertyType
    kABPersonSuffixProperty:             // Suffix ("Jr." "Sr." "III") - kABStringPropertyType
    kABPersonNicknameProperty:           // Nickname - kABStringPropertyType
    kABPersonFirstNamePhoneticProperty:  // First name Phonetic - kABStringPropertyType
    kABPersonLastNamePhoneticProperty:   // Last name Phonetic - kABStringPropertyType
    kABPersonMiddleNamePhoneticProperty: // Middle name Phonetic - kABStringPropertyType
    kABPersonOrganizationProperty:       // Company name - kABStringPropertyType
    kABPersonJobTitleProperty:           // Job Title - kABStringPropertyType
    kABPersonDepartmentProperty:         // Department name - kABStringPropertyType
    kABPersonNoteProperty:               // Note - kABStringPropertyType
    */
    
    
    if (!value || !person)
    {
        return;
    }
    
    CFTypeRef propertyValue = nil;
    CFStringRef propertyCFStr = (const CFStringRef)property;
    CFStringRef labelCFStr = (const CFStringRef)label;
    CFStringRef valueCFStr = (const CFStringRef)value;
    
    // kABMultiStringPropertyType
    if (property == kABEmailProperty ||               // Email(s) - kABMultiStringPropertyType
        property == kABURLsProperty   ||              // URL - kABMultiStringPropertyType
        property == kABRelatedNamesProperty ||        // Names - kABMultiStringPropertyType
        property == kABPhoneProperty)                 // Generic phone number - kABMultiStringPropertyType
    { 
        ABMutableMultiValueRef multiValue = (ABMutableMultiValueRef)ABRecordCopyValue(person, propertyCFStr);
        if (multiValue == nil) {
            multiValue = ABMultiValueCreateMutable();
        } else {
            CFRelease(multiValue);
            ABMultiValueRef mv;
            ABMutableMultiValueRef mutableMV;
            
            mv = (ABMultiValueRef)ABRecordCopyValue(person, propertyCFStr);
            multiValue = ABMultiValueCreateMutableCopy(mv);
            
            CFRelease(mv);
        }
        
        CFStringRef aLabel = nil;
        
        int currentPosition = 0;
        int index = 0;
        int total = ABMultiValueCount(multiValue);
        for (; index < total; index++) {
            aLabel = ABMultiValueCopyLabelAtIndex(multiValue, index);
            if (aLabel) {
                if (CFStringCompare(aLabel, labelCFStr, 0) == 0) {
                    if (currentPosition == pos) {
                        CFRelease(aLabel);
                        aLabel = nil;
                        break;
                    } else {
                        currentPosition++;
                    }
                    
                }
                CFRelease(aLabel);
                aLabel = nil;
            }
        }
        
        if (index < total) {
            if (CFStringGetLength(valueCFStr) == 0) {
                ABMultiValueRemove(multiValue, index);
                CFRelease(value);
            }
            else {
                bool ret = ABMultiValueReplaceValue(multiValue, value, index);
                int k = 0;
            }
        }
        else {
            if (CFStringGetLength(valueCFStr) != 0) {
                ABMultiValueAdd(multiValue, value, labelCFStr, nil);
            }
            else {
                CFRelease(value);
            }
        }
        propertyValue = multiValue;
                 
    } else if ((property == kABAddressProperty) ||             // Street Address - kABMultiDictionaryPropertyType
               (property == kABAIMInstantProperty))        // IM Addresses - kABMultiDictionaryPropertyType
    {
        ABMutableMultiValueRef multiValue = (ABMutableMultiValueRef)ABRecordCopyValue(person, propertyCFStr);
        if (multiValue == nil) {
            multiValue = ABMultiValueCreateMutable();
        } else {
            CFRelease(multiValue);
            ABMultiValueRef mv;
            ABMutableMultiValueRef mutableMV;
            
            mv = (ABMultiValueRef)ABRecordCopyValue(person, propertyCFStr);
            multiValue = ABMultiValueCreateMutableCopy(mv);
            
            CFRelease(mv);
        }
        
        CFMutableDictionaryRef indexValue = nil;
        CFStringRef aLabel = nil;
        
        int index;
        int total = ABMultiValueCount(multiValue);
        for (index = 0; index < total; index++)
        {
            aLabel = ABMultiValueCopyLabelAtIndex(multiValue, index);
            if (aLabel)
            {
                if (CFStringCompare(aLabel, labelCFStr, 0) == 0)
                {
                    CFDictionaryRef tempdict = (CFDictionaryRef)ABMultiValueCopyValueAtIndex(multiValue,index);
                    if (!tempdict) {
                        continue;
                    }
                    
                    indexValue = (CFMutableDictionaryRef)CFDictionaryCreateMutableCopy(kCFAllocatorDefault, 0, 
                            tempdict
                        );
                    CFRelease(tempdict);
                    CFRelease(aLabel);
                    aLabel = nil;
                    break;
                }
                
                CFRelease(aLabel);
                aLabel = nil;
            }
        }
        
        if (!indexValue)
        {
            indexValue = CFDictionaryCreateMutable(kCFAllocatorDefault, 0, nil, nil);
        }

        CFDictionarySetValue(indexValue, part, value);

        if (index < total)
        {
            if (CFStringGetLength(valueCFStr) == 0 && CFDictionaryGetCount(indexValue) == 0)
            {
                ABMultiValueRemove(multiValue, index);
            }
            else
            {
                ABMultiValueReplaceValue(multiValue, indexValue, index);
            }
        } else {
            ABMultiValueAdd(multiValue, indexValue, labelCFStr, nil);
        }
        
        if (indexValue) {
            CFRelease(indexValue);
        }
        
        propertyValue = multiValue;
    }
    else
    if (property == kABBirthdayProperty)           // Birthday associated with this person - kABDateTimePropertyType
    {
        if (CFStringGetLength(valueCFStr) == 0)
        {
            CFRelease(value);
            propertyValue = nil;
        }  
        else
        {
            int year, month, day;
            Funambol::StringBuffer sb = CFString2Std((CFStringRef)value);
            const char * tempvalue = sb.c_str();
            CFRelease(value);
            if (!tempvalue || !tempvalue[0]) {
                return;
            }
            if (sscanf(tempvalue, TEXT("%04d-%02d-%02d"), &year, &month, &day) == 3 || 
                sscanf(tempvalue, TEXT("%04d%02d%02d"), &year, &month, &day) == 3)
            {
                CFGregorianDate date;
                date.year   = year;
                date.month  = month;
                date.day    = day;
                date.hour   = 0;
                date.minute = 0;
                date.second = 0;

                /* dates are created with
                 * an absolute time in the current time zone, just like
            
                 */
                CFTimeZoneRef tz = CFTimeZoneCopyDefault();
                CFDateRef cfdate(CFDateCreate(NULL, CFGregorianDateGetAbsoluteTime(date, tz)));
                CFRelease(tz);
                propertyValue = cfdate;
            }    
        }
    }
    else
    {
        if (CFStringGetLength(valueCFStr) > 0)
        {
            propertyValue = value;
        }
        else
        {
            CFRelease(value);
            value = nil;
        }
    }
    

    if (value == nil)
    {
        if (!ABRecordRemoveValue(person, propertyCFStr))
        {
            [self throwError:@"Error removing property"];
        }
    }
    else
    {
        if (propertyValue != nil) {
            if (!ABRecordSetValue(person, propertyCFStr, propertyValue)) {
                if (verbose) {
                    NSMutableString * desc = [[NSMutableString alloc] init];
                    [desc appendString:@"Error setting property "];
                    //[desc appendString: property ];
                    [self throwError: desc];
                } else {
                    [self throwError:@"Error setting property"];
                }
            }
            
            CFRelease(propertyValue);
        }
    }    
}




- (CFStringRef) getPersonProperty:
                (ABPersonRef)   person
    property:   (NSString *)  property
{
    return [self getPersonProperty:person
        property:property
        part:nil
        ];
}
    
- (CFStringRef) getPersonProperty:
                (ABPersonRef)   person
    property:   (NSString *)  property
    part:       (NSString *)   part // Only used for Multi Prop Values
{
    return [self getPersonProperty:person
        property:property
        part:part
        label:nil
        propertyIndex:0    
        ];
}
    
- (CFStringRef) getPersonProperty:
                (ABPersonRef)   person
    property:   (NSString *)  property
    label:      (NSString *)   label // Only used for Multi Prop Values
    propertyIndex: (int) propertyIndex
{
    return [self getPersonProperty:person
        property:property
        part:nil
        label:label
        propertyIndex:propertyIndex
        ];
}    

- (CFStringRef) getPersonProperty: (ABPersonRef)   person
                property:       (NSString *)  property
                part:           (NSString *)  part  // Only used for Multi Prop Values - @"" means ignore
                label:          (NSString *)  label // Only used for Multi Prop Values
                propertyIndex:  (int)         propertyIndex
{
    /* Assume string type!
    kABPersonFirstNameProperty:          // First name - kABStringPropertyType
    kABPersonLastNameProperty:           // Last name - kABStringPropertyType
    kABPersonMiddleNameProperty:         // Middle name - kABStringPropertyType
    kABPersonPrefixProperty:             // Prefix ("Sir" "Duke" "General") - kABStringPropertyType
    kABPersonSuffixProperty:             // Suffix ("Jr." "Sr." "III") - kABStringPropertyType
    kABPersonNicknameProperty:           // Nickname - kABStringPropertyType
    kABPersonFirstNamePhoneticProperty:  // First name Phonetic - kABStringPropertyType
    kABPersonLastNamePhoneticProperty:   // Last name Phonetic - kABStringPropertyType
    kABPersonMiddleNamePhoneticProperty: // Middle name Phonetic - kABStringPropertyType
    kABPersonOrganizationProperty:       // Company name - kABStringPropertyType
    kABPersonJobTitleProperty:           // Job Title - kABStringPropertyType
    kABPersonDepartmentProperty:         // Department name - kABStringPropertyType
    kABPersonNoteProperty:               // Note - kABStringPropertyType
    */
    
    CFTypeRef value;
    CFStringRef propertyCFStr = (const CFStringRef)property;
    CFStringRef labelCFStr = (const CFStringRef)label;
    
    if (property == kABModificationDateProperty)
    {
        CFDateRef cfdate = (CFDateRef)ABRecordCopyValue(person, propertyCFStr);
        if (!cfdate)
            return nil;
        CFTimeZoneRef tz(CFTimeZoneCopyDefault());
        CFGregorianDate date = CFAbsoluteTimeGetGregorianDate(
            CFDateGetAbsoluteTime(
                cfdate
                ),
            tz);

        CFStringRef outStr = CFStringCreateWithFormat(kCFAllocatorDefault, NULL,
            CFSTR("%04d%02d%02d%02d%02d%02.3f"),
            date.year,
            date.month,
            date.day,
            date.hour,
            date.minute,
            date.second);

        CFRelease(cfdate);
        CFRelease(tz);
        
        return outStr;
    }
    else if (property == kABEmailProperty ||          // Email(s) - kABMultiStringPropertyType
        property == kABURLsProperty   ||               // URL - kABMultiStringPropertyType
        property == kABRelatedNamesProperty ||        // Names - kABMultiStringPropertyType
        property == kABPhoneProperty)                 // Generic phone number - kABMultiStringPropertyType
    {
        ABMultiValueRef multiVal = (ABMultiValueRef)ABRecordCopyValue(person, propertyCFStr);
        if (!multiVal) {
            return CFSTR("");
        }
         
        CFStringRef aLabel;
        CFStringRef ret = CFSTR("");
        int propertyLabelsFound = 0;
        
        for (int x = 0; x < ABMultiValueCount(multiVal); x++)
        {
            aLabel = ABMultiValueCopyLabelAtIndex(multiVal, x);
            if (aLabel)
            {
                if (CFStringCompare(aLabel, labelCFStr, 0) == kCFCompareEqualTo) {
                    if (propertyIndex == propertyLabelsFound) {
                        ret = (CFStringRef)ABMultiValueCopyValueAtIndex(multiVal, x);
                        CFRelease(aLabel);
                        break;
                    }
                    propertyLabelsFound++;
                }
                CFRelease(aLabel);
                aLabel = nil;
            }
        }
        //CFRelease(value); //CopyValue
        
        return ret;
    }
    else
    if (property == kABAddressProperty ||             // Street address - kABMultiDictionaryPropertyType
        property == kABAIMInstantProperty)        // Instant Messaging - kABMultiDictionaryPropertyType
    {
        ABMutableMultiValueRef value = (ABMutableMultiValueRef)ABRecordCopyValue(person, propertyCFStr);
    
        if (!value) {
            return nil;
        }
        
        CFDictionaryRef indexValue = nil;
        
        //getting the values of the person in the addressbook for the label given as parameter
        int index;
        int total = ABMultiValueCount(value);
        CFStringRef aLabel;
        for (index = 0; index < total; index++)
        {
            aLabel = ABMultiValueCopyLabelAtIndex(value, index);
            if (aLabel)
            {
                if (CFStringCompare(aLabel, labelCFStr, 0) == 0)
                {
                    indexValue = (CFDictionaryRef)ABMultiValueCopyValueAtIndex(value,index);
                    CFRelease(aLabel);
                    aLabel = nil;
                    break;
                }
                CFRelease(aLabel);
                aLabel = nil;
            }
        }
        
        //CFRelease(value);
        
        if (!indexValue)
        {
            return CFSTR("");
        }

        CFStringRef ret;
        if (!CFDictionaryGetValueIfPresent(indexValue, part, (const void**)(&ret))) {
            CFRelease(indexValue);
            return CFSTR("");
        }
        CFRelease(indexValue);

        ret = CFStringCreateCopy(kCFAllocatorDefault, ret);
        
        return ret;
    }
    else
    if (property == kABBirthdayProperty)           // Birthday associated with this person - kABDateTimePropertyType
    {
        CFDateRef cfdate = (CFDateRef)ABRecordCopyValue(person, propertyCFStr);
        if (!cfdate)
            return CFSTR("");
        CFTimeZoneRef tz(CFTimeZoneCopyDefault());
        CFGregorianDate date = CFAbsoluteTimeGetGregorianDate(CFDateGetAbsoluteTime(cfdate), tz);
        CFStringRef outStr = CFStringCreateWithFormat(kCFAllocatorDefault, NULL, CFSTR("%04d-%02d-%02d"), 
                                                      date.year, date.month, date.day);

        CFRelease(cfdate);
        CFRelease(tz);
        
        return outStr;
    }
    else
    {
        CFStringRef val = (CFStringRef)ABRecordCopyValue(person, propertyCFStr);
        if (!val)
            val = CFSTR("");
        return val;
    }
    
    // Some other unsupported property
    return nil;
}



- (void) setPersonPhoto:
                (ABPersonRef)           person
    value:      (CFDataRef)             data
{
    if (!ABPersonSetImageData(person, data))
    {
        // TODO : Handle error
    }
}
    
- (CFDataRef) getPersonPhoto:
                (ABPersonRef)           person
{
    CFDataRef ret = nil;
    CFDataRef data = ABPersonCopyImageData(person);
    if (data) {
    
    }
    
    return ret;
}

- (void) clearPersonPhoto:
                (ABPersonRef)           person
{
    ABPersonSetImageData(person, nil);
}


- (NSString *) getLabel:(ABPersonRef) person
              property: (VProperty *) property{
    if (property->containsParameter(TEXT("HOME"))) {
        return kABHomeLabel;
    } else if (property->containsParameter(TEXT("WORK"))) {
        return kABWorkLabel;
    } else {
        return kABOtherLabel;
    }
    
}

- (bool) isOtherAddress:(ABPersonRef) person
                property: (VProperty *) property {
    NSString * label = [self getLabel: person property: property];
    
    return (label == kABOtherLabel);
    
}

//
// Sets a Person's address to the values in property.
//
- (void) addressToPerson: (ABPersonRef) person
                property: (VProperty *) property {
    CFStringRef value;
    NSString * label = [self getLabel: person property: property];
    
    StringBuffer address1(property->getPropComponent(3));
    StringBuffer address2(property->getPropComponent(2));
    
    if (value = [self buildAddress:address1 secondLineAddress: address2]) {
        [self setPersonProperty:person  property:kABAddressProperty       value:(NSString *)value     part:kABAddressStreetKey      label:label  indexToModify:0];
        CFRelease(value);
    }
    if (value = Std2CFString(property->getPropComponent(4))) {
        [self setPersonProperty:person  property:kABAddressProperty       value:(NSString *)value     part:kABAddressCityKey        label:label  indexToModify:0];
        CFRelease(value);
    }
    if (value = Std2CFString(property->getPropComponent(5))) {
        [self setPersonProperty:person  property:kABAddressProperty       value:(NSString *)value     part:kABAddressStateKey       label:label  indexToModify:0];
        CFRelease(value);
    }
    
    if (value = Std2CFString(property->getPropComponent(6))) {
        [self setPersonProperty:person  property:kABAddressProperty       value:(NSString *)value    part:kABAddressZIPKey         label:label  indexToModify:0];
        CFRelease(value);
    }

    if (value = Std2CFString(property->getPropComponent(7))) {
        [self setPersonProperty:person  property:kABAddressProperty       value:(NSString *)value     part:kABAddressCountryKey     label:label  indexToModify:0];
        CFRelease(value);
    }
}


//
// Builds a address in the form address1'\n'address2 as required by the AddressBook framework
//
- (CFStringRef) buildAddress: (const char*) address1 secondLineAddress: (const char*) address2 {
    
    StringBuffer fullAddress(address1);
    if (strlen(address2) != 0) {
        fullAddress.append("\n");
        fullAddress.append(address2);
    }
    return Std2CFString(fullAddress);
    
}

//
// Fills the given VProperty vp with the values given in the addressbook record person
//
- (void) personToAddress: (ABPersonRef) person label:(NSString *) label intoVProperty: (VProperty&) vp {
    CFStringRef value;
    // TODO no PO Box Support
    vp.addValue(TEXT(""));
    
    if (value = [self getPersonProperty:person property:kABAddressProperty    part:kABAddressStreetKey      label:label     propertyIndex:0]) {
        StringBuffer address1, address2;
        [self getAddressInto: address1 secondLineAddress: address2 fromValue: value];
        vp.addValue(address2);
        vp.addValue(address1);
        CFRelease(value);
    } else {
        // added 2 values empty because in the previous statement we add 2 in any case. So if th property is not present we have to add
        // 2 otherwise the calculation of the crc fails and it continues to send the client contact again.
        vp.addValue("");
        vp.addValue("");
    }
    if (value = [self getPersonProperty:person property:kABAddressProperty    part:kABAddressCityKey        label:label     propertyIndex:0]) {
        vp.addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp.addValue("");
    }
    if (value = [self getPersonProperty:person property:kABAddressProperty    part:kABAddressStateKey       label:label propertyIndex:0]) {
        vp.addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp.addValue("");
    }
    if (value = [self getPersonProperty:person property:kABAddressProperty    part:kABAddressZIPKey         label:label propertyIndex:0]) {
        vp.addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp.addValue("");
    }
    if (value = [self getPersonProperty:person property:kABAddressProperty    part:kABAddressCountryKey     label:label propertyIndex:0]) {
        vp.addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp.addValue("");
    }
}

//
// Given a a line containing address1'\n'address2, it separates and returns the two lines of the address into
// address1 and address2
//
- (void) getAddressInto: (StringBuffer&) address1 secondLineAddress: (StringBuffer&) address2 fromValue: (CFStringRef) value
{    
    StringBuffer valueBuffer(CFString2Std(value));
    
    int i = 0;
    long len  = valueBuffer.length();
    char prec = 0;
    for (; i<len; ++i) {
        if (valueBuffer[i] == '\n') {
            if (prec != '\r') {
                break;
            }
        }
        
        prec = valueBuffer[i];
    }
    
    address1 = valueBuffer.substr(0, i);
    if (i < len-1) {
        address2 = valueBuffer.substr(i+1);
    } else {
        address2 = "";
    }
}

//
// Adds the person's home address into the given vcard
//
- (void) personToHomeAddress: (ABPersonRef)person intoVCard: (VObject&)vo {
    VProperty vp = VProperty(TEXT("ADR"));
    vp.addParameter(TEXT("HOME"), NULL);
    [self personToAddress:person label:kABHomeLabel intoVProperty:vp];
    vo.addProperty(&vp);
}

//
// Adds the person's other address into the given vcard
//
- (void) personToOtherAddress: (ABPersonRef)person intoVCard: (VObject&)vo {
    VProperty vp = VProperty(TEXT("ADR"));
    [self personToAddress:person label:kABOtherLabel intoVProperty:vp];
    vo.addProperty(&vp);
}

//
// Adds the person's work address into the given vcard
//
- (void) personToWorkAddress: (ABPersonRef)person intoVCard: (VObject&)vo {
    VProperty vp = VProperty(TEXT("ADR"));
    vp.addParameter(TEXT("WORK"), NULL);
    [self personToAddress:person label:kABWorkLabel intoVProperty:vp];
    vo.addProperty(&vp);
}


/** parses vcard and stores result in person */
- (void) vCardToPerson: (StringBuffer&)   vcardString
                person: (ABPersonRef)   person
{     
    VObject * vobj = VConverter::parse(vcardString.c_str());
    if (vobj == NULL) {
        // TODO: remove throw!
        [self throwError: @"Unable to parse vCard string"];
        return;
    }
    
    int  homeTel     = 0;
    int  businessTel = 0;
    bool otherAdr    = false;
    bool otherTel    = false;
    bool otherEmail  = false;
    
    // walk through all properties and handle them
    int propindex = 0;
    VProperty * vprop;
    while ((vprop = vobj->getProperty(propindex)) != NULL) {
        propindex++;
    
        char * n = vprop->getName();
        char * v = vprop->getValue();
        
        if (!n)
            continue;
        if(!v)
            continue;
        
        CFStringRef name  = Std2CFString(n);
        CFStringRef value = Std2CFString(v);
        
        if (!name)
            continue;
        if (!value)
            continue;
   
        NSString *nsValue = (const NSString *)value;
        
        if (CFStringCompare(name, CFSTR("N"), 0) == 0) {
            CFRelease(value);
            
            if (value = Std2CFString(vprop->getPropComponent(1))) {
                [self setPersonProperty:person  property:kABLastNameProperty  value:(NSString *)value];
                //CFRelease(value);
            }
            if (value = Std2CFString(vprop->getPropComponent(2))) {
                [self setPersonProperty:person  property:kABFirstNameProperty     value:(NSString *)value];
                //CFRelease(value);
            }
            if (value = Std2CFString(vprop->getPropComponent(3))) {
                [self setPersonProperty:person  property:kABMiddleNameProperty    value:(NSString *)value];
                //CFRelease(value);
            }
            if (value = Std2CFString(vprop->getPropComponent(4))) {
                [self setPersonProperty:person  property:kABTitleProperty        value:(NSString *)value];
                //CFRelease(value);
            }
            if (value = Std2CFString(vprop->getPropComponent(5))) {
                [self setPersonProperty:person  property:kABSuffixProperty        value:(NSString *)value];
                //CFRelease(value);
            }
        }
        else if(CFStringCompare(name, CFSTR("BDAY"), 0) == 0) {
            [self setPersonProperty:person  property:kABBirthdayProperty      value:nsValue];
        }
        else if(CFStringCompare(name, CFSTR("NOTE"), 0) == 0) {
            [self setPersonProperty:person  property:kABNoteProperty          value:nsValue];
        }
        else if(CFStringCompare(name, CFSTR("CLASS"), 0) == 0) {
            CFRelease(value); // Ignore
        }
        else if(CFStringCompare(name, CFSTR("CATEGORIES"), 0) == 0) {
            CFRelease(value); // Ignore
        }
        else if(CFStringCompare(name, CFSTR("EMAIL"), 0) == 0) {
            // Mapping is:
            // Email1Address <-> EMAIL;INTERNET:
            // Email2Address <-> EMAIL;INTERNET;HOME:
            // Email3Address <-> EMAIL;INTERNET;WORK:
            if (vprop->containsParameter(TEXT("HOME"))) {
                if (vprop->containsParameter(TEXT("X-FUNAMBOL-INSTANTMESSENGER"))) {
                    // TODO IMs require more complex mapping
                    /*
                    */
                    CFRelease(value);
                }
                else { 
                    [self setPersonProperty:person  property:kABEmailProperty         value:nsValue     label:kABHomeLabel indexToModify:0];
                }
            }
            else if (vprop->containsParameter(TEXT("WORK"))) {
                [self setPersonProperty:person  property:kABEmailProperty         value:nsValue     label:kABWorkLabel   indexToModify:0];
            }
            else if (!otherEmail) {
                otherEmail = true;
                [self setPersonProperty:person  property:kABEmailProperty         value:nsValue     label:kABOtherLabel  indexToModify:0];
            }
            else {
            }
        }
        else if(CFStringCompare(name, CFSTR("FN"), 0) == 0) {
            CFRelease(value); // Ignore
        }
        else if(CFStringCompare(name, CFSTR("NICKNAME"), 0) == 0) {
            [self setPersonProperty:person  property:kABNicknameProperty      value:nsValue];
        }
        else if(CFStringCompare(name, CFSTR("PRIORITY"), 0) == 0) {
            CFRelease(value); // Ignore
        }
        else if(CFStringCompare(name, CFSTR("ORG"), 0) == 0) {
            CFRelease(value);
        
            if (value = Std2CFString(vprop->getPropComponent(1))) {
                [self setPersonProperty:person  property:kABOrganizationProperty  value: (NSString *)value];
                //CFRelease(value);
            }
            
            if (value = Std2CFString(vprop->getPropComponent(2))) {
                [self setPersonProperty:person  property:kABDepartmentProperty    value: (NSString *)value];
                //CFRelease(value);
            }
            //TODO Can we save office location?
            /*
            if(value = (NSString *)Std2CFString(vprop->getPropComponent(3)))
                [self setPersonProperty:person  property:    value:value];
            */
        }
        else if(CFStringCompare(name, CFSTR("ROLE"), 0) == 0) {
            CFRelease(value); // Ignore
        }

        //
        // ---- Telephone fields ----
        //
        else if(CFStringCompare(name, CFSTR("TEL"), 0) == 0) {
            if (vprop->containsParameter(TEXT("WORK"))) {
                if (vprop->containsParameter(TEXT("FAX"))) {
                    [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value  label:kABPhoneWorkFAXLabel indexToModify:0];
                }
                else if (vprop->containsParameter(TEXT("PREF"))) {
                    [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value  label: (NSString *)kABPhoneCompanyLabel indexToModify:0];
                }
                else {      // default, would be "else if (vprop->containsParameter("VOICE"))"
                    // it should be only the first 2 phone numbers
                    [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value label:kABWorkLabel  indexToModify:businessTel];
                    businessTel++;
                    CFRelease(value);
                }
            }
            else if (vprop->containsParameter(TEXT("CAR"))) {
                [self setPersonProperty:person  property:kABPhoneProperty  value:nsValue  label: (NSString *)kABPhoneCarLabel indexToModify:0];
            }
            else if(vprop->containsParameter(TEXT("HOME"))) {
                if(vprop->containsParameter(TEXT("FAX"))) {
                    [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value  label:kABPhoneHomeFAXLabel indexToModify:0];
                }
                else {      // default, would be "else if (vprop->containsParameter("VOICE"))"
                    [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value  label:kABHomeLabel indexToModify:homeTel];
                    homeTel++;
                    CFRelease(value);
                }
            }
            else if (vprop->containsParameter(TEXT("CELL"))) {
                [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value     label:kABPhoneMobileLabel indexToModify:0];
            }
            else if (vprop->containsParameter(TEXT("FAX"))) {
                [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value     label: (const NSString *)kABPhoneFAXLabel indexToModify:0];
            }
            else if(vprop->containsParameter(TEXT("PAGER"))) {
                [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value     label:kABPhonePagerLabel indexToModify:0];
            }
            else if (vprop->containsParameter(TEXT("PREF"))) {
                [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value     label:kABPhoneMainLabel indexToModify:0];
            }
            /* Not supported
            else if(vprop->containsParameter("X-FUNAMBOL-CALLBACK")) {
                [self setPersonProperty:person  property:kABPhoneProperty         value:value];
                //setProperty("CallbackTelephoneNumber", element);
            }
            else if(vprop->containsParameter("X-FUNAMBOL-RADIO")) {
                [self setPersonProperty:person  property:kABPhoneProperty         value:value];
                //setProperty("RadioTelephoneNumber", element);
            }
            else if(vprop->containsParameter("X-FUNAMBOL-TELEX")) {
                [self setPersonProperty:person  property:kABPersonPhoneProperty         value:value];
                //setProperty("TelexNumber", element);
            }
            */
            else if (otherTel == false) {      // default, would be "else if (vprop->containsParameter("VOICE"))"
                [self setPersonProperty:person  property:kABPhoneProperty  value:(NSString *)value     label:kABOtherLabel indexToModify:0];
                otherTel = true;
            }
            else {
                CFRelease(value);
            }
        }
        
        else if(CFStringCompare(name, CFSTR("TITLE"), 0) == 0) {
            [self setPersonProperty:person  property:kABJobTitleProperty      value:nsValue];
        }

        else if(CFStringCompare(name, CFSTR("PHOTO"), 0) == 0) {
            CFRelease(value);
            if (strlen(v) != 0)
            {
                char * content = new char[strlen(v)];
                int len = b64_decode(content, v);
                if (getLastErrorCode() != 0)
                {
                    // Handle error
                }
                else
                {
                    CFDataRef val = CFDataCreate(kCFAllocatorDefault, (const UInt8*)content, len);
                    [self setPersonPhoto:person value:val];
                    CFRelease(val);
                }
                delete [] content;
            }
            else
            {
                [self clearPersonPhoto:person];
            }
        }

        //
        // ---- URL fields ----
        //
        else if(CFStringCompare(name, CFSTR("URL"), 0) == 0) {
            if (vprop->containsParameter(TEXT("WORK"))) {
                [self setPersonProperty:person  property:kABURLsProperty           value:nsValue     label:kABWorkLabel indexToModify:0];
            }
            else if(vprop->containsParameter(TEXT("HOME"))) {
                [self setPersonProperty:person  property:kABURLsProperty           value:nsValue     label:kABHomeLabel indexToModify:0];
            }
            else {
                [self setPersonProperty:person  property:kABURLsProperty  value:nsValue label:kABHomePageLabel indexToModify:0];
            }
        }

        //
        // ---- ADR fields ----
        //
        else if(CFStringCompare(name, CFSTR("ADR"), 0) == 0) {
            [self addressToPerson: person property: vprop];
            otherAdr = [self isOtherAddress: person property: vprop];
        }

        //
        // ---- vCard extensions for manager, assistant and spouse fields ----
        else if(CFStringCompare(name, CFSTR("X-MANAGER"), 0) == 0) {
            [self setPersonProperty:person  property:kABRelatedNamesProperty  value:nsValue label:kABManagerLabel indexToModify:0];
        } else if(CFStringCompare(name, CFSTR("X-ASSISTANT"), 0) == 0) {
            [self setPersonProperty:person  property:kABRelatedNamesProperty  value:nsValue label:kABAssistantLabel indexToModify:0];
        } else if(CFStringCompare(name, CFSTR("X-SPOUSE"), 0) == 0) {
            [self setPersonProperty:person  property:kABRelatedNamesProperty  value:nsValue label:kABSpouseLabel indexToModify:0];
        }
        
        //
        // ---- Funambol defined properties ----
        //
        // TODO 
        /*
        else if([name isEqual: @"X-FUNAMBOL-ANNIVERSARY"]) {
            setProperty("Anniversary", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-CHILDREN"]) {
            setProperty("Children", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-COMPANIES"]) {
            setProperty("Companies", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-FOLDER"]) {
            setProperty("Folder", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-GENDER"]) {
            setProperty("Gender", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-HOBBIES"]) {
            setProperty("Hobby", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-INITIALS"]) {
            setProperty("Initials", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-LANGUAGES"]) {
            setProperty("Language", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-MANAGER"]) {
            setProperty("ManagerName", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-MILEAGE"]) {
            setProperty("Mileage", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-SPOUSE"]) {
            setProperty("Spouse", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-SUBJECT"]) {
            setProperty("Subject", element);
        }
        else if([name isEqual: @"X-FUNAMBOL-TELEX"]) {
            setProperty("TelexNumber", element);
        }
        */
        else
        {
            CFRelease(value);
        }
        
        CFRelease(name);
    }
    
    delete vobj;
}




/** creates a vcard from a person */
- (void) personToVcard:
                (ABPersonRef)      person
    vcard:      (StringBuffer&) vcardString
{
    VObject* vo = new VObject();
    CFStringRef value;
    VProperty* vp = nil;
    
    vp = new VProperty(TEXT("BEGIN"), TEXT("VCARD"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    if (version3) {
        vp = new VProperty(TEXT("VERSION"), TEXT("3.0"));
    }
    else {
        vp = new VProperty(TEXT("VERSION"), TEXT("2.1"));
    }
    vo->addProperty(vp);
    delete vp; vp = NULL;

    // ------- Name -------
    // Add only if at least 1 property is supported, but include
    // all elements in the right order.
    vp = new VProperty(TEXT("N"));
    
    if (value = [self getPersonProperty:person property:kABLastNameProperty]) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp->addValue("");
    }
    if (value = [self getPersonProperty:person property:kABFirstNameProperty]) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp->addValue("");
    }
    if (value = [self getPersonProperty:person property:kABMiddleNameProperty]) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp->addValue("");
    }
    if (value = [self getPersonProperty:person property:kABTitleProperty]) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp->addValue("");
    }
    if (value = [self getPersonProperty:person property:kABSuffixProperty]) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    }else{
        vp->addValue("");
    }
    vo->addProperty(vp);
    delete vp; vp = NULL;


    if (value = [self getPersonProperty:person property:kABBirthdayProperty]) {
        vp = new VProperty(TEXT("BDAY"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABNoteProperty]) {
        vp = new VProperty(TEXT("NOTE"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    // Telephone numbers
    
    // first work phone
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:kABWorkLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("VOICE"), NULL);
        vp->addParameter(TEXT("WORK"),  NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    // second work phone
    value = [self getPersonProperty:person property:kABPhoneProperty label:kABWorkLabel propertyIndex:1];

    if (value) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp = new VProperty(TEXT("TEL"), "");
    }
    
    vp->addParameter(TEXT("VOICE"), NULL);
    vp->addParameter(TEXT("WORK"),  NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;
    
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:kABPhoneWorkFAXLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("WORK"), NULL);
        vp->addParameter(TEXT("FAX"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:(const NSString *)kABPhoneCompanyLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("WORK"), NULL);
        vp->addParameter(TEXT("PREF"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:kABHomeLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("VOICE"), NULL);
        vp->addParameter(TEXT("HOME"),  NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:kABPhoneHomeFAXLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("HOME"), NULL);
        vp->addParameter(TEXT("FAX"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:(const NSString *)kABPhoneFAXLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("FAX"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty label:(const NSString *)kABPhoneCarLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("CAR"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty  label:kABPhoneMobileLabel propertyIndex:0] ) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("CELL"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    // other phone numbers
    if (value = [self getPersonProperty:person property:kABPhoneProperty    label:kABOtherLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("VOICE"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    value = [self getPersonProperty:person property:kABPhoneProperty    label:kABOtherLabel propertyIndex:1];
    
    if (value) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        CFRelease(value);
    } else {
        vp = new VProperty(TEXT("TEL"), "");
    }
    
    vp->addParameter(TEXT("VOICE"), NULL);
    vo->addProperty(vp);
    delete vp; vp = NULL;
    
    if (value = [self getPersonProperty:person property:kABPhoneProperty    label:kABPhonePagerLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("PAGER"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABPhoneProperty    label:kABPhoneMainLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("TEL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("PREF"),  NULL);
        vp->addParameter(TEXT("VOICE"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    // End Tel
    if (value = [self getPersonProperty:person property:kABURLsProperty label:kABWorkLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("URL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("WORK"),  NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABURLsProperty    label:kABHomeLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("URL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("HOME"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABURLsProperty    label:kABHomePageLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("URL"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }

    /*
    // TODO Can we support categories?
    if (getProperty("Categories", element)) {
        vp = new VProperty("CATEGORIES", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO Can we save FileAs?
    if (getProperty("FileAs", element)) {
        vp = new VProperty("FN", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */

    // Mapping is:
    // Email1Address <-> EMAIL;INTERNET:
    // Email2Address <-> EMAIL;INTERNET;HOME:
    // Email3Address <-> EMAIL;INTERNET;WORK:
    
    
    if (value = [self getPersonProperty:person property:kABEmailProperty label:kABOtherLabel propertyIndex:0]) {
        VProperty vp(TEXT("EMAIL"), CFString2Std(value).c_str()); // XXX
        vp.addParameter(TEXT("INTERNET"), NULL);
        vo->addProperty(&vp);
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABEmailProperty label:kABHomeLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("EMAIL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("INTERNET"), NULL);
        vp->addParameter(TEXT("HOME"),     NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    if (value = [self getPersonProperty:person property:kABEmailProperty label:kABWorkLabel propertyIndex:0]) {
        vp = new VProperty(TEXT("EMAIL"), CFString2Std(value).c_str());
        vp->addParameter(TEXT("INTERNET"), NULL);
        vp->addParameter(TEXT("WORK"),     NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }

    if (value = [self getPersonProperty:person property:kABJobTitleProperty] ) {
        vp = new VProperty(TEXT("TITLE"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }

    if (value = [self getPersonProperty:person property:kABRelatedNamesProperty label:kABManagerLabel propertyIndex:0] ) {
        vp = new VProperty(TEXT("X-MANAGER"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    if (value = [self getPersonProperty:person property:kABRelatedNamesProperty label:kABAssistantLabel propertyIndex:0] ) {
        vp = new VProperty(TEXT("X-ASSISTANT"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    if (value = [self getPersonProperty:person property:kABRelatedNamesProperty label:kABSpouseLabel propertyIndex:0] ) {
        vp = new VProperty(TEXT("X-SPOUSE"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    
    /*
    // TODO Importance supported?
    if (getProperty("Importance", element)) {
        vp = new VProperty("PRIORITY", CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */

    if (value = [self getPersonProperty:person property:kABNicknameProperty]) {
        vp = new VProperty(TEXT("NICKNAME"), CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }

    /*
    // TODO Profession supported?
    if (getProperty("Profession", element)) {
        vp = new VProperty("ROLE", CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO Sensitivity supported?
    if (getProperty("Sensitivity", element)) {
        long sensitivity = _wtoi(CFString2Std(value).c_str());
        vp = new VProperty(TEXT("CLASS"));
        if(sensitivity == winPrivate) {
            vp->addValue(TEXT("PRIVATE"));
        }
        else if (sensitivity == winConfidential) {
            vp->addValue(TEXT("CONFIDENTIAL"));
        }
        else {  // default value
            vp->addValue(TEXT("PUBLIC"));
        }
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */


    // ----- ORG -----
    // Add only if at least 1 property is supported, but include
    // all elements in the right order.
    vp = new VProperty(TEXT("ORG"));
    if (value = [self getPersonProperty:person property:kABOrganizationProperty] ) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    }
    else
    {
        vp->addValue("");
    }
    if (value = [self getPersonProperty:person property:kABDepartmentProperty]) {
        vp->addValue(CFString2Std(value).c_str());
        CFRelease(value);
    }
    else
    {
        vp->addValue("");
    }
    // TODO Office Location?
    //if (value = [self getPersonProperty:person property:kABDepartmentProperty])   found = true;
    vp->addValue(TEXT(""));//CFString2Std(value).c_str());
    vo->addProperty(vp);
    delete vp; vp = NULL;


    // ----- Address HOME -----
    // Add only if at least 1 property is supported, but include
    // all elements in the right order.
    // "AddressPostOfficeBox" is not supported by WM.
    // "AddressExtended" is not supported by Outlook/WM.
    [self personToHomeAddress: person intoVCard: *vo];

    // ----- Address OTHER -----
    // Add only if at least 1 property is supported, but include
    // all elements in the right order.
    // "AddressPostOfficeBox" is not supported by WM.
    // "AddressExtended" is not supported by Outlook/WM.
    [self personToOtherAddress: person intoVCard: *vo];


    // ----- Address WORK -----
    // Add only if at least 1 property is supported, but include
    // all elements in the right order.
    // "AddressPostOfficeBox" is not supported by WM.
    // "AddressExtended" is not supported by Outlook/WM.
    [self personToWorkAddress: person intoVCard: *vo];

    CFDataRef val = nil;
    //PHOTO
    // Cannot reliably sync photos until Apple gives us info about the type stored!!
    //if (val = [self getPersonPhoto:person]) {
    if (val = ABPersonCopyImageData(person)) {
        int len = CFDataGetLength(val);
        char * content = b64EncodeWithSpaces((const char*)CFDataGetBytePtr(val), len);
        if (content) {
            vp = new VProperty("PHOTO", content);
            vp->addParameter("TYPE", "JPEG");
            vp->addParameter("ENCODING", "b");
            vp->addParameter("CONTENT-VALUE", "UNCHANGED");
            vo->addProperty(vp);
            delete vp; vp = NULL;
        } else {
            vp = new VProperty("PHOTO", "");
            vo->addProperty(vp);
            delete vp; vp = NULL;
        }
        delete [] content;

        //CFRelease(val);//CFDataGetBytePtr
        CFRelease(val);//ABPersonCopyImageData
        
    } else {
        vp = new VProperty("PHOTO", "");
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    //
    // ---- Funambol defined properties ----
    // Support for other fields that don't have a
    // specific correspondence in vCard.

    /* FIXME - convert property
    if (value = [self getPersonProperty:person property:kABDateProperty   label:kABAnniversaryLabel]) {
        vp = new VProperty(TEXT("X-FUNAMBOL-ANNIVERSARY"));
        vp->addValue(CFString2Std(value).c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
        CFRelease(value);
    }
    */
    /*
    // TODO No support
    if (getProperty("Children", element)) {
        vp = new VProperty("X-FUNAMBOL-CHILDREN", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No support
    if (getProperty("Companies", element)) {
        vp = new VProperty("X-FUNAMBOL-COMPANIES", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No support
    if (getProperty("Folder", element)) {
        vp = new VProperty("X-FUNAMBOL-FOLDER");
        vp->addValue(element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No support
    if (getProperty("Gender", element)) {
        vp = new VProperty("X-FUNAMBOL-GENDER", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No support
    if (getProperty("Hobby", element)) {
        vp = new VProperty("X-FUNAMBOL-HOBBIES", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    if (getProperty("IMAddress", element)) {
        vp = new VProperty("EMAIL", element.c_str());
        vp->addParameter("INTERNET", NULL);
        vp->addParameter("HOME",     NULL);
        vp->addParameter("X-FUNAMBOL-INSTANTMESSENGER", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("Initials", element)) {
        vp = new VProperty("X-FUNAMBOL-INITIALS", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("Language", element)) {
        vp = new VProperty("X-FUNAMBOL-LANGUAGES", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("ManagerName", element)) {
        vp = new VProperty("X-FUNAMBOL-MANAGER", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("Mileage", element)) {
        vp = new VProperty("X-FUNAMBOL-MILEAGE", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("RadioTelephoneNumber", element)) {
        vp = new VProperty("TEL", element.c_str());
        vp->addParameter("X-FUNAMBOL-RADIO", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support    
    if (getProperty("Spouse", element)) {
        vp = new VProperty("X-FUNAMBOL-SPOUSE", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("Subject", element)) {
        vp = new VProperty("X-FUNAMBOL-SUBJECT", element.c_str());
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */
    /*
    // TODO No Support
    if (getProperty("TelexNumber", element)) {
        vp = new VProperty("TEL", element.c_str());
        vp->addParameter("X-FUNAMBOL-TELEX", NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }
    */


    vp = new VProperty(TEXT("END"), TEXT("VCARD"));
    vo->addProperty(vp);
    delete vp; vp = NULL;


    //
    // Format the vCard.
    // -----------------
    //
    char * tmp = vo->toString();
    if (tmp) {
        vcardString = tmp;
        delete [] tmp;
    }
    delete vo;
}

@end
