/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "event/MacSyncListener.h"
#include "base/Log.h"
#include "stringUtils.h"
#include "constants.h"           // common constants definitions
#include "event/constants.h"     // event defines from Funambol API

using namespace Funambol;


MacSyncListener::MacSyncListener(const char* ssn) : SyncListener(ssn)
{
    nCenter = CFNotificationCenterGetDistributedCenter(); 
}
 
void MacSyncListener::syncBegin(SyncEvent &event)
{
    postNotification(SYNC_STATUS_NOTIFICATION, SYNC_BEGIN, event.getMessage());
    LOG.debug("Beginning synchronization");
}

void MacSyncListener::syncEnd(SyncEvent &event)
{
    postNotification(SYNC_STATUS_NOTIFICATION, SYNC_END, event.getMessage());
    LOG.debug("Synchronization ended");
}

void MacSyncListener::syncError(SyncEvent &event)
{
    postNotification(SYNC_STATUS_NOTIFICATION, SYNC_ERROR, event.getMessage());
    LOG.debug("sync error: %s", event.getMessage());
}

void MacSyncListener::sendInitialization(SyncEvent &event)
{
    postNotification(SYNC_STATUS_NOTIFICATION, SEND_INITIALIZATION, event.getMessage());
    LOG.debug("sending initialization");
}

void MacSyncListener::sendModifications(SyncEvent &event)
{
    LOG.debug("sending modifications");
}

void MacSyncListener::sendFinalization(SyncEvent &event)
{
    postNotification(SYNC_STATUS_NOTIFICATION, SEND_FINALIZATION, event.getMessage());
    LOG.debug("sending finalization");
}

void MacSyncListener::postNotification(const StringBuffer& NotificationName, int NotificationType, 
            const StringBuffer& SourceName)
{
    CFStringRef  notificationName = Std2CFString(NotificationName);
    CFNumberRef  notificationType = CFNumberCreate(NULL, kCFNumberIntType, &NotificationType);

    CFMutableDictionaryRef infoDict = CFDictionaryCreateMutable(NULL, 1, &kCFCopyStringDictionaryKeyCallBacks, 
                                                &kCFTypeDictionaryValueCallBacks);

    CFDictionaryAddValue(infoDict, CFSTR("NotificationType"), notificationType);

    CFNotificationCenterPostNotification(nCenter, notificationName, NULL, infoDict, TRUE);

    CFRelease(notificationName);
    CFRelease(notificationType);
    CFRelease(infoDict);
}
