/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "event/MacSyncSourceListener.h"
#include "stringUtils.h"
#include "constants.h"           // common constants definitions
#include "event/constants.h"     // event defines from Funambol API
#include "base/Log.h"

using namespace Funambol;

MacSyncSourceListener::MacSyncSourceListener(const char* ssn) : SyncSourceListener(ssn), totalClientItems(0), totalServerItems(0)
{
    nCenter = CFNotificationCenterGetDistributedCenter(); 
    receivingDataFromServer = false;
}

MacSyncSourceListener::~MacSyncSourceListener() {}
       
// listen for the Sync Begin Event
void MacSyncSourceListener::syncSourceBegin(SyncSourceEvent& event) {
    postNotification(SYNC_SOURCE_NOTIFICATION, SYNC_SOURCE_BEGIN, event.getSourceName(), totalClientItems);
    LOG.debug("syncSourceBegin");
}
    
// listen for the Sync End Event
void MacSyncSourceListener::syncSourceEnd(SyncSourceEvent& event) {
    postNotification(SYNC_SOURCE_NOTIFICATION, SYNC_SOURCE_END, event.getSourceName(), totalClientItems);
    LOG.debug("syncSourceEnd");
}
    
// listen for the server Sync Begin Event
void MacSyncSourceListener::syncSourceServerBegin(SyncSourceEvent& event) {
    postNotification(SYNC_SOURCE_NOTIFICATION, SYNC_SOURCE_SERVER_BEGIN, event.getSourceName(), totalClientItems);
    receivingDataFromServer = true;
    LOG.debug("syncSourceServerBegin");
}
    
// listen for the server Sync End Event
void MacSyncSourceListener::syncSourceServerEnd(SyncSourceEvent& event) {
    postNotification(SYNC_SOURCE_NOTIFICATION, SYNC_SOURCE_SERVER_END, event.getSourceName(), totalClientItems);
    LOG.debug("syncSourceServerEnd");
}
    
// listen for the SyncMode requested by the server
void MacSyncSourceListener::syncSourceSyncModeRequested(SyncSourceEvent& event) {
    LOG.debug("syncSourceSyncModeRequested");
}
   
// listen for total client items (number of changes) sent by Client.
void MacSyncSourceListener::syncSourceTotalClientItems(SyncSourceEvent& event) 
{
    totalClientItems = event.getData();    
    postNotification(SYNC_SOURCE_NOTIFICATION, SYNC_SOURCE_TOTAL_CLIENT_ITEMS, event.getSourceName(), totalClientItems);
    LOG.debug("syncSourceTotalClientItems");
}
    
// listen for total server items (number of changes) sent by Server.
void MacSyncSourceListener::syncSourceTotalServerItems(SyncSourceEvent& event) 
{
    totalServerItems = event.getData();    
    postNotification(SYNC_SOURCE_NOTIFICATION, SYNC_SOURCE_TOTAL_SERVER_ITEMS, event.getSourceName(), totalServerItems);
    LOG.debug("syncSourceTotalServerItems");
}

void MacSyncSourceListener::postNotification(const StringBuffer& NotificationName, int NotificationType, 
                                             const StringBuffer& SourceName, int ItemCount)
{
    CFStringRef  notificationName = Std2CFString(NotificationName);
    CFNumberRef  notificationType = CFNumberCreate(NULL, kCFNumberIntType, &NotificationType);
    CFNumberRef  itemCount = CFNumberCreate(NULL, kCFNumberIntType, &ItemCount);
    CFStringRef  sourceName = Std2CFString(SourceName);

    CFMutableDictionaryRef infoDict = CFDictionaryCreateMutable(NULL, 3, &kCFCopyStringDictionaryKeyCallBacks, 
                                                &kCFTypeDictionaryValueCallBacks);

    CFDictionaryAddValue(infoDict, CFSTR("NotificationType"), notificationType);
    CFDictionaryAddValue(infoDict, CFSTR("SourceName"), sourceName);
    CFDictionaryAddValue(infoDict, CFSTR("ItemCount"), itemCount);

    CFNotificationCenterPostNotification(nCenter, notificationName, NULL, infoDict, TRUE);

    CFRelease(itemCount);        
    CFRelease(notificationName);
    CFRelease(notificationType);
    CFRelease(sourceName);
    CFRelease(infoDict);
}
