/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "event/MacTransportListener.h"
#include "base/Log.h"
#include "stringUtils.h"
#include "event/constants.h"     // event defines from Funambol API
#include "constants.h"     // event defines from Funambol API
#include "AddressBookSyncSource.h"

using namespace Funambol;


MacTransportListener::MacTransportListener(const char* ssn) : TransportListener(ssn)
{
    nCenter = CFNotificationCenterGetDistributedCenter(); 
    source = NULL;
}

void MacTransportListener::sendDataBegin(TransportEvent &event)
{
    const char* name = source->getName();
    if (strcmp(name, CONTACTS_SOURCE_NAME) == 0) {
        ((AddressBookSyncSource*)source)->saveAddressBook();
    }
    //postNotification(TRANSPORT_NOTIFICATION, TRANSPORT_CONNECTING_EVENT, event.getSourceName());
    LOG.debug("%s", __FUNCTION__);
}

void MacTransportListener::sendDataEnd(TransportEvent &event)
{
    //postNotification(TRANSPORT_NOTIFICATION, TRANSPORT_DISCONNECT_EVENT, event.getSourceName());
    LOG.debug("%s", __FUNCTION__);
}

void MacTransportListener::receiveDataBegin(TransportEvent &event)
{
    //postNotification(TRANSPORT_NOTIFICATION, TRANSPORT_DISCONNECT_EVENT, event.getSourceName());
    LOG.debug("%s", __FUNCTION__);
}

void MacTransportListener::receivingData(TransportEvent &event)
{
    LOG.debug("%s", __FUNCTION__);
}

void MacTransportListener::receiveDataEnd(TransportEvent &event)
{
    LOG.debug("%s", __FUNCTION__);
}

void MacTransportListener::postNotification(const StringBuffer& NotificationName, 
            const StringBuffer& NotificationType, const StringBuffer& SourceName)
{
    CFStringRef  notificationName = Std2CFString(NotificationName);
    CFStringRef  notificationType = Std2CFString(NotificationType);
    CFStringRef  sourceName = Std2CFString(SourceName);

    CFMutableDictionaryRef infoDict = CFDictionaryCreateMutable(NULL, 2, &kCFCopyStringDictionaryKeyCallBacks, 
                                                &kCFTypeDictionaryValueCallBacks);

    CFDictionaryAddValue(infoDict, CFSTR("NotificationType"), notificationType);
    CFDictionaryAddValue(infoDict, CFSTR("SourceName"), sourceName);
  
    CFNotificationCenterPostNotification(nCenter, notificationName, NULL, NULL, TRUE);
       
    CFRelease(notificationName);
    CFRelease(notificationType);
    CFRelease(sourceName);
}
