/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonab ly 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include <stdio.h>
#include <unistd.h>
#include <getopt.h>

#include "base/adapter/PlatformAdapter.h"
#include "SyncProcessManager.h"
#include "SyncQueueManager.h"
#include "constants.h"

USE_NAMESPACE

int checkForUpdates();

int main (int argc, char **argv) 
{
    int c = 0, i = 0;
    int option_index = 0;
    int verbose_flag = 0;
    int sync_contacts = 0;
    int sync_config = 0;
	int sync_briefcase = 0;
    int cleanup_contacts = 0;
    int use_wait_queue = 0;
    int sources_num = 0;
    int status = EXIT_SUCCESS; // return code for main program
	int syncStatusCode = 0;    // status code from sync manager thread
    const char* sources_list[MAX_SYNC_SOURCES_NUM];
    int waitQueue = 0;
	const char *briefcase_folder = NULL;
    
    unsigned int syncType = 0;

    struct option long_options[] = {
        {"sync-addressbook",    no_argument, 0, 'a' },
        {"sync-config",         no_argument, 0, 'c' },
        {"cleanup-contacts",    no_argument, 0, 'A' },
		{"sync-briefcase",		optional_argument, 0, 'b' },
        {"verbose",             no_argument, 0, 'v' },
        {"help",                no_argument, 0, 'h' },
        {"queue",               no_argument, 0, 'W' },
        {0, 0, 0, 0}
    };
    
    NSMutableArray* sourcesNameList         = nil;
    NSAutoreleasePool *pool                 = nil;
    MacMappingStoreBuilder* builder         = NULL;
    NSThread* syncProcessManagerThread      = nil; // sync worker thread
    NSThread* sigHandlerThread              = nil; // sig handler thread
    NSThread* waitQueueSigHandlerThread     = nil; // sig handler thread for wait queue
    SyncProcessManager *syncProcessManager  = nil;
    SigHandler *sigHandler                  = nil;
    SigHandler *waitQueueSigHandler         = nil;
    NSString*  sourceName                   = nil;
    WaitQueueHandler* waitQueueHandler      = nil;       
    SyncQueueManager *syncInfo              = NULL;    
    ClientConfig* clientConf				= NULL;
    StringBuffer opt;
    StringBuffer help;
    
    help.sprintf("%s MacOsX Sync Client\n\nUsage:\n--sync-addressbook [-a] to sync contacts\n--cleanup-addressbook [-A]"
                  " to reset your local contacts\n--sync-config [-c] to authenticate\n--queue [-W] to queue the process "
                  "while another sync is running (default behaviour is exiting with error).\n\n", APPNAME);
    
    if (argc < 2) {
        fprintf(stderr, "%s", help.c_str());
        exit(EXIT_FAILURE);
    } 
  
    if (checkProcessRunning(UPDATER_APPLICATION_NAME, false) != 0){
        fprintf(stderr, "AutoUpdate in progress... sync cancelled\n");
        exit(EXIT_FAILURE);    
    }
    
    while ((c = getopt_long(argc, argv, "acAvhWb", long_options, &option_index)) != EOF) {
        switch (c) {
            case 'a':
                if (!sync_contacts) {
                    if (cleanup_contacts) {
                        fprintf(stderr, "mismatching options selected");
                        exit(EXIT_FAILURE);
                    }
                    
                    sync_contacts = 1;
                    sources_list[sources_num++] = CONTACTS_SOURCE_NAME;
                    syncType = SYNC_SOURCE_CONTACTS;         
                }
                
                break;
 
            case 'c':
                if (!sync_config) {
                    sync_config = 1;
                    sources_list[sources_num++] = CONFIG_SOURCE_NAME;
                    syncType = SYNC_SOURCE_CONFIG;         
                }
                
                break;
 
			case 'b':
				if (!sync_briefcase) {
					sync_briefcase = 1;
                    sources_list[sources_num++] = BRIEFCASE_SOURCE_NAME;
                    syncType = SYNC_SOURCE_BRIEFCASE;
					
					if (optarg && strlen(optarg)) {
						briefcase_folder = optarg;
					} else {
						briefcase_folder = BRIEFCASE_DEFAULT_PATH;
					}
				}
				
				break;
				
            case 'A':
                if (cleanup_contacts == 0) {
                    if (sync_contacts) {
                        fprintf(stderr, "mismatching options selected\n");
                        exit(EXIT_FAILURE);
                    }
                    
                    cleanup_contacts = 1; 
                    sources_list[sources_num++] = CONTACTS_SOURCE_NAME;
                    syncType = SYNC_SOURCE_CONTACTS | SYNC_SOURCE_RESET;
                }
                
                break;
 
            case 'v':
                verbose_flag = 1;
                break;
           
            case 'h':
                fprintf(stdout, "%s", help.c_str());
                exit(EXIT_SUCCESS);
 
            case 'W':
                use_wait_queue = 1;
                break;

            case '?':
            default:
                fprintf(stderr, "%s", help.c_str());
                exit(EXIT_FAILURE);
         }
    }
    
    // initialize the PlatformAdapter and create a default config if needed
    clientConf = ClientConfig::getInstance();
	
	if (clientConf == NULL) {
		LOG.error("can't get client configuration: exiting");
        
        exit(EXIT_FAILURE);
	}
	
	clientConf->init();
    
    if (sources_num == 0) {
        LOG.error("no valid source to sync specified: exiting");
        fprintf(stderr, "%s", help.c_str());
        
        exit(EXIT_FAILURE);
    }

    pool = [[NSAutoreleasePool alloc] init];

    syncInfo = new SyncQueueManager(syncType);

    if ((waitQueue = syncInfo->lockSyncQueue()) < 0) {
        LOG.error("error while checking sync queue");
        delete syncInfo;
        
        exit(EXIT_FAILURE);
    }

    if (waitQueue) {
        if (use_wait_queue) {
            int syncJobInQueue = 0;
            
            if ((syncJobInQueue =  syncInfo->checkSyncQueue()) < 0) {
                LOG.info("error checking sync wait queue");
           
                [pool drain];

                exit(EXIT_FAILURE);
            }
            
            if (syncJobInQueue) {
                LOG.info("another sync for the same source is currently running: exiting from wait queue");

                [pool drain];
                
                exit(EXIT_SUCCESS);
            }
            
            waitQueueHandler = [[WaitQueueHandler alloc] init];
            
            [waitQueueHandler setQueueManager: syncInfo];
            
            waitQueueSigHandler = [[SigHandler alloc] init];
            waitQueueSigHandlerThread = [[NSThread alloc] initWithTarget: waitQueueSigHandler
                                                          selector: @selector(runWaitQueueSigHandler:)
                                                            object: (id)waitQueueHandler];
            // start signals handler thread
            [waitQueueSigHandlerThread start];
        } else { 
            LOG.error("another sync is currently running: exiting...");
            [pool drain];
            
            exit(EXIT_FAILURE);
        }
    }

    // if sync queue is active this call will block until queue manager 
    // will gain access on queue lock (which is an ipc semaphore)
    if (syncInfo->setCurrentSyncType()) {
        LOG.error("error setting sync info\n");
        [pool drain];
        
        exit(EXIT_FAILURE);
    }
   
    if (waitQueue) {
        BOOL exitOnSignal = NO;
        
        if ([waitQueueSigHandler getSignalCatched]) {
            exitOnSignal = YES;
        }
        
        // stop sig handler thread 
        [waitQueueSigHandler stopSigHandler];
        [waitQueueSigHandlerThread dealloc];
        [waitQueueSigHandler dealloc];
        
        if (exitOnSignal) {
            LOG.info("signal catched: exiting");
            [pool drain];
            
            exit(EXIT_FAILURE);
        }
    }
    
    if (sync_briefcase) {
		fprintf(stderr, "setting \"%s\" as briefcase folder\n", briefcase_folder); 
		LOG.debug("setting \"%s\" as briefcase folder", briefcase_folder);
		clientConf->setBriefCaseFolder(briefcase_folder);
	}
	
    builder = new MacMappingStoreBuilder();
    MappingsManager::setBuilder(builder);
    
    sourcesNameList = [[NSMutableArray alloc] initWithCapacity: MAX_SYNC_SOURCES_NUM];
    
    for (i = 0; i < sources_num; i++) {
        NSString *sourceName = [NSString stringWithUTF8String: sources_list[i] ];
        if ([sourceName length]) {
            [sourcesNameList addObject: sourceName];
        }
        [sourceName release];
    }

    syncProcessManager = [[SyncProcessManager alloc] init];
    
    if (cleanup_contacts) {
        syncProcessManagerThread = [[NSThread alloc] initWithTarget: syncProcessManager
                                                           selector: @selector(runResetThread:)
                                                             object: sourcesNameList];
    } else {
        syncProcessManagerThread = [[NSThread alloc] initWithTarget: syncProcessManager
                                                           selector: @selector(runSyncThread:)
                                                             object: sourcesNameList];
    }
    
    sigHandler = [[SigHandler alloc] init];
    sigHandlerThread = [[NSThread alloc] initWithTarget: sigHandler
                                                         selector: @selector(runSyncSigHandler:)
                                                           object: (id)syncProcessManager];
    // start signals handler thread
    [sigHandlerThread start];
    
    // start sync thread 
    [syncProcessManagerThread start];
    
    // this loop is terminated by the runThread method. Needed to get the info from the UI
    CFRunLoopRun();
        
    // get sync process status    
	if ((syncStatusCode = [syncProcessManager syncStatus])) {
		status = EXIT_FAILURE;
	}

    LOG.info("sync ended with code: %d", syncStatusCode);
    
	
    // cleanup threads and resources
    [syncProcessManagerThread dealloc];
    [syncProcessManager dealloc];
    [sourcesNameList dealloc];
   
    delete syncInfo;

    // stop sig handler thread 
    [sigHandler stopSigHandler];
    [sigHandlerThread dealloc];
    [sigHandler dealloc];
   
    if (checkForUpdates()) { 
		// note: this error check is only for updater execution, not 
		// for update process: we don't wait for updater process, so 
		// we can't know for issues such as network errors, etc
        LOG.error("error executing updater process: %s", strerror(errno));
        status = EXIT_FAILURE;
    }
 
    [pool drain];
   
    exit(status);
}

int checkForUpdates()
{
    char* programArgs[3];
    int res = 0;

    programArgs[0] = (char *)UPDATER_APPLICATION_PATH;
    //setting a empty string because execv wants the argument not to be null    
    programArgs[1] = (char *)"";
    programArgs[2] = NULL;

    res = execProcess((char* const*)programArgs, false);

    return res;
}
