/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef __PERSON_DATA_CONVERTER_H__
#define __PERSON_DATA_CONVERTER_H__

#import <Foundation/Foundation.h>
#import <CoreFoundation/CoreFoundation.h>

#include <AddressBook/ABAddressBookC.h>
#include <AddressBook/ABGlobals.h>
#include <AddressBook/ABTypedefs.h>

#import "DataConverter.h"
#include "vocl/VConverter.h"
#include "spds/SyncItem.h"
#include "base/util/StringBuffer.h"

USE_NAMESPACE

@interface PersonDataConverter : DataConverter
{
    BOOL version3;
    BOOL verbose;
    CFErrorRef * error;
}

- (void) convertToSyncItem:
                (CFTypeRef)     person
   syncItem:    (SyncItem*)     item;

- (void) convertFromSyncItem:
                (SyncItem*) item
    native:     (ABPersonRef) person;

- (void) setPersonProperty:
                (ABPersonRef)           person
    property:   (NSString *)          property
    value:      (NSString *)           value;
    
- (void) setPersonProperty:
                (ABPersonRef)           person
    property:   (NSString *)          property
    value:      (NSString *)           value
    part:       (NSString *)           part; // Only used for Multi Prop Values
    
- (void) setPersonProperty:
                (ABPersonRef)        person
    property:   (NSString *)         property
    value:      (NSString *)         value
    label:      (NSString *)         label // Only used for Multi Prop Values
    indexToModify: (int)             pos;

- (void) setPersonProperty:
                (ABPersonRef)          person
    property:   (NSString *)           property
    value:      (NSString *)           value
    part:       (NSString *)           part   // Only used for Multi Prop Values
    label:      (NSString *)           label // Only used for Multi Prop Values
    indexToModify: (int)               pos;

- (CFStringRef) getPersonProperty:
                (ABPersonRef)           person
    property:   (NSString *)            property;
    
- (CFStringRef) getPersonProperty:
                (ABPersonRef)           person
    property:   (NSString *)            property
    part:       (NSString *)            part;
    
- (CFStringRef) getPersonProperty:
                (ABPersonRef)         person
    property:   (NSString *)          property
    label:      (NSString *)          label
    propertyIndex:  (int)              propertyIndex;

- (CFStringRef) getPersonProperty:
                (ABPersonRef)          person
    property:   (NSString *)           property
    part:       (NSString *)           part  // Only used for Multi Prop Values - @"" means ignore
    label:      (NSString *)           label
    propertyIndex:  (int)              propertyIndex;

- (void) setPersonPhoto:
                (ABPersonRef)           person
    value:      (CFDataRef)             value;
    
- (CFDataRef) getPersonPhoto:
                (ABPersonRef)           person;

- (void) clearPersonPhoto:
                (ABPersonRef)           person;


- (void) vCardToPerson: (StringBuffer&)         vcardString
         person:        (ABPersonRef)           person;
    

- (void) personToVcard:
                (ABPersonRef)           person
    vcard:      (StringBuffer&)          vcardString;
 
- (CFStringRef) buildAddress: (const char*) address1 
    secondLineAddress:        (const char*) address2;

- (void) addressToPerson: (ABPersonRef) person
    property:   (VProperty *)           property;

- (NSString *) getLabel:(ABPersonRef) person
                property: (VProperty *) property;

- (bool) isOtherAddress:(ABPersonRef) person
               property: (VProperty *) property;

- (void) getAddressInto: (StringBuffer&) address1 
      secondLineAddress: (StringBuffer&) address2 
              fromValue: (CFStringRef)  value;
- (void) personToAddress: (ABPersonRef) person 
                   label: (NSString *) label 
           intoVProperty: (VProperty&)  vp;

- (void) personToHomeAddress: (ABPersonRef)person 
                   intoVCard: (VObject&)vo;

- (void) personToWorkAddress: (ABPersonRef)person 
                   intoVCard: (VObject&)vo;

- (void) personToOtherAddress: (ABPersonRef)person 
                   intoVCard: (VObject&)vo;

@end

#endif //#ifndef __PERSON_DATA_CONVERTER_H__
