/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef __PERSON_SYNC_PROCESS_MANAGER_H__
#define __PERSON_SYNC_PROCESS_MANAGER_H__

#import <Foundation/Foundation.h>

#include "base/Log.h"
#include "spdm/DeviceManagementNode.h"
#include "customization.h"
#include "utils.h"
#include "MacMappingStoreBuilder.h"
#include "spds/MappingsManager.h"
#include "ClientConfig.h"
#include "AddressBookSyncSource.h"
#include "client/SyncClient.h"
#include "client/FileSyncSource.h"
#include "client/OptionParser.h"
#include "client/ConfigSyncSource.h"

#include "event/ManageListener.h"
#include "event/MacSyncItemListener.h"
#include "event/MacSyncListener.h"
#include "event/MacSyncSourceListener.h"
#include "event/MacTransportListener.h"
#include "stringUtils.h"
#include "SyncQueueManager.h"

/**
 * Objc wrapper class for passing SyncQueueManager c++ 
 * objects to NSThread methods 
 */
@interface WaitQueueHandler : NSObject
{
    SyncQueueManager *waitQueueManager;
};

- (id) init;
- (void) setQueueManager: (SyncQueueManager *) queueManager;
- (void) resetQueue;

@end

@class SyncProcessManager;

@interface SigHandler : NSObject
{
    CFRunLoopRef threadRunLoop;
    BOOL threadRunning;
    BOOL signalCatched;
}

- (id)   init;
- (void) runSyncSigHandler: (SyncProcessManager *) syncManager;
- (void) runWaitQueueSigHandler: (WaitQueueHandler *)queueManager;
- (void) stopSigHandler;
- (BOOL) getSignalCatched;

@end


/**
 * Class that drive the sync process in a task. It is called by the main process that
 * start the SyncProcessManager into a thread. This is able to get the requests from the Menubar
 * asking to cancel the sync in progress.
 */

@interface SyncProcessManager : NSObject
{
    CFNotificationCenterRef nCenter;
    int syncStatus;
}

/**
 * Starts the sync into a thread and control all the sync process.
 * @param object - is an array of StringBuffer that contains the name
 * of the sources to sync
 */
- (void) runSyncThread: (NSMutableArray *) sourcesNameList;

/**
 * Starts the reset into a thread and control all the sync process.
 * @param object - is an array of StringBuffer that contains the name
 * of the sources to sync
 */

- (void) runResetThread: (NSMutableArray *) sourcesNameList;


/**
 * Cancel the sync in progress. It sets the abort flag inside the config (read by the SyncManager).
 * If after some seconds the sync doesn't stop by itself, it forces to stop the sync exiting by the process.
 * 
 */
- (void) cancelSync;

/**
 * Perform a sync of a single source. It prepare everything and start the sync
 * 
 * @param sourcename - the name of the source to be synced
 */
- (int) doSync: (const char*) sourcename;

/**
 * Send the sync source name to the menubar UI.
 * 
 * @param sourcename - the name of the source to be synced
 */
- (void) postSourceNameNotification: (const char*) sourcename;
/**
 * Send a sync start notification event
 */
-(void) startSyncProcessNotification;

/** 
 * generate getter for sync process status
 */
@property (readonly) int syncStatus;

@end 

#endif
