/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/util/utils.h"
#include "PersonDataConverter.h"
#include "testUtils.h"
#include "base/Log.h"

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include "base/globalsdef.h"
#include "base/fscapi.h"

#define DIR_TESTCASES "../../test/testcases"

USE_NAMESPACE

/**
 * Test case for the class PersonDataConverter.
 */
class PersonDataConverterTest : public CppUnit::TestFixture {
    CPPUNIT_TEST_SUITE(PersonDataConverterTest);
    CPPUNIT_TEST(testvCard);
    CPPUNIT_TEST(testInsertAddress);
    CPPUNIT_TEST(testDeleteFields);
    CPPUNIT_TEST_SUITE_END();
    
private:
    
    ABAddressBookRef addressbook;
    
public: 
    void setUp() {
        
        addressbook = nil;
    }
    
    ABAddressBookRef getAddressBook()
    {
        if (!addressbook) {
            addressbook = ABGetSharedAddressBook();
            CFRetain(addressbook);
        }
        
        return addressbook;
    }
    
    void disposeAddressBook(){
        if(addressbook){
            CFRelease(addressbook);
            
        }
    }
    
    //////////////////////////////////////////////////////// Test /////
    
    void testInsertvCard() {
        
        ABAddressBookRef ab = getAddressBook();
        
        ABPersonRef abItem = ABPersonCreate();
        
        char* c = loadTestFile("PersonDataConverterTest", "vcard1.txt", true);
        StringBuffer s(c);
        
        PersonDataConverter *pdc = [[PersonDataConverter alloc] init];
        
        [pdc vCardToPerson:s person:abItem];
        [pdc release];        
        
        CPPUNIT_ASSERT(abItem);
        CPPUNIT_ASSERT(ABAddRecord(ab, abItem));
        CFRelease(abItem);
        CPPUNIT_ASSERT(ABSave(ab));
        CFRelease(ab);
        delete [] c;
    }
    
    void testInsertAddress() {
        
        StringBuffer address1("address1");
        StringBuffer address2("address2");
        StringBuffer address3("");
        PersonDataConverter *pdc = [[PersonDataConverter alloc] init];
        CFStringRef fullAdd1 = [pdc buildAddress:address1.c_str() secondLineAddress:address2.c_str()];
        CFStringRef fullAdd2 = [pdc buildAddress:address1.c_str() secondLineAddress:address3.c_str()];
        [pdc release];
        StringBuffer res1(CFStringGetCStringPtr(fullAdd1, nil));
        StringBuffer res2(CFStringGetCStringPtr(fullAdd2, nil));
        
        CPPUNIT_ASSERT(res1.find("\n") != StringBuffer::npos);
        CPPUNIT_ASSERT(res2.find("\n") == StringBuffer::npos);
    }
    
    void testDeleteFields() {
        
        ABAddressBookRef ab = getAddressBook();
        
        ABPersonRef abItem = ABPersonCreate();
        
        char* c = loadTestFile("PersonDataConverterTest", "testDeleteField1.vcf", false);
        StringBuffer s(c);
        
        PersonDataConverter *pdc = [[PersonDataConverter alloc] init];
        [pdc vCardToPerson:s person:abItem];
        
        
        CPPUNIT_ASSERT(abItem);
        CPPUNIT_ASSERT(ABAddRecord(ab, abItem));
        CPPUNIT_ASSERT(ABSave(ab));

        char* c1 = loadTestFile("PersonDataConverterTest", "testDeleteField2.vcf", false);
        StringBuffer s1(c1);
        [pdc vCardToPerson:s1 person:abItem];
        CPPUNIT_ASSERT(abItem);
        CPPUNIT_ASSERT(ABAddRecord(ab, abItem));
        CPPUNIT_ASSERT(ABSave(ab));
        
        StringBuffer res;
        [pdc personToVcard:abItem vcard:res];
        s1.replaceAll("\r\n", "");
        res.replaceAll("\r\n","");
        
        LOG.debug(s1);
        LOG.debug(res);
        
        CPPUNIT_ASSERT(strcmp(s1.c_str(), res.c_str()) == 0);
        
        CPPUNIT_ASSERT(ABRemoveRecord(ab, abItem));
        CPPUNIT_ASSERT(ABSave(ab));
        
        CFRelease(abItem);
        CFRelease(ab);
        [pdc release];      
        
        delete [] c;
    }
    
    CFStringRef insertContact(StringBuffer& data, ABAddressBookRef ab) {
        
        ABPersonRef abItem = ABPersonCreate();
        
        // insert into the db
        PersonDataConverter *pdc = [[PersonDataConverter alloc] init];
        [pdc vCardToPerson:data person:abItem];
        
        CPPUNIT_ASSERT(abItem);
        
        CPPUNIT_ASSERT(ABAddRecord(ab, abItem));            
        CPPUNIT_ASSERT(ABSave(ab));
        
        CFStringRef uid = ABRecordCopyUniqueId(abItem);
        
        [pdc release];     
        CFRelease(abItem);
        
        return uid;
        
    }
    
    VObject* getContactAsVObject(CFStringRef uid, ABAddressBookRef ab) {
        
        StringBuffer vCard;
        // retrieve the contact just inserted
        PersonDataConverter *pdc = [[PersonDataConverter alloc] init];
        
        ABRecordRef person = ABCopyRecordForUniqueId(ab, uid);     
        [pdc personToVcard:(ABPersonRef)person vcard:vCard];
        [pdc release]; 
        
        VObject* vobj = VConverter::parse(vCard.c_str());    
        CPPUNIT_ASSERT(vobj);
        return vobj;
        
    }
    
    void removeContactFromAddressBook(CFStringRef uid, ABAddressBookRef ab) {
        
        ABRecordRef abItem = ABCopyRecordForUniqueId(ab, uid);   
        CPPUNIT_ASSERT(ABRemoveRecord(ab, abItem));
        CPPUNIT_ASSERT(ABSave(ab));
        
    }    
    
    /**
     * 
     */
    bool isInTheList(const char* propName, ArrayList& list) {
        
        StringBuffer* l;
        bool ret = false;
        for(l = (StringBuffer*)list.front(); l; l = (StringBuffer*)list.next()) {
            if (*l == propName) {
                ret = true;
                break;
            }
        }
        return ret;
    }
    
    
    bool compareString(char* a, char* b, ArrayList& arrayOfProperties) {
        StringBuffer s1(a);
        StringBuffer s2(b);
        
        // check if the strings are already equals
        if (s1 == s2) {
            return true;
        }
        
        ArrayList tokens1;
        ArrayList tokens2;
        
        // split for ":" to get parts of the string
        s1.split(tokens1, ":");
        s2.split(tokens2, ":");
        
        int size1 = tokens1.size();
        int size2 = tokens2.size();
        
        // check the token of the keys
        
        StringBuffer key1;
        StringBuffer key2;
        
        StringBuffer value1;
        StringBuffer value2;
        
        
        // for the key we get the first element
        // for the value we get the second element
        
        if (size1 >= 2) {
            key1.append((StringBuffer*)tokens1.get(0));
            for (int i = 1; i < size1; i++) {
                value1.append((StringBuffer*)tokens1.get(i));
            }
        } else {
            LOG.debug("errors parsing the first parameter: %s", s1.c_str());
            return false;
        }

        if (size2 == 2) {
            key2.append((StringBuffer*)tokens2.get(0));
            for (int i = 1; i < size2; i++) {
                value2.append((StringBuffer*)tokens2.get(i));
            }
        } else {
            LOG.debug("errors parsing the second parameter: %s", s2.c_str());
            return false;
        }
        
                
        // check if we have to consider as verified 
        if (isInTheList(key1.c_str(), arrayOfProperties)) {
            LOG.debug("\t\tThe element %s is in the filter list", key1.c_str());
            return true;
        }            
        
        // just inserted for PHOTO: sometimes we can have 
        // ENCODING=b or ENCODING=BASE64
        key1.replace("ENCODING=BASE64", "ENCODING=b");
        key2.replace("ENCODING=BASE64", "ENCODING=b");
        
        
        
        // now split the key1 using the ; separator. If there are no ";" consider the full key as the only one
        bool keyfound = true;
        bool valuefound = true;
        
        ArrayList tokensKey1;
        ArrayList tokensKey2;
        
        int tokensKeyCount1 = 0;
        int tokensKeyCount2 = 0;

        key1.split(tokensKey1, ";");
        tokensKeyCount1 = tokensKey1.size();
        
        key2.split(tokensKey2, ";");
        tokensKeyCount2 = tokensKey2.size();
        
        // if the number of the keys element is different, they are not equals for sure!
        if (tokensKeyCount1 != tokensKeyCount2) {
            return false;
        }
        
        for (int i = 0; i < tokensKeyCount1; i++) {
            StringBuffer* chunk = (StringBuffer*)tokensKey1.get(i);
            if (key2.ifind(chunk->c_str()) == StringBuffer::npos) {
                keyfound = false;
            }
        } 
        
        if (keyfound == false) {
            return false;
        }
        
        // strip white spaces from photos b64 values 
        if (key1 == "PHOTO") {
            StringBuffer normalizedVal1 = normalizeB64Value(value1);    
            StringBuffer normalizedVal2 = normalizeB64Value(value2);

            
            if (normalizedVal1 == normalizedVal2) {
                valuefound = true;
            }
        } else if (value1 == value2) {
            valuefound = true;
        } 
        
        if (valuefound == false) {
            //
            // check only if they differs from the ; final. we found something like that in ORG field
            //
            
            StringBuffer tmp1(value1);
            StringBuffer tmp2(value2);
            
            if (value1.endsWith(";")) {
                tmp1 = value1.substr(0, value1.length() -1);
            } 
            if (value2.endsWith(";")) {
                tmp2 = value2.substr(0, value2.length() -1);
            }
            if (tmp1 == tmp2) {
                LOG.debug("\t\tThe Keys are equals, but values are different only for the last \";\". %s \t %s \t %s", key1.c_str(), value1.c_str(), value2.c_str()); 
                valuefound = true;
            }
            
        }
        
        if (valuefound) {
            return true;
        } else {
            return false;
        }
  
    }
    
    StringBuffer normalizeB64Value(const StringBuffer& val) 
    {
        if (val.empty()) {
            return val;
        }
        
        LOG.info("normalizing b64 value");
        
        char* normValue = new char[ val.length() + 1 ];
        int c = 0, i = 0;
        const char* origValue = val.c_str();
        
        while ((c = *origValue++) != '\0') {
            if (!isspace(c)) {
                normValue[i++] = c;
            }
        }
        
        StringBuffer normalizedValue = normValue;
        delete [] normValue;
        
        return normalizedValue;
    }
    
    //    
    // use WCHAR to be compliant but the WCHAR is defined a char.
    // so also the method are referred to char
    //    
    int compare(VObject* v1, VObject* v2, ArrayList& arrayOfProperties) {
        
        int ret = -1;
        
        WString failed;
        if (v1 == NULL || v2 == NULL) {
            LOG.debug("v1 or v2 is NULL");
            return ret;
        }
        if (v1 && v2 && strcmp(v1->getVersion(), v2->getVersion()) != 0) {
            LOG.debug("version are different: %s - %s", v1->getVersion(), v2->getVersion());
            return ret;
        }
        WCHAR* version = v1->getVersion();
        
        int v1Count = v1->propertiesCount();
        int v2Count = v2->propertiesCount();
        int failures = 0; // means the properties that fails
        int missing = 0;
        
        LOG.debug("property num v1: %i; property num v2: %i", v1Count, v2Count);
        
        for (int i = 0; i < v1Count; i++) {
            
            VProperty *prop   = v1->getProperty(i);        
            WCHAR* propName   = prop->getName();
            
            WCHAR* propString = prop->toString(version);
            WCHAR* toCompare = NULL;
            bool found = false;
            if (v2->containsProperty(propName)) {
                for (int j = 0; j < v2Count; j++) {
                    VProperty *prop2 = v2->getProperty(j);        
                    if (strcmp(propName, prop2->getName()) == 0) {
                        toCompare = prop2->toString(version);
                        if (compareString(propString, toCompare, arrayOfProperties)) {
                            found = true;
                            delete [] toCompare;
                            break;
                        } else {
                            delete [] toCompare;
                            continue;
                        }
                    }                     
                }
                
                if (found == false) {
                    LOG.debug("FAILED Property name: %s; \t v1 value: %s;", propName, propString);
                    failures++;
                }
                
            } else {
                LOG.debug("MISSING Property %s not in v2 object", propName);
                LOG.debug("The value in v1 is %s");
                missing++;
            }
            delete [] propString;
            
        }
        for (int i = 0; i < v2Count; i++) {
            VProperty *prop   = v2->getProperty(i);        
            WCHAR* propName   = prop->getName();
            if (v1->containsProperty(propName) == false) {
                WCHAR* propString = prop->toString(version);
                LOG.debug("In v2 but not in v1: \t %s", propString);
                delete [] propString;
            }
        }
        
        ret = failures;
        if (missing > 0) {
           
        }
        return ret;
        
        
    }    
    
    StringBuffer getLogFilename() {
        char buffer[256];
        struct tm tm;
        time_t t = time(NULL);
        StringBuffer b("PersonDataConverterTest_Log");
        b.append(".");
        
        sprintf(buffer, "%lu", t);
        b.append(buffer);
        return b;
        
    }    
    
    ArrayList fillArrayOfProperties(StringBuffer& filename) {
        ArrayList properties;
        StringBuffer list;
        char* c = loadTestFile("PersonDataConverterTest", filename.c_str(), false);
        list = c;
        delete [] c;
        list.split(properties, "\r\n");
        
        return properties;
        
    }
    
    void testvCard() {
        
        LOG.setLogName(getLogFilename().c_str());
        LOG.reset();
        LOG.setLevel(LOG_LEVEL_DEBUG);
        ABAddressBookRef ab = getAddressBook();
        
        int count = 0;
        StringBuffer dir = getTestDirFullPath("PersonDataConverterTest");
        char** files = readDir(dir.c_str(), &count, false);

        ArrayList arrayOfProperties;
        StringBuffer data; 
        StringBuffer filename;
        int ret = 0; // it collects all the results. If all the checks work fine, every value added to ret is 0
        int results = 0;

        // get the properties file with the properties we have to not consider in the test
        for (int i = 0; i < count; i++) {            
            filename = files[i];
            if (filename == "list.vcard.properties") {
                arrayOfProperties = fillArrayOfProperties(filename);
                break;
            }
        }
        
        for (int i = 0; i < count; i++) {
            
            filename = files[i];
            if (filename == "list.vcard.properties") {
                continue;
            }
            
            LOG.debug("*****************");        
            LOG.debug("*****************");
            LOG.debug("File to process: %s", filename.c_str());
            LOG.debug("*****************");
            
            char* c = loadTestFile("PersonDataConverterTest", filename.c_str(), false);
            data = c;
            delete [] c;
            
            // insert contact if error in inserting exit the test session
            CFStringRef uid = insertContact(data, ab);
            
            // get the contact as vobject just inserted
            VObject* vobjInserted = getContactAsVObject(uid, ab);
            
            // get the original VObject
            VObject* vobjOriginal = VConverter::parse(data.c_str());
            CPPUNIT_ASSERT(vobjOriginal);
            
            // remove the contact from address book
            removeContactFromAddressBook(uid, ab);
            LOG.debug("v1 (vobject original) -- v2 (vobject iserted)");
            
            // compare if the objects are equals and log if there are problems
            // if ret is < 0 is not 
            int tmpResult = compare(vobjOriginal, vobjInserted, arrayOfProperties);
            if (tmpResult < 0) {
                LOG.debug("The compare action returned -1");
                ret = -1;
            } else {
                results += tmpResult;
            }
            
        }
        disposeAddressBook();
        
        for (int i = 0; i < count; i++) {
            delete [] files[i];
        }
        delete [] files;
        
        if (ret != -1) {
            ret = results;
        }
            
        CPPUNIT_ASSERT_EQUAL_MESSAGE("Total number of properties in all vcards", 0, ret);
        
    }    
    
};

CPPUNIT_TEST_SUITE_REGISTRATION( PersonDataConverterTest );


