#!/bin/bash

test_list="test_sync_queue test_sync_noqueue"
verbose=1

## just to be sure that sync-engine has enough time to 
## exit after check on queue
wait_sync_process()
{
    sleep 1
}

test_sync_queue()
{
    no_queue_sync=0
    queue_sync=1

    log_message "starting config sync.."
    pid1=$(sync_config $no_queue_sync)
    
    log_message "starting contacts sync (wait mode)"
    pid2=$(sync_contacts $queue_sync)

    log_message "starting second contacts sync (wait mode)"
    pid3=$(sync_contacts $queue_sync)

    wait_sync_process
    check_sync_process "$pid1 $pid2 $pid3"
    sync_running=$?

    log_message "sync process found running: $sync_running"
    if [ $sync_running -gt 2 ]; then
        echo "failed check on sync queue..."

        return $EXIT_FAILURE
    fi

    log_message "starting second config sync (wait mode)"
    pid3=$(sync_config $queue_sync)

    wait_sync_process
    check_sync_process "$pid1 $pid2 $pid3"
    sync_running=$?

    log_message "sync process found running: $sync_running"
    if [ $sync_running -gt 2 ]; then
        log_error "failed check on sync queue..."

        return $EXIT_FAILURE
    fi

    cleanup_sync_running

    return $EXIT_SUCCESS
}

test_sync_noqueue()
{
    no_queue_sync=0
    queue_sync=1

    log_message "starting contacts sync.."
    pid1=$(sync_contacts $no_queue_sync)
    
    log_message "starting contacts sync (no wait mode)"
    pid2=$(sync_contacts $no_queue_sync)

    wait_sync_process
    check_sync_process "$pid1 $pid2"
    sync_running=$?

    log_message "sync process found running: $sync_running"
    if [ $sync_running -gt 1 ]; then
        log_error "failed check on sync queue..."

        return $EXIT_FAILURE
    fi

    log_message "starting contacts sync in wait mode"
    pid2=$(sync_contacts $queue_sync)

    wait_sync_process
    check_sync_process "$pid1 $pid2"
    sync_running=$?

    log_message "sync process found running: $sync_running"
    if [ $sync_running -gt 2 ]; then
        log_error "failed check on sync queue..."

        return $EXIT_FAILURE
    fi

    cleanup_sync_running

    return $EXIT_SUCCESS
}

main() 
{
    tests_run=0
    tests_failed=0

    # source in unit tests common utilities
    source unit-test-common >/dev/null 2>&1

    if [ $? != 0 ]; then
        echo >&2 "can't source in unit-test-common..."

        exit 1
    fi    

    if [ ! -x "$sync_engine" ]; then
        log_error "sync engine application not set: check build output in $build_dir or consider 
            exporting FUNAMBOL_SYNC_ENGINE in your enviroment"
        exit $EXIT_FAILURE
    fi

    log_message "## starting sync queue unit tests...."

    for test_name in $test_list; do
        log_message "## executing test: $test_name"

        $test_name

        if [ $? != 0 ]; then
            log_message "unit test \"$test_name\" failed!"
            tests_failed=$(expr $tests_failed + 1)
        fi

        tests_run=$(expr $tests_run + 1)
    done

    # wipe out all the syncs still running 
    cleanup_sync_running

    log_message "## total test run: $tests_run"
    log_message "## failed: $tests_failed"

    if [ $tests_failed -gt 0 ]; then
        exit $EXIT_FAILURE
    else
        exit $EXIT_SUCCESS
    fi
}

main $@
# EOF
