/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#import "UpdaterController.h"
#import "customization.h"
#import "updater/Updater.h"
#include "http/URL.h"
#include "base/adapter/PlatformAdapter.h"
#include "base/util/StringBuffer.h"
#include "ClientConfig.h"
#include "customization.h"
#include "UpdaterManager.h"
#import <Foundation/Foundation.h>
#include <pwd.h>
#import "utils.h"
#import "constants.h"


USE_NAMESPACE

@implementation UpdaterController

- (id)init {
    self = [super init];    
    return self;
}

- (void) awakeFromNib
{
    
    ClientConfig::getInstance()->init();
    ClientConfig::getInstance()->read();
    StringBuffer appuri(PlatformAdapter::getAppContext());
    
    upconfig = new UpdaterConfig(appuri.c_str());
    upconfig->read();
    //if(upconfig->getRecommended().empty()){
    if(upconfig->getUrlCheck().empty()){
        upconfig->createDefaultConfig();
        
        URL url;
        url.setURL(ClientConfig::getInstance()->getSyncURL());
        StringBuffer port(":80");
        if (url.port != 0) { 
            port = ":"; 
            port += itow(url.port); 
        }
        
        // Compose the url
        StringBuffer updateUrl(url.protocol);
        updateUrl += "://"; 
        updateUrl += url.host;
        updateUrl += port;
        updateUrl += UPDATE_SOFTWARE_URL_RESOURCE;
        
        upconfig->setUrlCheck(updateUrl);
        upconfig->setRecommended("1");
        upconfig->save();
        LOG.debug("New update url = %s", updateUrl.c_str());
    }
    
    StringBuffer oldv = ClientConfig::getInstance()->getFunambolSwv();
    Updater* up = new Updater(UPDATER_COMPONENT, oldv, *upconfig);
    
    StringBuffer val; val = upconfig->getUrlCheck();
    
    UpdaterManager* upman = new UpdaterManager(self);
    up->setUI(upman);
    int lastcheck = atoi (upconfig->getLastCheckUpdate());
    lastcheck = lastcheck + upconfig->getIntervalCheckUpdateHttp();
    int time_check = time(0);
    if (time_check > lastcheck) {
        BOOL isnewVersionAvailable = NO;
        
        if (checkProcessRunning(SYNC_APPLICATION_NAME,1)) {
            //CFNotificationCenterRef nCenter = CFNotificationCenterGetDistributedCenter();;
            NSDistributedNotificationCenter* nCenter = [NSDistributedNotificationCenter defaultCenter];
            NSMutableDictionary* infoDict = [NSMutableDictionary dictionaryWithCapacity: 1];
            [nCenter postNotificationName: (NSString*)CFSTR(SYNC_CANCELLED_NOTIFICATION) object:nil userInfo: infoDict deliverImmediately: YES];
            
            /*CFMutableDictionaryRef infoDict = CFDictionaryCreateMutable(NULL, 1, &kCFCopyStringDictionaryKeyCallBacks, 
                                                                        &kCFTypeDictionaryValueCallBacks);
            NSLog(@"sending cancel sync notification...");
            CFNotificationCenterPostNotification(nCenter, CFSTR(SYNC_CANCELLED_NOTIFICATION), NULL, infoDict, TRUE);*/
            waitForRunningProcesses(SYNC_APPLICATION_NAME);
        }
        
        isnewVersionAvailable = up->start();
        
        if (isnewVersionAvailable) {
            //notify the menubar (and the prefPane) with the info that there is a new version to be downloaded
            NSDistributedNotificationCenter* nCenter = [NSDistributedNotificationCenter defaultCenter];
            NSMutableDictionary* infoDict = [NSMutableDictionary dictionaryWithCapacity: 1];
            [nCenter postNotificationName: (NSString*)CFSTR(UPDATE_SOFTWARE_NOTIFICATION) object:nil userInfo: infoDict deliverImmediately: YES];
        }
    
        if (!isnewVersionAvailable || alertRecommendedVal != 0 || alertVal !=0 
            || strcmp(upconfig->getVersion(),upconfig->getSkipped()) == 0) {
            exit(EXIT_SUCCESS); 
        } 
    } else {
        exit(EXIT_SUCCESS);
    }
}


- (int) showRecommendedAlert{
    [okReccomended setTitle:NSLocalizedString(@"yesText", @"-")];
    [okReccomended setKeyEquivalent:@"\r"];
    [cancelReccomended setTitle:NSLocalizedString(@"noText",@"-")];
    [alertRecommendedText setStringValue:NSLocalizedString(@"normalText", @"-")];
    [alertRecommendedText2 setStringValue:NSLocalizedString(@"reccomendedText", @"-")];
    [reccomendedImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
    NSString* title = [[NSString alloc] initWithFormat:NSLocalizedString(@"updgradeTitle", @"-"), APPNAME];
    [alertRecommended setTitle:title];
    
    [alertRecommended makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
    updater_session = [NSApp beginModalSessionForWindow:alertRecommended];
    for (;;) {
        if ([NSApp runModalSession:updater_session] != NSRunContinuesResponse)
            break;
    }
    
    if (alertRecommendedVal != 0) {
        [self showUpdateManuallyAlert];
    }
    return alertRecommendedVal;
}

- (int) showNormalAlert{   
    [okNormal setTitle:NSLocalizedString(@"yesText", @"-")];
    [okNormal setKeyEquivalent:@"\r"];
    [cancelNormal setTitle:NSLocalizedString(@"noText",@"-")];
    [skipNormal setTitle:NSLocalizedString(@"skipText",@"-")];
    [alertNormalText setStringValue:NSLocalizedString(@"normalText", @"-")];
    [normalImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
    NSString* title = [[NSString alloc] initWithFormat:NSLocalizedString(@"updgradeTitle", @"-"), APPNAME];
    [alert setTitle:title];
    
    [alert makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
    updater_session = [NSApp beginModalSessionForWindow:alert];
    for (;;) {
        if ([NSApp runModalSession:updater_session] != NSRunContinuesResponse)
            break;
    }
    if (alertVal != 0) {
        [self showUpdateManuallyAlert];
    }
    return alertVal;
    
}

- (void) downloadTask{
    StringBuffer tmpVal = upconfig->getUrlUpdate().c_str();
    if (tmpVal.empty()) {
        LOG.error("Update url empty");
        exit(EXIT_FAILURE);
    }
    
    downloadedZipFile = new StringBuffer(UPDATER_FOLDER_DESTINATION);
    downloadedZipFile->append( tmpVal.substr(((tmpVal.rfind("/")) +1 ), tmpVal.length()));
    
    NSString* val = [[NSString alloc] initWithFormat:NSLocalizedString(@"downloadingPercent" ,@"-"), 0];
    [progressStatusText setStringValue:val];
    
    NSString* urlupdate = [[NSString alloc] initWithCString:upconfig->getUrlUpdate().c_str()];


    NSURLRequest *theRequest=[NSURLRequest requestWithURL:[NSURL URLWithString:urlupdate]
                                              cachePolicy:NSURLRequestUseProtocolCachePolicy
                                          timeoutInterval:60.0];
    // create the connection with the request
    // and start loading the data
    NSURLDownload  *theDownload=[[NSURLDownload alloc] initWithRequest:theRequest
                                                              delegate:self];
    if (theDownload) {
        // set the destination file now
       // [theDownload setDestination:@UPDATER_PACKAGE_DESTINATION allowOverwrite:YES];
        NSString* dest = [[NSString alloc]initWithCString:downloadedZipFile->c_str()];
        [theDownload setDestination:dest allowOverwrite:YES];
    } else {
        // inform the user that the download could not be made
        
        [self showErrorWithString:NSLocalizedString(@"updateNetworkError",@"-")];
    }
}

- (void) showErrorWithString:(NSString*)text{
    
    [errorButton setTitle:NSLocalizedString(@"ok", @"-")];
    [errorText setStringValue:text];
    [errorImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
    NSString* title = [[NSString alloc] initWithFormat:NSLocalizedString(@"updgradeTitle", @"-"), APPNAME];
    [errorWindow setTitle:title];
    [errorWindow makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
}

- (void) unzipTask {
    [progressBar setDoubleValue:105];
    [progressStatusText setStringValue:NSLocalizedString(@"unzippingPackage",@"-")];
    ziptask = [[NSTask alloc]init];
    
    [[NSNotificationCenter defaultCenter] addObserver:self  
                                             selector:@selector(checkZipTaskStatus:)
                                                 name:NSTaskDidTerminateNotification
                                               object:ziptask];
    
    //NSString* arg = @UPDATER_PACKAGE_DESTINATION;
    NSString* arg = [[NSString alloc] initWithCString:downloadedZipFile->c_str()];
    NSString* arg1 = @"-o";
    NSString* arg2 = @"-d";
    NSString* arg3 = @UPDATER_FOLDER_DESTINATION;
    NSMutableArray* args = [NSMutableArray array];

    [args addObject:arg1];
    [args addObject:arg2];
    [args addObject:arg3];
    [args addObject:arg];
    NSString* path = [[NSString alloc] initWithFormat:@"%s", "/usr/bin/unzip"];
    [ziptask setLaunchPath:path];
    [ziptask setArguments:args];
    [ziptask launch];

    
}

- (void) installTask{
    [progressBar setDoubleValue:115];
    
    [progressStatusText setStringValue:NSLocalizedString(@"installingPackage",@"-")];

    uid_t vuid = getuid();
    
    passwd* p = getpwuid(vuid);
    
    
    OSStatus status;
    AuthorizationFlags authFlags = kAuthorizationFlagDefaults;              
    AuthorizationRef authorizationRef;                                  
    
    status = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, authFlags, &authorizationRef);
    
    if (status != errAuthorizationSuccess) {
        AuthorizationFree(authorizationRef, kAuthorizationFlagDefaults);
        exit(status);
    } else {
        AuthorizationItem authItems = {kAuthorizationRightExecute, 0, NULL, 0};
        AuthorizationRights authRights = {1, &authItems};        
        
        authFlags = kAuthorizationFlagDefaults | kAuthorizationFlagInteractionAllowed |
        kAuthorizationFlagPreAuthorize | kAuthorizationFlagExtendRights;
        
        status = AuthorizationCopyRights(authorizationRef, &authRights, NULL, authFlags, NULL );
        
        if (status == errAuthorizationSuccess) {
            const char progPath[] = UPDATER_UNZIP_PATH"/installer";
            const char *progArguments[7];
            
            progArguments[0] = "-u";
            progArguments[1] = p->pw_name; 
            progArguments[2] = "-U";
            progArguments[3] = "-c";
            progArguments[4] = UPDATER_UNZIP_PATH;
            progArguments[5] = "-C";
            progArguments[6] = NULL;
            
            // remove system directories with appropriate privileges
            status = AuthorizationExecuteWithPrivileges(authorizationRef, progPath, kAuthorizationFlagDefaults, (char **)progArguments, NULL);
        }
    }
    
    switch (status) {
        case errAuthorizationSuccess:
            break;
        case errAuthorizationCanceled:
            unlink(downloadedZipFile->c_str());
            removeDir(UPDATER_UNZIP_PATH);
            exit(EXIT_FAILURE);
            break;
        case errAuthorizationDenied:
            unlink(downloadedZipFile->c_str());
            removeDir(UPDATER_UNZIP_PATH);
            exit(EXIT_FAILURE);
            break;
        case errAuthorizationToolExecuteFailure:
            goto failed;
            break;
        case errAuthorizationToolEnvironmentError:
            goto failed;
            break;
        default:
            break;
    }
    
    
    unlink(downloadedZipFile->c_str());
    [progressBar setDoubleValue:120];
    [self storeConf];
    
    return;

failed:
    unlink(downloadedZipFile->c_str());
    removeDir(UPDATER_UNZIP_PATH);
    [self showErrorWithString:NSLocalizedString(@"installationFailed", @"-")];
}

- (int) showProgressWindow{
    [progressBar setIndeterminate:NO];
    [progressBar setDoubleValue:0.0];
    [okProgress setTitle:NSLocalizedString(@"ok", @"-")];
    [progressText setStringValue:NSLocalizedString(@"progressText", @"-")];
    [progressImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
    NSString* title = [[NSString alloc] initWithFormat:NSLocalizedString(@"updgradeTitle", @"-"), APPNAME];
    [progressWindow setTitle:title];
    
    [progressWindow makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
    
    
    [self downloadTask];
    
    return 1;
}

- (IBAction) okError:(id)sender{
    exit(EXIT_FAILURE);
}

- (void) storeConf{
    
    upconfig->createDefaultConfig();
    upconfig->setVersion(ClientConfig::getInstance()->getFunambolSwv());
    upconfig->save();
    NSString* val = [[NSString alloc] initWithFormat:NSLocalizedString(@"upgradeSucces",@"-"), APPNAME];
    [progressText setStringValue:val];
    [progressStatusText setStringValue:@""];
    [progressWindow makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];

    [okProgress setEnabled:YES];
    [okProgress setKeyEquivalent:@"\r"];
}


-(void)checkZipTaskStatus:(NSNotification *) aNotification 
{
    [ziptask release];
    [[NSNotificationCenter defaultCenter]  removeObserver: self];
    [progressBar setDoubleValue:110];
    [self installTask];
}

- (NSString*) getApplicationName {
    
    NSString* appName = [NSString stringWithUTF8String:APPNAME];
    return appName;
}

- (void)modalSheetDidEnd:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo{
    [sheet orderOut:self];
}

- (void)download:(NSURLDownload *)download didFailWithError:(NSError *)error
{
    // release the connection
    [download release];
    
    [self showErrorWithString:[error localizedDescription]];
    // inform the user
}

- (void)downloadDidFinish:(NSURLDownload *)download
{
    // release the connection
    
    [download release];
    [self unzipTask];
    
    // do something with the data
}

- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(unsigned)length
{
    //long long expectedLength=[[self downloadResponse] expectedContentLength];
    long long expectedLength=[downloadResponse expectedContentLength];
    
    bytesReceived=bytesReceived+length;
    
    if (expectedLength != NSURLResponseUnknownLength) {
        // if the expected content length is
        // available, display percent complete
        float percentComplete=(bytesReceived/(float)expectedLength)*100.0;
        [progressBar setDoubleValue:(double)percentComplete];
        NSString* val = [[NSString alloc] initWithFormat:NSLocalizedString(@"downloadingPercent" ,@"-"), (int)percentComplete];
        [progressStatusText setStringValue:val];
        [val release];
        NSLog(@"Percent complete - %f",percentComplete);
    } else {
        // if the expected content length is
        // unknown just log the progress
        NSLog(@"Bytes received - %d",bytesReceived);
    }
}

- (void)setDownloadResponse:(NSURLResponse *)aDownloadResponse
{
    [aDownloadResponse retain];
    [downloadResponse release];
    downloadResponse = aDownloadResponse;
}

- (void)download:(NSURLDownload *)download didReceiveResponse:(NSURLResponse *)response
{
    // reset the progress, this might be called multiple times
    bytesReceived=0;
    
    // retain the response to use later
    [self setDownloadResponse:response];
}

- (IBAction) okAlertNormal:(id)sender{
    
    [NSApp endModalSession:updater_session];
    [NSApp stopModal];
    [alert close];
    alertVal = 0;

}
- (IBAction) cancelAlertNormal:(id)sender{
    [NSApp endModalSession:updater_session];
    [NSApp stopModal];
    [alert close];
    alertVal = 1;

}
- (IBAction) laterAlertNormal:(id)sender{
    [NSApp endModalSession:updater_session];
    [NSApp stopModal];
    [alert close];
    
    alertVal = 2;

}

- (IBAction) okAlertR:(id)sender{
    [NSApp endModalSession:updater_session];
    [NSApp stopModal];
    [alertRecommended close];
    
    alertRecommendedVal = 0;

}
- (IBAction) cancelAlertR:(id)sender{
    [NSApp endModalSession:updater_session];
    [NSApp stopModal];
    [alertRecommended close];
    
    alertRecommendedVal = 1;
}

- (IBAction) okProgress:(id)sender{
    exit(EXIT_SUCCESS);
}


- (int) showUpdateManuallyAlert {    
    // You may choose to upgrade to the new version of the application at any time from the application's menu.
    [okUpdateManually setTitle:NSLocalizedString(@"ok", @"-")];
    [okUpdateManually setKeyEquivalent:@"\r"];
    [alertUpdateManuallyText setStringValue:NSLocalizedString(@"upgradeManually", @"-")];
    // [alertUpdateManuallyText setStringValue:@"You may choose to upgrade to the new version of the application at any time from the application's menu."];
    [updateManuallyImage setImage:[NSImage imageNamed:@"logoImage.tiff"]];
    NSString* title = [[NSString alloc] initWithFormat:NSLocalizedString(@"updgradeTitle", @"-"), APPNAME];
    [alertUpdateManually setTitle:title];
    
    [alertUpdateManually makeKeyAndOrderFront:self];
    [NSApp activateIgnoringOtherApps:YES];
    updater_session = [NSApp beginModalSessionForWindow:alertUpdateManually];
    for (;;) {
        if ([NSApp runModalSession:updater_session] != NSRunContinuesResponse)
            break;
    }
    
    return 0;
    
    
}

@end
